// Copyright 2003 Regents of the University of California

// Astropulse is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2, or (at your option) any later
// version.

// Astropulse is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.

// You should have received a copy of the GNU General Public License along
// with Astropulse; see the file COPYING.  If not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/* ap_client_main.C */
/* Main program for AstroPulse client */
#include "ap_config.h"
#ifdef _WIN32
#include "boinc_win.h"
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef _WIN32
#include <windows.h>
#define BOINC_APP_GRAPHICS 1
#endif

#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <iostream>

#include "boinc_api.h"
#include "util.h"
#include "str_util.h"

#include "astropulse.h"
#include "diagnostics.h"

#ifdef BOINC_APP_GRAPHICS
#include "ap_graphics.h"
#include "graphics_api.h"
// #include "sah_gfx.h"
#endif

#include <time.h>

long starttime = time(0);
int lastcputicks = clock();
float cputime = 0;
int maxcputicksinterval = 0;

// #define DEBUGGING_0 1 // If set to 1, print program status regularly (i.e. run in a verbose manner)

#ifdef _WIN32
#include <windows.h>
#include "stackwalker_win.h"

extern int main(int argc, char** argv);

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR Args, int WinMode) {
  LPSTR command_line;
  char* argv[100];
  int argc;

  //InitAllocCheck();

  command_line = GetCommandLine();
  argc = parse_command_line(command_line,argv);

  int ret = main(argc, argv);

  //DeInitAllocCheck();

  return ret;
}
#endif

/* Default thresholds */
static float thresh_def[] = {
                              (float)21.50,
                              (float)24.748650,
                              (float)30.015163,
                              (float)38.773587,
                              (float)53.662645,
                              (float)79.565603,
                              (float)125.71365,
                              (float)209.85380,
                              (float)366.44671,
                              (float)662.93874,
                              (float)1232.2705,
                              0, 0, 0, 0
                            };

#ifdef HAVE_GL_LIB
#ifdef UNIX

#include <GL/glut.h>

void displayCB(void)  /* function called whenever redisplay needed */
{
  glClear(GL_COLOR_BUFFER_BIT);  /* clear the display */
  glColor3f(1.0, 1.0, 1.0);  /* set current color to white */
  glBegin(GL_POLYGON);   /* draw filled triangle */
  glVertex2i(200,125);   /* specify each vertex of triangle */
  glVertex2i(100,375);
  glVertex2i(300,375);
  glEnd();    /* OpenGL draws the filled triangle */
  glFlush();    /* Complete any pending operations */
}

void keyCB(unsigned char key, int x, int y) /* called on key press */
{
  if ( key == 'q' ) exit(0);
}

#endif
#endif

/*
enum OnOff {on, off};
 
class timer {
  private:
  clock_t ticks;
  clock_t last_clock;
  clock_t start_clock;
 
  long seconds;
  long last_time;
  long start_seconds;
 
  OnOff state;
 
  public:
  timer() {
    ticks = 0;
    seconds = 0;
    start_clock = clock();
    start_seconds = time(NULL);
    state = off;
  }
  void start() {
    last_clock = clock();
    last_time = time(NULL);
    assert(state == off);
    state = on;
  }
  void stop() {
    clock_t ticks_this_time = clock() - last_clock;
    ticks += ticks_this_time;
    long seconds_this_time = time(NULL) - last_time;
    seconds += seconds_this_time;
    assert(state == on);
    state = off;
  }
  void reset() {
    ticks = 0;
    seconds = 0;
    start_clock = clock();
    start_seconds = time(NULL);
    state = off;
  }
  clock_t get_ticks() {
    return ticks;
  }
  clock_t get_total_ticks() {
    return clock() - start_clock;
  }
  long get_seconds() {
    return seconds;
  }
  long get_total_seconds() {
    return time(NULL) - start_seconds;
  }
};
 
void print_time() {
  long seconds = time(NULL) - starttime;
  long minutes = seconds / 60;
  int hours = minutes / 60;
  int days = hours / 24;
  printf("Time since start: %d days, %d hours, %ld minutes, %ld seconds\n",
   days, hours % 24, minutes % 60, seconds % 60);
}
*/

// This was a function used temporarily for debugging.
// It looks at a short time series, and determines whether
// the power stays above a certain threshhold for a certain length
// of time in consecutive bins.
// Not sure why I wanted to know this.
#if 0
int detect_hit(float *arr, float& total) {

  int stage = 0;
  total = 0;
  int count = 0;
  for (int i = 0; i < 10; i++) {
    if ( arr[i] < 1 && stage == 0) {
      return 0;  // not a hit; goes below 1 too soon.
    }
    if ( arr[i] > 5 && stage == 0) { // first time above 5
      stage = 1;
      count++;
    }
    if ( arr[i] > 5 && stage == 1) { // Above 5, but not first time
      count++;
    }
    if ( arr[i] < 5 && stage == 1) { // Going below 5
      if (count < 5) return 0; // goes below 5 too soon
      else stage = 2; // goes below 5 at the right time
    }
    total += arr[i];
  }

  if (count < 5) return 0;
  return 1;
}
#endif

/* updatecputime
 *
 * Purpose: determine the runtime of the program.
 * Sets the global variable cputime to equal the number of seconds the program has been running so far.
 * Sets the global variable maxcputicksinterval to the maximum interval in clock ticks so far between
 * calls to this function.
 */
void updatecputime() {
  int cputicksinterval = abs(clock() - lastcputicks);
  lastcputicks = clock();
  maxcputicksinterval = (maxcputicksinterval > cputicksinterval)?maxcputicksinterval:cputicksinterval;
  cputime += ((float)cputicksinterval / (float)CLOCKS_PER_SEC);
}

/* worker
 *
 * Function is not called unless graphics are turned on.
 */
void worker() {
  Astropulse::client.init();
  Astropulse::client.science.mainloop();
  Astropulse::client.finish();
  boinc_finish(0);
}

// Command-line arguments must be declared globally, because worker() takes no arguments
bool skipffa = false;
bool debug_msg = false;
bool write_lcgf = false;
bool print_best = false;
int single_dm_chunk = -1;
int data_byte_limit = -1;
int single_dm = -1;
int pulse_limit = 30;
int fold_dm = 0;

/* Main Program */
int main(int argc, char *argv[]) {

  Astropulse::main_timer.start();

  //char* foo = (char*) (void*) 0xbadf00d;
  //*foo = 'x';

  //fprintf(stderr, "START TIME: %ld\n", time(NULL));

  int i, retval;
  bool standalone = false;

  /* Possible arguments:
   * -standalone
   * -reset: reset ap_state.dat and pulse.out
   * -thresh0 %f: Set the power threshhold value for zero co-adds
   * -skipffa: Don't run the fast folding algorithm
   * -print_best: Print out the best pulses on a regular basis
   * -debug_msg: Print out debugging messages
   */
  for (i=1; i<argc; i++) {
    if (!strcmp(argv[i], "-standalone")) {
      standalone = true;
      fprintf(stderr, "## debug standalone mode\n");
    }
    if (!strcmp(argv[i], "-reset")) {
      fprintf(stderr, "## resetting ap_state.dat and pulse.out\n");
      unlink("ap_state.dat");
      unlink("pulse.out");
      continue;
    }
    if (standalone && !strcmp(argv[i], "-thresh0") && ++i < argc) {
      fprintf(stderr, "## thresh[0] = %f\n",
              (Astropulse::client.state.thresh[0] =(float)atoi(argv[i])) );
      continue;
    }
    if (!strcmp(argv[i], "-skipffa")) {
      skipffa = true;
      fprintf(stderr, "Skipping the ffa\n");
    }
    if (!strcmp(argv[i], "-debug_msg")) {
      debug_msg = true;
      fprintf(stderr, "Debugging messages on\n");
    }
    if (!strcmp(argv[i], "-write_lcgf")) {
      write_lcgf = true;
    }
    if (!strcmp(argv[i], "-print_best")) {
      print_best = true;
    }
    /* -dm_chunk (the command line argument) should be set to
     * dm_low + multiple of dm_chunk (the values defined in
     * the header of the workunit.)
     */
    if (!strncmp(argv[i], "-dm_chunk", 9)) {
      single_dm_chunk = atoi(argv[i] + 9);
      fprintf(stderr, "Running on a single dm chunk: %d\n", single_dm_chunk);
      /* -dm (the command line argument) should be set to a value
       * between 0 and dm_chunk (defined in the header of the workunit)
       */
    } else if (!strncmp(argv[i], "-dm", 3)) {
      single_dm = atoi(argv[i] + 3);
      fold_dm = single_dm;
      fprintf(stderr, "Running on a single dm: %d\n", single_dm);
    } else if (!strncmp(argv[i], "-pulse_limit", 12)) {
      // To get no pulse limit, set -pulse_limit0
      pulse_limit = atoi(argv[i] + 12);
    }
    /* -data_byte_limit should be set to a value between 0 and
     * datasize (which is 1/4 of nsamples, defined in the header of
     * the workunit.)  Actually the upper limit is datasize - (something small)
     * The smallest meaningful amount of data bytes is 4096, which is
     * (fft_len / 4) / 2, see below for discussion.
     */
    if (!strncmp(argv[i], "-data_byte_limit", 16)) {
      data_byte_limit = atoi(argv[i] + 16);
      fprintf(stderr, "Running on data chunks starting up to: %d\n", data_byte_limit);
    }
  }

  /* Initialize Diagnostics
   *
   * The diagnostics flags are defined in boinc/lib/diagnostics.h
   * There are many flags, and we are setting some of them.  E.g.
   * BOINC_DIAG_REDIRECTSTDERR redirects stderr to stderr.txt
   */
  unsigned long dwDiagnosticsFlags = 0;

  dwDiagnosticsFlags =
    BOINC_DIAG_DUMPCALLSTACKENABLED |
    BOINC_DIAG_HEAPCHECKENABLED |
    BOINC_DIAG_TRACETOSTDERR |
    BOINC_DIAG_REDIRECTSTDERR;

  /* boinc_init_diagnostics is a function in the boinc API.
   */
  retval = boinc_init_diagnostics(dwDiagnosticsFlags);
  if (retval) {
    fprintf(stderr, "boinc_init_diagnostics failed: %d\n", retval);
    exit(retval);
  }

#ifndef BOINC_APP_GRAPHICS
  /* The function below is defined in boinc/api/boinc_api.C */
  retval = boinc_init();
  if (retval) {
    fprintf(stderr, "boinc_init failed: %d (try -standalone for testing)\n", retval);
    exit(retval);
  }
#else
  //app_graphics_init();
  ap_graphics.data_struct_init();
  /* Boinc API: initialize boinc with graphics.  worker() is a function
  * that performs the role of main(), whereas main() should run
  * the graphics.  It's not clear to me that this is in fact what happens:
  * it looks to me like both main and worker are playing the same role here.
  */
  retval = boinc_init_graphics(worker);
  if (retval) {
    fprintf(stderr, "boinc_init_graphics failed: %d\n", retval);
    exit(retval);
  }
#endif

  /* The 3 functions below are all defined in this file */
  if (debug_msg) printf("In ap_client_main.cpp: at Astropulse::client.init()\n");
  Astropulse::client.init();
  if (debug_msg) {
    printf("In ap_client_main.cpp: at Astropulse::client.science.mainloop()\n");
    fflush(stdout);
  }
  Astropulse::client.science.mainloop();
  if (debug_msg) {
    printf("In ap_client_main.cpp: at Astropulse::client.finish()\n");
    fflush(stdout);
  }
  Astropulse::client.finish();

  //fprintf(stderr, "END TIME: %ld\n", time(NULL));

  Astropulse::main_timer.stop();

  /* The function below is defined in boinc/api/boinc_api.C
   * The argument 0 means we've successfully finished the result.
   */
  boinc_finish(0);
  return 0;
}

double interpolate_dec(double jd0, double jd1, double pulse_jd, double dec0, double dec1);
double interpolate_ra(double jd0, double jd1, double pulse_jd, double ra0, double ra1);

void segfault_atexit() {
  assert(0);
}

/* variables and functions defined here may be called via
 * Astropulse::<variable> or Astropulse::<function>
 */
namespace Astropulse {
/* The class below is defined in astropulse.h
 * Each Client contains an Outfile, Wufile, Foldfile, Statefile, and Science.
 */
Client client;
timer main_timer;

/* calculate_fold_level
 *
 * Argument: int dm_chunk 
 * Return value: The highest power of 2 that is less than or equal to dm_chunk.
 * So calculate_fold_level(32) == 5 and calculate_fold_level(63) == 5.
 */
inline int calculate_fold_level(int dm_chunk) {
  /* Figure out sizes */
  int level = 0;
  for (;; ++level) {
    int temp = dm_chunk >> level;
    if (temp == 0) {
      fprintf(stderr, "Error calculating fold_level.\n");
      exit(PARSE_ERR);
    }
    if (temp == 1) {
      return level;
    }
  }
}

/* Calculates the highest power of two that is a factor
 * of the input.  For instance,
 * input 4 => output 2
 * input 8 => output 3
 * input 12 => output 2
 * UNUSED FUNCTION
inline int calculate_power_of_two_factor(int dm) {
  int retval = 0;
  assert(dm != 0);
  while(dm % 2 == 0) {
    retval++;
    dm /= 2;
  }
  return retval;  
}
*/

state_t::state_t() {
  /* Default parameters for a state_t.
   * This constructor function is mostly useless.
   * In our program, the only state_t is the global variable client.state.
   * In Client::init, some of the members of client.state are reset by
   * Wufile::parse_header_to_state, which alters the
   * global variable by reading
   * from in.dat.  Other members are set in the body of Client::init
   * 
   * Variables that do not appear to get reset include:
   * best[].peak_power, best[].period
   * data_chunk_now, frac_done, and result_count
   */
  datasize  = 8*1024*1024;
  dm_low    = 1;                                      // ~ 0 pc/cm^3
  dm_hi     = 15000;                                  // ~ 100 pc/cm^3
  // NOTE: I think the correct conversion is
  // dm = DM * 18.11,
  // so dm_hi = 15000 corresponds to DM = 828 pc / cm^2
  dm_chunk  = 32;                           // DM resolution for folding
  fft_len   = 32768;                 // Needs to be at least twice DM_HI
  max_coadd = 10;
  for (int l = 0; l < max_coadd; ++l) {
    thresh[l] = thresh_def[l];

    // NOTE: none of these initial values except peak_power
    // should ever be used
    best[l].peak_power = -INFINITY;
    best[l].period     = -1.0;
  }

  /* Starting place within computation */
  data_chunk_now = 0;
  dm_chunk_now   = dm_low;
  dm_now         = 0;
  dm_sign        = 1;
  sub_buffer    = 0;
  freq           = 0;
  min_freq		= 137;
  fold_buf_loc   = 0;
  frac_done      = 0.0;
  result_count   = 0;
}

/* Client::init
 * 
 * This function initializes the Client by performing a large number of
 * unrelated actions.  See below for descriptions of these actions.
 */
void Client::init() {
  /* Read the statefile, to see if we have been working on this computation before.
   * Initialize the wufile (in.dat)
   * Initialize the outfile (pulse.out)
   */
  if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), at statefile.read()\n");
  statefile.Read();
  if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), at wufile.init()\n");
  wufile.init();
  // We have to parse the header from in.dat
  // to client.wuheader each time, because
  // it is not part of the state, hence it does not get saved.
  wufile.parse_header_for_writeout();
  if (!statefile.resumed) {
    /* if no state file exists, then we have to start from scratch.
     * Parse the data header from client.wuheader into client.state, 
     * then set the starting dm chunk
     */
    wufile.parse_header_to_state();
    state.dm_chunk_now = state.dm_low;
  } else {  // state file exists, so there might be a result file
    if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), reading signal_vector\n");
    read_signal_vector(signal_vector);
  }

  // if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), at outfile.init()\n");
  // outfile.init();  Not initing this here anymore

  /* Figure out the fold level, which is the exponent in the power of 2 that
   * describes the number of consecutive samples to sum, for the purpose of
   * the folding algorithm.  (Which detects repeating pulses.)  The folding
   * algorithm works on groups of samples, not individual samples, because
   * there are too many samples to do otherwise.
   * The fold level is the same as the exponent in the power of 2 that
   * describes the dm_chunk size.  Why?
   */
  state.fold_level = calculate_fold_level(state.dm_chunk);

  /* Figure out the fold_buf_size, which is the number of complex samples in the
   * entire wufile, divided by the fold level's power of 2.  This is enough to
   * describe the entire workunit, because the folding algorithm combines samples 
   * into groups of 2^state.fold_level, this is
   */
  state.fold_buf_size = (state.datasize*4)>>state.fold_level;

  /* In Client::init.  Initialize variables, fft plans, etc. */

  if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), at science.init()\n");
  science.init();

  /* Power array needs to hold half of the fft_len time bins that will result from the
   * dechirping process.  I'm not sure why only half.  ?????
   */
  power.resize(state.fft_len/2);

  /* See above for the definition of state.fold_buf_size */
  fold_buf.resize(state.fold_buf_size);

  /* In Client::init.  Read/write files */

  if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), at foldfile.read()\n");
  if (foldfile.Read()) {
    /* Couldn't read fold file => reset to beginning of dm chunk */
    state.fold_buf_loc = 0;
    state.dm_now       = 0;
    state.dm_sign      = 1;
  }

  if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), at wufile.read_raw_data()\n");
  wufile.read_raw_data();
  if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), at wufile.finish()\n");
  wufile.finish();
  if (debug_msg) printf("in ap_client_main.cpp, in Client::init(), at statefile.write()\n");
  statefile.write();
}

/* time_series_b_t::build
 *
 * Arguments:
 * array: The power array, containing a power level for each of fft_len/2 time samples
 *        If we are dealing with the FFA, we instead have a power level for each folded & coadded sample.
 * index: m, the index of a power bin which is above threshhold.
 * range: fft_len/2, or smaller (fft_len/(2 >> ell)) if there is a coadd ell > 0.
 * n_bins: the number of bins that we add up to get each element in the power array.
 *   Since the average noise level is 1 per bin, n_bins serves as a baseline or typical value
 *   for the total power.
 *
 * This function builds a time series of length at most MAXLENGTH, which is defined in astropulse.h.
 * A time series consists of a series of power values from the array, but generally 
 * not the whole array.  The power values are normalized so that array[index] has 
 * value 255, and all power values can be stored as unsigned chars.
 * 
 * struct time_series_b_t is defined in astropulse.h
 */
void time_series_b_t::build(float array[], int index, int range, float n_bins) {
  int temp_data_val;
  /* Set peak to equal the power that was above threshhold */
  float peak = array[index];
  /* Remove all elements from the data vector, a member of time_series_b_t */
  data.clear();
  if (range >= MAXLENGTH) {
    /* Create a time series of length MAXLENGTH, centered at index.
     * In some cases, this may be impossible, i.e. if the index is very
     * close to the beginning or end of the array.  In this case, we
     * do the best we can.
     */
    data.resize(MAXLENGTH);
    index -= MAXLENGTH/2;
    if (index < 0) {
      index = 0;
    } else if (index + MAXLENGTH > range) {
      index = range - MAXLENGTH;
    }
    for (int j = 0; j < MAXLENGTH; ++index, ++j) {
      /* normalize so the peak has height 255, and n_bins has height 63. */
      temp_data_val = (int) (63 + 192 * (array[index] - n_bins)/(peak - n_bins));
      if (temp_data_val < 0) temp_data_val = 0;
      if (temp_data_val > 255) temp_data_val = 255;
      data[j] = (unsigned char) temp_data_val;
    }
    length = MAXLENGTH;
  } else {
    /* Create a time series out of all the data */
    data.resize(range);
    for (index = 0; index < range; ++index) {
      /* normalize so that the peak has height 255.  or 256? */
      temp_data_val = (int) (63 + 192 * (array[index] - n_bins)/(peak - n_bins));
      if (temp_data_val < 0) temp_data_val = 0;
      if (temp_data_val > 255) temp_data_val = 255;
      data[index] = (unsigned char) temp_data_val;
    }
    length = range;
  }
}

/* time_series_f_t::build
 * 
 * This function serves the same purpose as time_series_b_t::build, but 
 * it creates an array of floats instead of an array of chars.  Therefore, 
 * the power values do not need to be normalized.
 */
void time_series_f_t::build(float array[], int index, int range) {
  /* Remove all elements from the data vector.  No need to store a float called
   * <peak> in this function, because the <array> already consists of floats.
   */
  data.clear();
  data.resize(MAXLENGTH);
  if (range >= MAXLENGTH) {
    /* Create a time series out of part of the data */
    index -= MAXLENGTH/2;
    if (index < 0) {
      index = 0;
    } else if (index + MAXLENGTH > range) {
      index = range - MAXLENGTH;
    }
    data.assign(array+index, array+index+MAXLENGTH);
    length = data.size();
  } else {
    /* Create a time series out of all the data */
    data.assign(array, array+range);
    length = data.size();
  }
}

void state_t::compute_fraction_done(double ell) {
    int dm_chunk_log = int_log2(dm_chunk);

    if(data_chunk_now >= datasize - (fft_len/2)/4) {   // In ffa
      assert(ell == -1 || ell == -2);
	  if(ell == -1) {  // In main ffa
        frac_done = log(freq / min_freq) / log((double)2);
        frac_done += (float)sub_buffer / (float)num_sub_buffers;
        frac_done /= num_sub_buffers;
        frac_done *= frac_in_main_ffa[dm_chunk_log];
        frac_done += (1 - frac_in_main_ffa[dm_chunk_log]);
        frac_done += (dm_chunk_now - dm_low) / dm_chunk;
        frac_done /= ((dm_hi - dm_low + dm_chunk - 1)/dm_chunk);
	  } else { // ell == -2, we are in build_threshold_table
	    frac_done = (float)nfb / (float)fold_buf_size;
		frac_done *= frac_in_btt[dm_chunk_log];
		frac_done += (1 - frac_in_btt[dm_chunk_log] - frac_in_main_ffa[dm_chunk_log]);
		frac_done += (dm_chunk_now - dm_low) / dm_chunk;
		frac_done /= ((dm_hi - dm_low + dm_chunk -1)/dm_chunk);
	  }
	} else   {   // In single pulse
      assert(ell >= 0);  // Not in ffa
      frac_done = ell / (double)max_coadd;
      frac_done += (1-dm_sign) / 2;
      frac_done /= 2;
      frac_done += dm_now;
      frac_done /= dm_chunk;
      frac_done += data_chunk_now/((fft_len/2)/4);
      frac_done /= (datasize/((fft_len/2)/4))-1;
      frac_done *= (1 - frac_in_btt[dm_chunk_log] - frac_in_main_ffa[dm_chunk_log]);
      frac_done += (dm_chunk_now-dm_low)/dm_chunk;
      frac_done /= ((dm_hi - dm_low + dm_chunk - 1)/dm_chunk);
    }
    /* This is the old way of doing it.   I think the dm_sign has issues.
    frac_done = ell/(double)max_coadd;
    frac_done += (1-dm_sign);
    frac_done /= (2+1);
    frac_done += dm_now;
    frac_done /= dm_chunk;
    frac_done += data_chunk_now/((fft_len/2)/4);
    frac_done /= (datasize/((fft_len/2)/4))-1;
    frac_done += (dm_chunk_now-dm_low)/dm_chunk;
    frac_done /= 1+((dm_hi - dm_low)/dm_chunk);
    */
}


void Science::mainloop() {
  // Uncomment the following line (atexit) if the program is exiting
  // without saying why.  It will segfault when the program exits.
  // atexit(segfault_atexit);

  // FILE* hit_file;
  // hit_file = fopen("hit_file.txt", "w");
  // FILE* power_file = fopen("power_file.txt", "w");  // temp for debugging
  float total; // temp for debugging
  int last_hit; // temp for debugging
  int count = 0;  // temp for debugging
  state_t& state = client.state;
  /* It's not clear why the line below is necessary.  It's basically setting a pointer
   * called power, to equal the address of the first element in a std::vector<float>
   * called client.power.  Why can't we just use the vector instead of the pointer?
   */
  float*   power = &(client.power[0]);
  if (debug_msg) {
    printf("state.dm_low: %d\n", state.dm_low);
  }

  FILE *best_file;
  if (print_best) {
    best_file = fopen("best_file.txt", "w");
  }

  timer checkpoint_timer;
  timer ffa_timer;

  /* mainloop level 1:
   * This loop iterates over dm chunks.  Each dm_chunk consists of, say, 32 dms.
   * The number 32 (or whatever it is) comes from state.dm_chunk, which
   * is defined in in.dat
   */
  for (;state.dm_chunk_now < state.dm_hi;
       state.dm_chunk_now += state.dm_chunk) {
    if (single_dm_chunk != -1) {
      if (state.dm_chunk_now != single_dm_chunk) continue;
    }
    client.checkpoint();

    /* Print best pulses so far */
    if (print_best) {
      for (int scale = 0; scale < state.max_coadd; scale++)
        fprintf(best_file, "scale %d, peak_power %f\n", scale, state.best[scale].peak_power);
      fprintf(best_file, "\n");
      fflush(best_file);
    }

    /* Create an array of chirps, containing state.dm_chunk different chirps,
    * with dms ranging from state.dm_chunk_now to state.dm_chunk_now + 
    * state.dm_chunk - 1
    */
    build_chirp_table(state.dm_chunk_now);
    /* mainloop level 2
            * This loop iterates over all data chunks.  Each data chunk consists of fft_len
            * complex samples, or fft_len / 4 bytes.  The number state.data_chunk_now follows the
     * pattern: 0, 4096, 4096 * 2, 4096 * 3, ...
     *
     * However, there is an overlap of fft_len / 2 complex samples, or fft_len / 8
     * bytes.  That is, each time we iterate to the next data chunk, we only move
     * ahead by fft_len / 8 bytes of data.
     *
     * This means that the loop runs state.datasize / (state.fft_len / 8)
     * times per data chunk.
            */
    for (; state.data_chunk_now<(state.datasize-state.fft_len/2/4); state.data_chunk_now+=(state.fft_len/2)/4) {
      if (data_byte_limit != -1) {
        if (state.data_chunk_now >= data_byte_limit) continue; // Only consider the first few data chunks, up to the one that starts with data_byte_limit bytes.
      }
      /* Print debugging information about the current dm_chunk and data_chunk
              * We do this on data chunks 4096 * {0, 1, 2, 4, 8, ...} for the first dm of each dm_chunk
       */
      if (debug_msg) {
        int i = state.data_chunk_now / ((state.fft_len/2)/4);
        if (i==0) i=1; // Handle the edge case that i==0
        while (i % 2 == 0 && i > 1) {
          i /= 2;
        }
        if (i == 1) {
          printf("In ap_client_main.cpp, Science::mainloop\n");
          printf("state.dm_chunk_now: %d, state.data_chunk_now: %d\n", state.dm_chunk_now,
                 state.data_chunk_now);
          main_timer.print_total_time();
          fflush(stdout);
        }
      }
      client.checkpoint();
      /* convert_bits_to_float calls splitter_bits_to_float,
      * which is defined in sbtf.cpp.  This function converts each bit into
       * a float, resulting in a large array of floats.  The number of complex
      * samples converted is equal to the length of one FFT. 
      */
      convert_bits_to_float();
      client.checkpoint();
      /* FFT the time-domain data, stored in the float array client.science.data,
      * into frequency-domain data 
      */
      compute_forward_fft();
      /* Filter the frequency-domain data to get rid of low frequency "noise".
       * This line should be removed if at a later time we figure out how to get rid of the noise.
       */
      high_pass_filter();

      /* Graphics stuff */
#ifdef BOINC_APP_GRAPHICS
      double max = 0.0;
      GRAPH_BUFFER* abp;
      abp = ap_graphics.get_generate_buffer(true);
      if (abp) abp->rarray.init_data(state.fft_len/2, state.dm_chunk); // const
#endif

      /* mainloop level 3
        * This loop iterates over dms within a dm chunk.
       */
      for (; state.dm_now<state.dm_chunk; state.dm_now++) {
        if (single_dm != -1) {
          if (state.dm_now != single_dm) continue;
        }
        /* Checks if app client is in standalone mode, for testing
         * purposes.  (As opposed to being run by the boinc
         * client.)  If so, force checkpoint if over 60 seconds.
         */
        if (boinc_is_standalone()) {
          if(checkpoint_timer.get_total_seconds() > 60) {
            checkpoint_timer.reset();
            client.do_checkpoint();
          }
        }
        /* mainloop level 4
         * This loop iterates over the two dm signs, positive and negative
         */
        for (; state.dm_sign>=-1; state.dm_sign-=2) {
          client.checkpoint();

          /* Dechirp the frequency domain data via multiplication by a dechirping
           * function.  Then perform the inverse fourier transform
           */
          dechirp(state.dm_now, power, state.dm_sign);

          /* Graphics stuff */
#ifdef BOINC_APP_GRAPHICS
          if (abp && state.dm_sign > 0) { // add positive dispersions to graph
            abp->rarray.add_source_row(power);
          }
#endif

          /* measuring the time that has passed since the program started running */
          updatecputime();

          /* mainloop level 5 * This loop iterates over coadd levels.
           */
	  for (int l=0; l<state.max_coadd; l++) {

            /* Uses ell, along with some members of client.state
             * Stores the result in state.frac_done.
             * The sole purpose of this computation is to print the fraction
             * done on the screen.
             */
            state.compute_fraction_done(l);
            /* boinc API boinc_fraction_done tells BOINC how much
             * of the current workunit has been completed.  This is
             * used to inform the core client GUI of the % complete.
             */
            boinc_fraction_done(state.frac_done);

            /* This is the size of the power array.
                           * It starts at state.fft_len / 2, for ell = 0.  As ell gets larger, 
             * we perform coadds in place, and this reduces the effective 
             * size of the power array.
                    * Note that from the very beginning, we ignore the second
             * half of the power bins.  Due to the overlap in ffts,
             * this second half is not necessary.
              */
            int dechirped_range_data_length = state.fft_len >> (l+1);
            /* mainloop level 6
            * This loop iterates over all of the power bins, searching for
            * a bin that exceeds the threshhold power.
            */
            last_hit = -10;
            for (int m=0; m<dechirped_range_data_length; m++) {

              // fprintf(power_file, "m = %d, power[m] = %f, state.data_chunk_now = %d\n", m, power[m], state.data_chunk_now);  // temp for debugging
              if (state.dm_sign == 1 && l == 0 && m < dechirped_range_data_length - 10) {
                /* This was for debugging.  Not sure for what exactly.
                if(m > last_hit + 10 && detect_hit(&power[m], total)) {
                  count++;
                  fprintf(hit_file, "m = %d, sample = %d, total = %f, num = %d\n", m, m + state.data_chunk_now * 4, total, count);
                  last_hit = m;
                } */
              }
              /* Graphics stuff */
#if BOINC_APP_GRAPHICS
              if (power[m] > max) {
                ap_graphics.time_series.build(power, m, dechirped_range_data_length);
                max = power[m];
              }
#endif

              /* If power exceeds threshhold, record a pulse.
               * This involves first creating an ap_signal, which is defined in
               * astropulse/server/db/ap_schema.h.  Then, we write the signal
               * to pulse.out using Outfile::output_result.
               */
              if (power[m] > state.thresh[l]) {
                ap_signal s;
                time_series_b_t time_series;
                time_series.build(power, m, dechirped_range_data_length, pow(2.0, l));
                state.pulse_jd = state.jd0 + (state.data_chunk_now + state.dm_now)/(state.datasize)*(state.jd1-state.jd0);

                s.dm= state.dm_sign*(state.dm_chunk_now+state.dm_now); //dm
                s.scale=l;                   // scale
                s.peak_power=power[m];       // peak_power
                s.fft_num=state.data_chunk_now*4;
                s.peak_bin=state.data_chunk_now * 4 + (m << l); // index
                s.time=state.pulse_jd;       // time
                s.period=0.0;                // period
                // Interpolate betw. ra0 & ra1 using state.pulse_jd
                s.ra = interpolate_ra(state.jd0, state.jd1, state.pulse_jd, state.ra0, state.ra1);
                // Interpolate betw. dec0 and dec1
                s.decl = interpolate_dec(state.jd0, state.jd1, state.pulse_jd, state.dec0, state.dec1);
                s.time_series_len=time_series.length;
                s.time_series=time_series.data;
                s.time_series.encoding=_base64;
		if(add_signal(client.signal_vector, s)) state.result_count++;
                // client.outfile.output_result(s);
                
                if(pulse_limit && state.result_count >= pulse_limit) {
                  // Quit early
                  fprintf(stderr, "Found %d pulses, exiting.\n", pulse_limit);
		  printf("Found %d pulses, exiting.\n", pulse_limit);
		  fflush(stdout);
		  client.outfile.write_main(client.signal_vector);
                  client.outfile.finish_results();
                  client.finish();
                  main_timer.stop();
                  boinc_finish(0);
		  exit(0);
                }
              }

              /* We record the best peak at each coadd level.  That is, there is
               * one "best" value for each coadd level, for the entire run of
               * the client.
               */
              if (power[m] > state.best[l].peak_power) {
                state.pulse_jd = state.jd0 + (state.data_chunk_now + state.dm_now/state.dm_chunk)/(state.dm_hi)*(state.jd1-state.jd0);
                state.best[l].fft_num=state.data_chunk_now*4;
                state.best[l].peak_bin = state.data_chunk_now * 4 + (m << l);
                state.best[l].peak_power = power[m];
                state.best[l].scale = l;
                state.best[l].dm = state.dm_sign*(state.dm_chunk_now+state.dm_now);
                state.best[l].period = 0.0;
                state.best[l].time = state.pulse_jd;
              }
            } /* End mainloop level 6, bin number (m) */

            /* When l is at a certain value (called state.fold_level), we are ready
             * to contribute data to the fast folding algorithm.  This happens only once per
             * (dm_chunk, data_chunk) pair.  So we only hit one out of every 32
             * dms in this way.  This is why we do it at a fold_level of 5
             * (2^5 = 32) with the dm chunk size being 32 as well.
             */
            // Temporary fix
            if ((l==state.fold_level) && (state.dm_now==fold_dm) && (state.dm_sign==1)) {
              /* Save for folding */
              // need to add neg DM folding also.
              memcpy(&client.fold_buf[state.fold_buf_loc], power,
                     sizeof(float)*(state.fft_len>>(l+1)));
              state.fold_buf_loc += (state.fft_len>>(l+1));
            }

            /* coadd_in_place is defined in ap_science.cpp
             * Its purpose is to halve the size of the power array by adding
             * array elements in consecutive pairs.
             * The second argument is the initial array size.  Note that
             * 
             */
            coadd_in_place(power, state.fft_len>>(l+1));
          } /* End mainloop level 5, coadd level (l) */
          client.checkpoint();
        } /* End mainloop level 4, dm sign (s) */
        state.dm_sign = 1;
      } /* End mainloop level 3, inner dm (k) */
      state.dm_now = 0;

      /* Graphics stuff */
#ifdef BOINC_APP_GRAPHICS
      if (abp) {
        ap_graphics.generate_done(abp);
      }
#endif

    } /* End mainloop level 2, data (j) */

    /* When we've finished the data loop, we will have hopefully
     * filled up the fold buffer, by successively copying the power array
     * into it.  We are then ready to reset the fold buffer to the beginning
     * of the buffer, and reset the data chunk to the first data chunk.
     */
    if (state.fold_buf_loc != state.fold_buf_size) {
      /*fprintf(stderr, "Fold buffer didn't fill up (lol=%d, size=%d).\n",
        state.fold_buf_loc, state.fold_buf_size);*/
    }

    /* do fold - results output within ffa */
    // fclose(hit_file);  // temp for debugging
    ffa_timer.start();
    if(!skipffa) {
      ffa(&client.fold_buf[0], 1, write_lcgf, debug_msg, pulse_limit);
    }
    ffa_timer.stop();

    state.fold_buf_loc = 0;
    state.data_chunk_now = 0;

  } /* End mainloop level 1, DM chunk (i) */

  /* output the top result no matter how many we have >= threshold, so
   * that if none >= threshold we can still validate redundant results
   * against each other.
   */
  /* Was used for debugging before the ap_timer class was invented
  FILE* progress_file = fopen("progress_file.txt", "a");
  fprintf(progress_file, "Total time: %d\n", time(0) - starttime);
  fprintf(progress_file, "Total cpu time: %f\n", cputime);
  fprintf(progress_file, "Max cpu ticks interval: %d\n", maxcputicksinterval);
  fclose(progress_file);
  */

  /* Write the best pulses, and various other xml tags. */
  client.outfile.write_main(client.signal_vector);
  client.outfile.finish_results();
  if (debug_msg)
    printf("FFA timer reads: %ld seconds, %ld ticks\n", ffa_timer.get_seconds(), ffa_timer.get_ticks());
} /* End mainloop */

/* Client::finish
 *
 * Calls all finish functions.
 */
void Client::finish() {
  // Clean up stuff and exit
  // unloadGl();
  science.finish();
  // outfile.finish();
  statefile.finish();
  foldfile.finish();
}

}

double interpolate_ra(double jd0, double jd1, double pulse_jd, double ra0, double ra1) {
  assert(jd1 > jd0);
  double time_fraction = (pulse_jd - jd0) / (jd1 - jd0);
  double retval;
  if (fabs(ra0 - ra1) < 12) {
    return ra0 + (ra1 - ra0) * time_fraction;
  } else {  // We've crossed RA 0 -- RA 24.
    if (ra0 > ra1) // crossed from high RA to low RA
      retval = (ra0 - 24) +
               (ra1 - (ra0 - 24)) * time_fraction;
    else // crossed from low RA to high RA
      retval = ra0 +
               ((ra1 - 24) - ra0) * time_fraction;
    if (retval > 0) return retval;
    else return retval + 24;
  }
}

double interpolate_dec(double jd0, double jd1, double pulse_jd, double dec0, double dec1) {
  assert(jd1 > jd0);
  double time_fraction = (pulse_jd - jd0) / (jd1 - jd0);
  return (dec0 + (dec1 - dec0) * time_fraction);
}
