#ifndef INCagcMsgh
#define INCagcMsgh
/*
* include file for messages sent from pntProg to the  agcProg 
*history..
*xx.x,01apr96,pjp  started
*xx.x,02apr96,pjp  move ttProg.h into this file
*                 msgs for debugging.
*xx.x,13apr96,pjp switch to doubles...
*/
/*****************************************************************************
*  azimuth,gregorian,carriage related messages
*/
/*
 *  identify a particular mode for the agcProg
*/
#define PNTAGC_MODE_STOP        0
#define PNTAGC_MODE_RATE        1
#define PNTAGC_MODE_POS         2
#define PNTAGC_MODE_STOW        3
#define PNTAGC_MODE_PROGTRACK   4 
#define PNTAGC_MODE_AUX         5 
/*
 *  identify the 3 axis to control
*/
#define PNTAGC_AXIS_AZ      	1
#define PNTAGC_AXIS_GR      	2
#define PNTAGC_AXIS_CH      	4

#define PNTAGC_INDEX_AZ     	0
#define PNTAGC_INDEX_GR     	1
#define PNTAGC_INDEX_CH     	2

#define PNTAGC_NUM_AXIS     	3
/*
 *   for program track cmd, flag bits
*/
#define PNTAGC_PTFLAG_CLRSTACK  1
#define PNTAGC_PTFLAG_ENTPTMODE 2
/*
 * for xfer control..
*/
#define PNTAGC_TC_LCU           1
#define PNTAGC_TC_OCU           2
#define PNTAGC_TC_PCU           3
#define PNTAGC_TC_VME           4
/*
 * for monitor request
*/
#define PNTAGC_MONREQ_CBLK      1
#define PNTAGC_MONREQ_FBLK      2
#define PNTAGC_MONREQ_OPARM     3
#define PNTAGC_MONREQ_POS       4
/*
 * define who is master
*/
#define PNTAGC_MASTER_GR_COMP_CH   1 
#define PNTAGC_MASTER_CH_COMP_GR   2
#define PNTAGC_MASTER_GR_SUPPLY_CH 3 
#define PNTAGC_MASTER_CH_SUPPLY_GR 4 
/*
 * reset faults. use axis 1,2,4 and 8 for general reset
*/
#define PNTAGC_RESET_GENERAL       8 


/*
 * for operating parameters request there is a flag
 * that includes differnt options
 * 1,2,4, are the axis.. for az,greg,ch encoder offset
 * 8 is program track time offset
*/
#define PNTAGC_OPARM_FLAG_TIMEOFFSET 8
/*
 *  defines for bits in status word
*/
#define PNTAGC_ST_RUNNING           0x00000001 
#define PNTAGC_ST_CONNECTED         0x00000002 
#define PNTAGC_ST_LAST_IO_OK        0x00000004 
#define PNTAGC_ST_WAITTICK_ERR      0x00000008 

#define PNTAGC_ST_MSK_PT            0x00000070
#define PNTAGC_ST_AZ_PT             0x00000010
#define PNTAGC_ST_GR_PT             0x00000020
#define PNTAGC_ST_CH_PT             0x00000040


#define PNTAGC_ST_MSK_VMECTRL       0x00000700
#define PNTAGC_ST_AZ_VMECTRL        0x00000100
#define PNTAGC_ST_GR_VMECTRL        0x00000200
#define PNTAGC_ST_CH_VMECTRL        0x00000400

#define PNTAGC_ST_MSK_FAULT         0x0000f000
#define PNTAGC_ST_GEN_FAULT         0x00001000
#define PNTAGC_ST_AZ_FAULT          0x00002000
#define PNTAGC_ST_GR_FAULT          0x00004000
#define PNTAGC_ST_CH_FAULT          0x00008000

#define PNTAGC_ST_MSK_AUXMOT        0x00070000
#define PNTAGC_ST_AZ_AUXMOT         0x00010000
#define PNTAGC_ST_GR_AUXMOT         0x00020000
#define PNTAGC_ST_CH_AUXMOT         0x00040000

#define PNTAGC_ST_GR_MASTER         0x00100000
#define PNTAGC_ST_MSK_PREPROC       0x00e00000 
#define PNTAGC_ST_SHIFT_PREPROC     21
#define PNTAGC_ST_PREPROC_AZ        0x00200000 
#define PNTAGC_ST_PREPROC_GR        0x00400000 
#define PNTAGC_ST_PREPROC_CH        0x00800000
/*
 * typedefs
*/
/*
 *  message structures used for sendin msgs from pnt to agc program.
*/
typedef struct {
    	double  secs;   /* from midnite for this data point*/
    	double  posRd[3];   /* az,gr,ch (radians)*/
	} PNTAGC_PT_DATA;

/* 
 * mode msg passes as pointer to malloced buf
*/
typedef struct {
    	int     mode;   /* 0-5*/
    	int     axis;   /* for those that use it...1,2,4*/
    	union     {
            double   posVel[3];/*for pos or rate rd, rd/sec*/
            int      aux[3];   /* motor/group*/ 
    	} u;
    } PNTAGC_MSG_MODE;
/* 
 * set operating params msg
*/
typedef struct {
    	int     flag;   /* 1 azEnc,2 grEnc,4 chEnc,8ptOffset.. or together*/
    	double  encCorRd[3];/* encoder correction az,gr,ch*/
    	double  timeOffsetPT;/* time offset program track. secs*/
    } PNTAGC_MSG_OPARM;
/* 
 * get info short query
*/
typedef struct {
    	int     timeMs; /* for data */
    	int     posTTD[3]; /* pos az,gr,ch in 1/10000 of a degree*/
    	int     velTTDS[3]; /* vel az,gr,ch in 1/10000 of a degree/sec*/
    } PNTAGC_MSG_MON_POS;
#endif
