/*
 *	include file for bufpool routines
 *	this is the version that uses vxWorks doubly linked lists...
*modification history:
*06sep94, added misc1,misc2 to bufnode ...
*/
#include "aoqLib.h"
#ifndef INCbufPoolLib
#define INCbufPoolLib
#include 	<semLib.h>
#include	<lstLib.h>
#include	"aoErrNum.h"
#include	"aoqLib.h"
/*
 *	node types. data or command
*/
#define	BPN_TYPE_DATA	0
/*	need 16 extra bytes to have a buffer the size they request*/
#define BP_MEM_PART_OVERHEAD 16
/*
 *	buf pool defintions: 
 *	bufpool made up of -
 *            hdrinfo..    ptr to freelist (AOQ_PTR)
 *                         bufsize for data bufs
 *	                   max number of data buffers
 *                         ptr to large malloc used for data bufs
 *
 *	      databufs. this is pnted to by the malloc ptr in bufpool.
 *
 *	                   node linked list ptrs    - 
 *                         bufpool_ptr		     |
 *                         int cmd		     | - node 1
 *                         databuf		     |
 *                           ..			     |
 *                         --------------------------
 *	                   node linked list ptrs    - 
 *                         bufpool_ptr		     |
 *                         int cmd		     | - node 2
 *                         databuf		     |
 *                           ..			     |
 *                         --------------------------
 *	                   node linked list ptrs    - 
 *                         bufpool_ptr		     |
 *                         int cmd		     | - node 3
 *                         databuf		     |
 *                           ..			     |
 *                         --------------------------
 *                         
 *	a bufpool is initialized then all of the nodes are put on the free list
 *	you then take nodes off the free list, use, and put back on the free
 *	list.
 *
 *	define a buffer pool structure
 *    bufPool     structure tag
 *    BUFPOOL 	  typedef for struct bufPool
 *    BUFPOOL_PTR typedef ptr to bufpool.
 *    bufNode     structure tag for a node.
 *    BUFNODE     typedef for a bufnode
 *    BUFNODE_PTR typedef ptr to bufNode
*/
typedef	struct bufPool {
	AOQ_PTR	paoq;		/* ptr to queue*/
	int	bufSize;	/* fixed length buffers*/
	int     maxbufs;	/* started with*/
	char    bpname[8];	/* name for bp*/
	UCHAR   *pmalloc;	/* for big buffer*/
	} BUFPOOL;

typedef  BUFPOOL *BUFPOOL_PTR;	/* ptr to a bufpool*/

/*
 *	define a bufnode. It will consist of some node info followed by the 
 *      actual data. Make the node info 16 bytes. If we alloc
 *      sizeof(memNode + databytes), the data bytes should start on an
 *      8 byte boundary (at least in sunos).
*/

typedef struct bufNode {
	NODE  	  llNode;		/* 2 wds. node used by linke list*/
  	BUFPOOL_PTR pbufPool;		/* 1 wd. ptr to buf pool belongs to*/
/*
 *	new
*/
	int	usecnt;			/* <=0 before free*/
	int     misc1;			/* misc storage for programs*/
	int     misc2;			/* ditto*/
	} BUFNODE;

typedef	BUFNODE	*BUFNODE_PTR;	/* ptr to a node.*/

/*	globals 	*/	

extern  PART_ID bpMemPartId;         /* take bufPools from here*/
extern  char*   bpMemPartpPool;      /* ptr in sysmemPart for large buf*/
extern  int     bpMemPartMaxSize;    /* max size of memory for buf pools*/
/*
 *	macros
*/
/*  bufnode ptr to vw linked list node ptr*/
#define PN_BUFTOVW(pbufnode) (&((pbufnode)->llNode))
#define PN_VWTOBUF(pvwnode)  ((BUFNODE_PTR)(pvwnode))
#define BUFNODE_DATAPTR(pbufNode)  ((char *)(pbufNode) + sizeof(BUFNODE))
#define BUFNODE_DATANODE(pbufNode) (((pbufNode)->nodetype) == BPN_TYPE_DATA)
#define bpCurCount(pbufPool) (aoqCurCount((pbufPool)->paoq))
/*
 *	prototypes for functions
*/
STATUS      bpGlInit(void);
STATUS	    bpMemPartCreate(int numbytes);
STATUS	    bpMemPartDelete(void);
BUFPOOL_PTR bpInit(int buflen, int numbufs,char *bpname);
BUFNODE_PTR bpGetNodeFl (BUFPOOL_PTR pbufPool,FUNCPTR chkstop,int timeout);
STATUS      bpPutNodeFl(BUFNODE_PTR pbufNode,FUNCPTR chkstop,int timeout);
STATUS      bpPutNodeQ(AOQ_PTR paoq,BUFNODE_PTR pbufNode,FUNCPTR chkstop,
                       int timeout);
BUFNODE_PTR bpGetNodeQ(AOQ_PTR paoq,FUNCPTR chkstop,int timeout);
void        bpInfo(FILE *fp,BUFPOOL_PTR pbufPool);
STATUS      bpRemove(BUFPOOL_PTR pbufPool);	/* delete bufPool*/

#endif /* INCbufPoolLib*/
