#ifndef EPHM_EMBC_LIBH
#define EPHM_EMBC_LIBH
/*
 *	include file for ephmEmbcLib.c. This is a set of library routines
 *	that access the binary earth moon barycenter file.
 *  .. no longer used for datataking ..
*/
/*	includes	*/

#include	<ephm77.h>
/*	defines		*/

#define	EPHM_EMBC_REC_SPACING_DAYS   4

/* 1st record of file has header info. below is offset to 1st data rec	*/
#define	EPHM_EMBC_1STREC_OFFSET_RECS 1

#define EPHM_EMBC_REC_LEN_BYTES      sizeof(EPHM_EMBC_REC)

/*  earth moon ratio, this is at the beginning of the ephm ascii file header */
#define EPHM_EMBC_EMB   82.3005594483797

/*  default filename for binary ephemeris file 	*/

#define	  EPHM_EMBC_DEF_FILENAME  "/usr/local/pulsar/ephm/ephemr"


/*  	typedefs 	*/
/*
 *	structure to hold position/velocity of earth or moon.
 *	positions are measured in au
 *	velocities are measured in au/day
*/
typedef	struct {
	double	p[3];		/* pos  x,y,z position */		
	double  v[3];		/* velocity  vx,vy,vz */
	} POS_VEL;
/*
 *	record layout of embc file 
 *	 - each record is sizeof(EPHM_EMBC_REC)  bytes.
 *	 - The first record does not contain position. It contains
 *	   the 1st/last julian dates of the binary ephemeris file
 *	   as two double at the beginning of the rec (8*2 bytes). It is
 *	   a normal record length.
 *	The data is in 1950 coordinates.
*/
typedef	struct {
	double	jdate;		/* full julian date */
	POS_VEL   e_Sun; 	/* earth pos/vel  relative to sun */     
	double  unknown1;
	POS_VEL   e_SSBC;       /* earth  pos/vel rel to sol system barycenter*/
	POS_VEL   m_EMBC[8];    /* moon position/vel rel earth-moon barycenter*/
	double    delPsi[8];    /* nutation. delta psi (longitude) in ecliptic*/
	double    delEps[8];    /* nutation. delta eps (obliquity) of ecliptic*/
	} EPHM_EMBC_REC;
/*
 *	handle used to keep track of ephemeris file. returned on open call
*/
typedef struct {
	int	fd;			/* file descriptor for file */
	double  julday1st;		/* first julian day in file */
	double  juldayLast;		/* last julian day in file */
	} EPHM_EMBC_HANDLE;
/*
 *	prototypes
*/
STATUS  ephmEmbcOpen(char* fileName,EPHM_EMBC_HANDLE* fhandle);
STATUS  ephmEmbcRd(EPHM_EMBC_HANDLE  *fhandle,double juldate,
                EPHM_EMBC_REC* prec);
#endif
