#ifndef INCgcsrLibh
#define INCgcsrLibh
#include	<stdio.h>
/*
 *	defines
*/
#define GCSR_MV167_GCSR_GRP	0xcc
#define GCSR_MV162_GCSR_GRP	0xc2
/*
 * communications protocol
*
* 6 gcsr registers:
* reg 0  cmd  reg from master to slave
* reg 1  stat reg from slave to master
* reg 2-3  1st 32 bit value ... addresses
* reg 4-5  2st 32 bit value ... word count 
*
* master sends command:
* 1. check that slave is done with cmd, mod counter== expected value
* 2. load data regs 2-5
* 3. generate mb interrupt this slave
* 4. wait for reply by polling reg 1 status done of slave and mod counter
*    one greater than last value. You can also wait on interrupt from 
*    slave if interrupt bit was set.
* 5. get status and any reply info.
*    if ERR_LAST_CMD != 0, look at reg2-3 for the errno.
*
* slave receives commands:
* 1. set status bit to done with command,
* 2. wait for mb interrupt.
* 3. set status to not_done with command
* 4. when done, set done with command, increment stat_cntr modulo 4.
*/
#define GCSR_CMD_BIT_REPLY_WITH_INT 0x8000
#define GCSR_CMD_MASK		    0x7fff

/*
 *	commands that are gcsr specific
*/
#define GCSR_CMD_SLV_EXIT	0x100
#define GCSR_CMD_SLV_OPEN 	0x101
#define GCSR_CMD_SLV_CLOSE 	0x102

#define GCSR_STAT_ERR_LAST_CMD	0x08

typedef struct {
	short 	cmd;		/* passed around*/
	short   useIntReply;	/* reply with interrupt*/
	int	data[2];	/* data passed*/
	} GCSR_DAT;	

typedef struct {
	int	late;		/* for interrupt*/
	int	timeOut;	/* to timeout in ticks*/
	int	intCnt;		/* interrupt count*/
	} GCSR_INT_INFO;
typedef	struct {
	int	cpuNum;		/* this slave*/
	int	errNum;		/* last transaction this slave*/
	int	slvTInd;	/* slave table index for gcsrLib*/
	}  GCSR_DEV_SLV;
/*
 *	structure holding info on slaves initialized by master
*/
typedef	struct {
	int	     numSlvs;	/* in this device*/
	int	     cpuGrp;		/* mv162/mv167 group number*/
	GCSR_DEV_SLV *slvI;		/* holds info on all slaves this device*/
	}  GCSR_DEV;
#if FALSE
/*
 *	structure holding info on slaves initialized by master
*/
typedef	struct {
	int	numSlvs;	/* in this device*/
	int	cpuGrp;		/* mv162/mv167 group number*/
	int	*cpuList;	/* array we allocate for cpuNums*/
	int	*slvTList;	/* allocated by library*/
	}  GCSR_DEV;
#endif
/*
 * 	prototypes for functions
*/
void    gcsrDbg(void);
STATUS  gcsrMasBrdAvail(int slvTInd,int *pbrdAvail);
STATUS  gcsrMasDevFree(GCSR_DEV *pdev);
STATUS  gcsrMasDevInit(int numSlvs,int cpuGrp,int *cpuList,GCSR_DEV *pdev);
STATUS  gcsrMasDevReset(GCSR_DEV *pdev,int wait,int timeOutSecs);
STATUS  gcsrMasIntGen(int slvTInd);
void    gcsrMasPrDev(FILE *fp,GCSR_DEV *pdev);
void    gcsrMasDumpGcsr(FILE *fp,GCSR_DEV *pdev);

STATUS  gcsrSlvInit(void);
STATUS  gcsrSlvWaitInt(GCSR_INT_INFO *pintI);
/*
*/
STATUS  gcsrMasSendCmd(int slvTInd,GCSR_DAT *pdat);
STATUS  gcsrMasWaitReply(int slvTInd,int timeout,int *plate,int *pslvRetErr);
STATUS  gcsrMasSndRcv(int slvTInd,int cmd,int intReply,int word1,int word2);

STATUS  gcsrSlvGetCmd(GCSR_DAT *pdat);
STATUS  gcsrSlvReply(int cmdErrno,int numBytes,char *plocBuf,
	             char *pmasBuf);

#endif	/*INCgcsrLibh*/
