/*
 *	include file for gioProg.c used to write to tape.
*/
#ifndef INCgioProgh
#define INCgioProgh
#include	<semLib.h>
#include	"bufPoolLib.h"
#include	"aoMsgQLib.h"
#include	"gioProgLib.h"

#define	    GIOPROG_NAME	"gioProg"
#define	    GIOPROGR_NAME	"giorProg"
#define	    MAX_REMOTES		1

/*
 *      typedef for setup structure passed to gioProgfrom outside
 *      program. A pointer to this is passed in CMDGLSETUP
*/
#define	GIO_RSTAT_NOT_RUNNING	0		
#define	GIO_RSTAT_ENABLED       1
#define	GIO_RSTAT_WAIT_REPLY    2

typedef struct {
	double		last;	/* last time*/
	double		min;	/* min time*/
	double		max;	/* max time*/
	int			numSent;
	double	    sum;	/* for average*/
	} GIO_TM;

typedef struct {
	char	    cpuName[20];	/* name of cpu to send to (on this backplane)*/
	int			stat;           /* status remote program*/
	AOM_ADR     aoAdr;			/* address to use*/
	GIO_TM	    tm;			    /* to send info*/
	} GIO_REMOTE;

typedef struct  {
	int		   stopProgFlag;			/* true if stopProg Requested*/
	AOMSG_Q_ID inpCmdQ;         /* inpQ for commands*/
	AOMSG_Q_ID replyQ;          /* for replys*/
	AOM_ADR    replyA;			/* for our messsages to remote*/
	int		   chktimeout;		/* wake this often. in ticks*/
	volatile int progRunning;   /* true when running*/
	volatile int progStatStartup;/* set ERROR if trouble on startup*/
	int        ioPermitted;		/* if we can do i/o*/
	int		   statLastWrite;

	int		   seqNum;			/* sequence number for outputs*/
	int		   bufsPerMsg;	    /* hold this many bufs before send to remote*/
	int		   numRemActive;	/* number of remote programs active*/	
	int		   curRem;			/* index for current remote.cnt from 0*/
	GIO_REMOTE remList[MAX_REMOTES];/* hold list of possible remotes*/
} GIOINFO;

typedef struct {
	int	stopProgFlag;		/* for this program to quit*/
	AOMSG_Q_ID inpCmdQ;         /* inpQ for commands*/
	AOMSG_Q_ID replyQ;          /* for replys*/
	int		   chktimeout;		/* wake this often. in ticks*/
	int		   dmaTimeout;		/* ticks before dma read times out*/
	volatile int progRunning;   /* true when running*/
	volatile int progStatStartup;/* set ERROR if trouble on startup*/
	int        ioPermitted;		/* if we can do i/o*/

	int		   fdIpGio;			/* for sending data */
	int		   lastSeqNum;		/* we received*/
	int		   lastByteLen;		/* we received*/
	int		   bufToOutput;		/* true if we have a buffer to output*/

	int		   totBufsInpOk;
	int		   totBufsInpErr;
	double     totBytesInpIo;		/* transferred since prog booted*/

	int		   totBufsOutOk;	/* error we had on output*/
	int		   totBufsOutErr;	/* error we had on output*/
	int		   statLastWrite;
	int		   lastErrOutput;
	int		   noHdrErrCnt;     /* how many ouput buffers no hdr_*/
	double     totBytesOutIo;		/* transferred since prog booted*/
	GIO_TM 	   wtm;					/* write times*/

	int		   maxLenLocBuf;	/* largest buf we can handle.. bytes*/
	char	   *pbuf;			/* we malloc this*/
	}GIORINFO;

/*
 *	function prototypes
*/
int     gioIoPerm(void);
void	gioProgDbg(void);
STATUS  gioProgIsRunning(void);
STATUS  gioProgStart(void);
AOMSG_Q_ID gioRetpCmdQ(void);

#endif  /* gioProgh */
