#ifndef incHP1660LIBH
#define incHP1660LIBH

#include	<stdio.h>
/*	defines */
#define HP1660_DEV_1    "/hp1660_1"
#define HP1660_DEF_ADR  0x07
#define HP1660_TIMEOUT  60*20

#define LARMT_READ     0x00a

/*
 * data order returned from hp1660:
 *    HP1660_BLKLEN_DESCRIPTOR   24 bytes (includes my 1 extra byte)
 *    HP1660_SECHDR		 16 bytes
 *    HP1660_DATA_PREAMBLE       160 bytes
 *	data as      : HP1660_DATA_ROW[] 
 *	timestamps as: HP1660_DATA_ROW[]
 * 
*/
/*
 *	block length descriptor
*/
typedef	struct {
	char    sp;		/* i add a space when i read it back*/
	char	cmd[13];	/*:SYSTEM:DATA_*/
	char	pnd;		/* # */
	char    eight;		/* 8 */
	char	blkLen[8];	/* block length in bytes*/
	} HP1660_BLKLEN_DESC;
/*
 *	 section header 
*/
typedef struct {
	char    	sname[10];	/* data   not null terminated*/	
	char		rsv1;		
	char		modId;		/* module id*/
	int		secLenBytes;	/* section data length in bytes */
	} HP1660_SECHDR;
/*
 * 	data preamble
*/
typedef struct {
	char	mDatMode;	/* machine data mode*/
	char	rsv1;		
	unsigned usePodFr1:7;	/* which pods used this ananlyzer*/
	unsigned usePod8:1;
	unsigned usePod7:1;
	unsigned usePod6:1;
	unsigned usePod5:1;
	unsigned usePod4:1;
	unsigned usePod3:1;
	unsigned usePod2:1;
	unsigned usePod1:1;
	unsigned usePodFr2:1;
	char	 timeStateTagChip;
	char	 masterChip;		/* this analyzer*/
	char	 rsv2[6];
	char	smpPeriod[8];	   /* 8 byte integer in pico secs*/
	char	 rsv3[8];
	char	 stateTagType;		/*0off,1 time,2 state*/
	char	 rsv4;
	char     trigToIMBOutPort[8];	/* 8byte integer, in picosecs*/
	char	 rsv5[2];
	} HP1660_ANALYZER_INFO;

typedef struct {
	char	rsv1[10];
	unsigned short       p[8];	
	} HP1660_PODLIST;
/*
 *	 heres the actual preamble
*/
typedef struct {
	short	instrId;	/* instrument id*/
	char	revCode;	/* revision code*/
	char	numAcqChips;	/* used last acquisition*/
	HP1660_ANALYZER_INFO anz1;
	HP1660_ANALYZER_INFO anz2;
	HP1660_PODLIST       numDRows;    /* pods 8 thru 1*/	    
	HP1660_PODLIST       rowTrigOff;  /* row offset for trigger each pod*/ 
	char	rsv2[24];
	} HP1660_DATA_PREAMBLE;
/*
 *	single row of data for one time sample
*/
typedef struct {
	unsigned short	clk;
	unsigned short	p[8];
	} HP1660_DATA_ROW;
/*
 *	time tag samples
*/
typedef struct {
        char	timeTag[8];	
	} HP1660_TIME_TAG;
/*
 *	how user wants data formatted 
*/
typedef	struct {
	  int	numBits;	/*bits to use (lower part)0--> ignore pod*/
	  int   twosComplement; /* true --> twos complement data */
	  int   usePod[8];	/* pod 8 thru 1 non-> zero --> return it*/
	} HP1660_DATA_FORMAT;
/*
 *	 section info data
*/
typedef	struct {
	HP1660_SECHDR		sHdr;		/* section header*/
	HP1660_DATA_PREAMBLE    dPre; 	/* data preamble*/
	HP1660_DATA_ROW		*pdata; 	/* ptr data rows*/
	int                      maxNumDataRows; /* we have in data */
	HP1660_TIME_TAG		*ptimeTags;	/* set for each chip*/
	} HP1660_INFO;
	
/* 
 *	prototypes
*/
#ifdef VXWORKS
int     hp1660Create(int gpibAdr,int devNum);
int     hp1660Open(void);
STATUS	hp1660RdData(int fd,char* buf,int maxLen,int *pbytesRead);
STATUS  hp1660Run(int fd,int timeOut);
#endif
int     hp1660FormDat(HP1660_INFO *pI,HP1660_DATA_FORMAT *pdF,int *pdat);
void    hp1660Info(char *pbuf,HP1660_INFO *phpI);
double  hp1660LongInt(char *pdat);
void    hp1660OutBlkLen(FILE *fp,char* buf);
void    hp1660OutDataPre(FILE *fp,HP1660_DATA_PREAMBLE *pdPre);
void    hp1660OutSecHdr(FILE *fp,HP1660_SECHDR *pshdr);

	

/*  #8000027126   
    section hdrlength 16
    section length=27110*/

#endif /*incHP1660LIBH*/



