#ifndef incHP3488LIBH
#define incHP3488LIBH

/*	defines */
#define HP3488_DEV_D1    "/hp3488_d1" 
#define HP3488_DEV_D2    "/hp3488_d2" 
#define HP3488_DEV_D3    "/hp3488_d3" 
#define HP3488_DEV_U1    "/hp3488_u1" 
#define HP3488_DEV_U2    "/hp3488_u2" 
#define HP3488_D1_DEF_ADR  0x1b
#define HP3488_D2_DEF_ADR  0x1c
#define HP3488_D3_DEF_ADR  0x1d
#define HP3488_U1_DEF_ADR  0x1e
#define HP3488_U2_DEF_ADR  0x14
#define HP3488_TIMEOUT     20  

/*
 * error reg bits
*/
#define	HP3488_EREG_SYNTAX 	0x01
#define HP3488_EREG_EXEC   	0x02
#define HP3488_EREG_TRIG   	0x04
#define HP3488_EREG_LOGIC  	0x08
#define HP3488_EREG_TRIG   	0x04
#define HP3488_EREG_POWER  	0x10
/*
 * card types
*/
#define HP3488_CTYPE_NO_CARD	0
#define HP3488_CTYPE_RELAY_CARD 44470
#define HP3488_CTYPE_GP_RELAY   44471
#define HP3488_CTYPE_VHF_SW     44472
#define HP3488_CTYPE_MATRIX_SW  44473
#define HP3488_CTYPE_DIGITAL_IO 44474
#define HP3488_CTYPE_BREADBOARD 44475
/*
 * digital read/write port/byte locations
*/
#define HP3488_DWRITE_LOW8	0
#define HP3488_DWRITE_HI8	1
#define HP3488_DWRITE_16        2
/*
 * typedefs
*/
/*
 * when building switch tables.
*/
typedef struct {
        int     swNum;
        char    swName[12];      /* hold name of switch*/
        int     slot;           /* in 3488 1->5*/
        int     bitSt;          /* 1st bit.. 0 base*/
        int     bitEnd;         /* last bit..0base*/
        int     mutExcl;        /* true--> 1 bit at a time if multi bit*/
        } SWI_3488;

typedef struct {
	   int data;           /* for slot*/ 
	   int okToUse;        /* slot is ok to use*/ 
	   }   HP3488_SLOT; 

/*
 *	 info on an hp3488 device
*/
typedef struct  { 
    int     fd;          /* for this device    */ 
    int     numSlots;    /* in use*/ 
    HP3488_SLOT slI[5];  /* info for maximum of 5 slots*/ 
	SWI_3488 *ptbl;		 /* ptr to static switch table*/
    }  HP3488_INFO;



/* 
 *	prototypes
*/
int     hp3488Create(int gpibAdr,char *devNum);
int     hp3488Open(char *devNum);
STATUS  hp3488ChOpen(int fd,int numAdr,int *padr,int *pchn);
STATUS  hp3488ChClose(int fd,int numAdr,int *padr,int *pchn);
STATUS  hp3488ChView(int fd,int adr,int chn,int *pval);
STATUS  hp3488DMode(int fd,int adr,int mode,int polarity);
void    hp3488DbgPr(FILE *fp,HP3488_INFO *pinfo,char *name);
STATUS  hp3488DRead(int fd,int adr,int port,int *pdata);
STATUS  hp3488DWrite(int fd,int adr,int port,int data);
STATUS  hp3488QSlType(int fd,int adr,int *pctype);	
STATUS  hp3488RdErr(int fd,int *perrVal);
STATUS  hp3488Recall(int fd,int reg);
STATUS  hp3488SlClose(int fd,int adr,int maxSlots);
STATUS  hp3488SlOpen(int fd,int adr,int maxSlots);
STATUS  hp3488SlReset(int fd,int adr);
STATUS  hp3488SlView(int fd,int adr,int maxSlots,int *pval);
STATUS  hp3488Store(int fd,int reg);


#endif /*incHP3488LIBH*/
