#ifndef INCif1ProgLibh
#define INCif1ProgLibh

/******************************************************************************
*DESCRIPTION
* defines/typedefs used for if1 switch tables
*/
/*
modifications
-------------
12jul98,<pjp001> removed pole z filters
*/
#include	"if1Con.h"
#include	"hp3488Lib.h"

/*	defines */
#define IF1_DLOG_PWR_NAME  "/share/obs4/if1/pwr/log"

#define IF1_SWCLOSED    1
#define IF1_SWOPEN     	0 

/*   
 *	switch numbers  used as key in switch table
 *	first set on hp87130 device, second on hp3488
*/
#define	SW1A1	1
#define	SW1A2	2
#define	SW1B1 	3
#define	SW1B2	4
#define	SW2A 	5
#define	SW2B 	6
#define	SW3A 	7
#define	SW3B 	8
#define	SW4  	9
#define	SW5A   10
#define	SW5B   11 
#define	SW6A   12
#define	SW6B   13 
#define	SW7A   14
#define	SW7B   15 
#define	SW8A   16
#define	SW8B   17 
#define SWATTN_RF_A    18 
#define SWATTN_RF_B    19
#define SWATTN_IF1_A   20
#define SWATTN_IF1_B   21 
/* 
 * following on hp3488 device #1 
*/
#if FALSE
/* switched removed 2jan03 pjp*/
#define SWATTN5_ZM_NDBIG   100
#define SWATTN6_ZM_ND      101
#define SWATTN7_ZM_POLA    102
#define SWATTN8_ZM_POLB    103
#define SWND_ZM_ONOFF      105
#define SWND_ZM_TOAB       106
#endif

#define SWNUM_START_HP3488 100
#define SW750_NB           100
#define SW2_12WB           101
#define SWTILT             102
#define SWXFER_ZM          104
#define SWPHADJ_SIG        107
#define SWPHADJ_LO         108
#if FALSE
<pjp001>
#define SWPZ430_NOTINUSE_1 109 
#endif
#define SWTERACON 		   110
#define SWLBWCIRLIN        111
#define SWSHUT_SB          112
#define SWCALTYPE          113
#define SWLBFBA            114
#define SWLBFBB            115
#define SWCALRCVR          116
#define SWCBCIRLIN         117
#define SWCBCIRLINM        118
#define SWAC1SW            119
#define SWAC2SW            120
#define SWSHUT_RCV         121
/* 
 * following on hp3488_2 device #2 
*/
#define SW430FB             150
#define SWFREE_151          151
#define SWSBWFBA            152
#define SWSBWFBB            153
#define SWALFAFBPWR         154
#define SWALFAFB1A          155
#define SWALFAFB2A          156
#define SWALFAFB1B          157
#define SWALFAFB2B          158

/* 
 * following on hp34970 device 
*/
#define SWNUM_START_HP34970 200
#define SWLBNFB_FILTER      200
#define SWLBNFB_BRK         201
#define SWMF1_FREE          202
#define SWMF2_FREE          216
#define SWMUX_LBNA          232
#define SWMUX_LBNB          233

/*	typedefs	*/

/*
 *  the switches are configured via switch tables.
 *
 *  hp87130 switch table entry
*/
typedef struct {
        int     swNum;
        char    swName[12];      /* hold name of switch*/
        int     verify;         /* true --> this switch can be verified*/
        int     mutExcl;        /* true --> mutually exclusive. one bit only*/
        int     card;
        int     chn1;
        int     chn2;           /* < 0 --> only 1 channel*/
        } SWI_87130;
/*
 * 
*/
typedef struct {
        int     swNum;
        char    swName[12];      /* hold name of switch*/
        int     card;           /* in 34970 1->3*/
        int     bitSt;          /* 1st bit.. 0 base*/
        int     bitEnd;         /* last bit..0base*/
        int     mutExcl;        /* true--> 1 bit at a time if multi bit*/
        } SWI_34970;

/* 
 *	map rfNum/mixer num  to freq/name
*/
typedef struct {
	int		num;		/* rf number*/
	int	   freq;		/* i mhz*/
	char  name[8];		/* i mhz*/
	} IF1_NUM_TO_FREQ;

/*
 *  info on 1 switch
*/
typedef struct {
        int     swNum;          /* switch number.. see table above*/
        int     swPos;
        } SW_SET;
/*        
 *      rf switch table
*/
typedef struct {
        int     rfNum;      /* 1 thru N */
        int     numSwNeeded;/* to set this rcv*/
        SW_SET       sws[6];/* max 6 switches. 1st half pol a, 2nd half polb*/
        } RF_ENT;
          
/*
 *  hold info on each IF1  setting
*/
typedef struct  {
        int             ifNum;   /* if1 num 1..n */
        SW_SET          synth;  /* for synthesizer*/
        SW_SET          chnA[2];/* pol a switch position*/
        SW_SET          chnB[2];/* pol a switch position*/
        } MIXER_ENT;
/*        
 *      hold info on cable switch
*/
typedef struct {
        SW_SET  fibChnA;
        SW_SET  helChnA;
        SW_SET  fibChnB;
        SW_SET  helChnB;
        } CABLE_TBL;
/*        
 *      hold info on hybrid switch for 10ghz if
*/
typedef struct {
        SW_SET  out[4];
        SW_SET  in[4];
        } HYBRID_TBL;
/*        
 *      hp87130 switch errors
*/
typedef struct {
    int swNum;
    int myErrno;    /* my errno*/
    int hpErr1; /* hp 1st error*/
    int hpErr2; /* hp 2nd error or 0*/
    } SW87130_ERR;

typedef struct {
    int          numErrs;      /* number valid entries in swErr*/
    SW87130_ERR  sw[IF1_87130_MAX_SW_ERRS];
    } SW87130_ERRS;
/*
 * structure for program logging
*/
typedef struct {
    int     timeStamp;  /* dddsssss when power measured*/
    short   pwr1;       /* units .01 dbm*/
    short   pwr2;       /* units .01 dbm*/
    } IF1_PWR_LOG;

/*	prototypes	*/

int        if1LAttnDbToSw(int attnDb);

SWI_3488  *if1LGet3488SwPtr(int swNum);
SWI_3488  *if1LGet3488TblPtr(void);
SWI_3488  *if1LGet2_3488SwPtr(int swNum);
SWI_3488  *if1LGet2_3488TblPtr(void);
int        if1L3488MakeMask(SWI_3488 *pswI);
SWI_34970  *if1LGet34970SwPtr(int swNum);
SWI_34970  *if1LGet34970TblPtr(void);
int        if1L34970MakeMask(SWI_34970 *pswI);
SWI_87130 *if1LGet87130SwPtr(int swNum);
SWI_87130 *if1LGet87130TblPtr(void);
int        if1L87130MapBits(int card,int bitVals,int maxSw,int *pswNums);
SW_SET    *if1LGetCablePtr(int polA,int cableType);
SW_SET    *if1LGetHybridPtr(int hybridIn);
MIXER_ENT *if1LGetMixerPtr(int ifNum);
RF_ENT    *if1LGetRfPtr(int rfNum);
STATUS     if1LGetSwName(int swNum,char *swName,int maxLenName);
STATUS     if1LIfNumLkup(int ifNum,int *pifFreq,char **ppname);
STATUS     if1LRfNumLkup(int rfNum,int *prfFreq,char **ppname);

#endif /*INCif1ProgLibh*/
