/*
 *	include file for lrMon program
*/
#ifndef INClrMonh
#define INClrMonh

#include	<lr.h>

/* status   */

typedef struct {
        unsigned int fill:15;
/*
 *  If either of the next 2 fields are non zero then we have failed to
 *  read the data at least that many times.. Either the distomat program
 *  is hung up, the network is having trouble, or the disc files are full.
 *
 *	we had trouble with the file not growing (timeouts)
 *  count up to a max of 15 times then sit there. 1 good file grow
 *  resets counter to 0.
 * .. need to implement..
*/
		unsigned int fileGrowErr:4;
/*
 *	consecutive number of times we had trouble reading the file (or seeking)
 *  count up to 15 then sit there. One good i/o on the file resets counter
 *  to zero.
*/
		unsigned int readFileErr:4;
/*
 *	 The below status is for the data in lrmon_datcmp. It was successfully
 *   input but may have a bad heigth or temp. inpOk will always be 1

*/
        unsigned int hgtOk :1;
        unsigned int tempOk:1;
        unsigned int inpOk :1;
        unsigned int distOk:6;
        } LRMON_STATWD;
/* coord sys from jayrham chengalur (right handed)
    x-axis:west  y-axis:north  z-axis:towards bowl
*/
/*
 * computed data
*/
typedef struct {
    double   dx;  /* delta motion from standard position */
    double   dy;
    double   dz;
    double xrot; /* rotations about an axis*/
    double yrot;
    double zrot;
    double avgHgt;  /* feet above sea level*/
    double hgt[3];  /*feet above sea level. td12,td4,td8*/
    } LRMON_DATCMP;
    /*
     * LRMON_DATA. 
	 * holds :
	 * 1. last successful input from disc data .. inp (has time stamp)
	 * 2. computed distances input data
	 * 3. secMid,dayNum when data 1,2 was input by lrMon program.
	 *    the timestamp of the data in inp may be different if we just
	 *    started lrmon (on a reboot).
	 * 4. statWd status of data in 1,2, and the status of last i/o attempt
	 *    it may not have been successful..
	 * 5  useCnt Increment each time we try to do an i/o
    */
typedef struct {

    LRMON_DATCMP  cmp;     /* computed data*/
    LR_DATA       inp;      /* input data */

    int           secMid;  /* last successful input*/
    int           dayNum;  /* for secMid*/
    LRMON_STATWD  statWd;  /* 1 in bit is ok*/
    int           useCnt; /* incr each time we disc i/o from file is ok*/
    } LRMON_DAT;


#endif	/* INClrMonh*/
