#ifndef INCmctrlLibh
#define INCmctrlLibh

/*
modification history:
xx.x,07jun96,pjp.. in slv io structure remember last msgid sent since we
				   may do 2 writes before a read on the same network..
*/
#include 	<semLib.h>
#include	<wdLib.h>
#include	<stdio.h>
/*
 * micro controller lib include file for vxWorks programs.
*/

#define MCTRL_TUR_NET		1
#define MCTRL_TER_NET	    2
#define MCTRL_TIE_NET	    3
#define MCTRL_TUR_BAUD		38400
#define MCTRL_TER_BAUD		38400
#define MCTRL_TIE_BAUD		38400
#define MCTRL_TIMEOUT_TICKS     60

#define	MCTRL_M_HDRID1	    0xaa55
#define	MCTRL_M_HDRID2	    0x55aa

#define MCTRL_S_MAXLEN_DATA 251
#define MCTRL_S_MAX_SLAVES   5
/*
 *	commands that master uses on rs-232 link.
*/
#define MCTRL_M_CMD_TO_MICROS	1
#define MCTRL_M_CMD_ECHO      	3

/*
 *	communications status returned from master to vme in cmdStat location
 *	the status of the slave is embedded in the data portion.
 *
 *   stat        hdr/data   description
 *
 * 	OK	   hdr/data snd/rcv ok      
 *      slvtimout  hdr      slave didn't reply within timeout
 *      slvBADCRC  hdr      bad crc returned from slave
*/
#define MCTRL_COM_STAT_OK	           0  
#define MCTRL_COM_STAT_M_SND_TMOUT 	   1
#define MCTRL_COM_STAT_M_BCRC_FROM_S   2
#define MCTRL_COM_STAT_M_BSTAT_SEND	   3
#define MCTRL_COM_STAT_S_BCMD_FROM_M   4
#define MCTRL_COM_STAT_S_BCRC_FROM_M   5
#define MCTRL_COM_STAT_S_BDATA_FROM_M  6
#define MCTRL_COM_STAT_M_BCMD_FROM_VME 7
#define MCTRL_COM_STAT_S_PARAM_NOT_SAVED  8
#define MCTRL_COM_STAT_S_INDEX_OUT_OF_RNG 9
#define MCTRL_COM_STAT_S_BUF_OVERFLOW   0xa

/*
 *	commands that all slave understand from master
*/
#define MCTRL_S_CMD_ECHO	3
	/*
	 * header sent from 167 to master micro via rs232
	*/
typedef struct {
	unsigned short	hdrId1;	/* 0xaa55 */
	unsigned char	cmdStat;/*to master cmd, from master status MCTRL_CMD*/	
	unsigned char	adr;	/* 485 link to or from address*/
	unsigned char	byteLen;/* number bytes data in msg*/
	unsigned char   msgId;		/* ...   */
	unsigned short  hdrId2;	/* 0x55aa*/
	} MCTRL_COM_HDR;
	/*
	*	keep track of last i/o read/ writes for debugging
	*/
typedef struct {
	MCTRL_COM_HDR	hdr;		/* header*/
	int		bytesHdr;	/* # bytes xfered last i/o header*/
	int		bytesData; 	/* # bytes xfered last i/o data*/
	int		error;		/* any error code we had*/
	} MCTRL_IO_INFO;

typedef struct {
	int	wdCnt;		/* watchdog errors this slave.*/
	int	wrCntTry;	/* total attempted writes*/
	int	wrCntOk;
	int	wrErrs;		/* write errors*/
	int	wrLastErr;  /* always remember last error*/
	int	rdCntTry;	/* reads where we got a good address*/
	int	rdCntOk;	/* reads where we got a good address*/
	int	rdErrs;		/* total errors rd*/
	int	rdErrMsgIndex;  /* msg index mismatch snd/rcv*/

	int	rdErrMSndTmout;	/* master timed out on slave read          1*/
	int	rdErrMBCrcFS;	/* master rcvd bad crc from slave          2*/
	int	rdErrMBStatFS;/* master rcvd bad stat from send/rcv opto22 3*/
	int	rdErrMBCmdFV;/* master rcvd bad cmd from vme               7*/
	int rdErrSBCmdFM;	/* slave received bad cmd from master      4*/
	int rdErrSBCrcFM;   /* slave received bad crc				   5*/
	int rdErrSBDataFM;/* slave received data out of legal range    6*/
	int rdErrSPNotSaved;/* parameter not saved			           8*/
	int rdErrSBIndex;      /* index out of range  		           9*/
	int rdErrSBufFull;     /* tracking buf overflow 			   a*/
	int wrLastMsgId;/* last msg id on write for this slave*/
	} MCTRL_SLV_INFO;
	
	/*
	 * hold struct for a micro controller.
	*/
typedef struct {
	  int		fdPort;		/* com port*/
	  SEM_ID	wdSem;		/* watch dog semaphore*/
	  WDOG_ID	wdId;		/* watch dog Id*/
	  unsigned int  wdCnt;	 	/* wd timeouts in task*/
	  int		wdErr;		/* true if wd error occured*/
	  int	    progsExit;	/* everyone exit*/
	  int	  	wrCnt;		/* # write msg attempts*/
	  int	  	rdCnt;		/* # read msg attempts*/
	  int		syncErCnt;	/* times we called sync*/
	  int		syncOkCnt;	/* times we sucessfully synced*/
	  int		dbg[10];
	  MCTRL_IO_INFO cWr;		/* current write*/
	  MCTRL_IO_INFO cRd;		/* current read*/
	  MCTRL_IO_INFO lWr;		/* last write*/
	  MCTRL_IO_INFO lRd;		/* last read*/
	  MCTRL_SLV_INFO slv[MCTRL_S_MAX_SLAVES +1]; /* use slave num as index*/
	} MCTRL_INFO;

typedef  MCTRL_INFO  *MCTRL_ID;

/*	prototypes	*/

void     mctrlClose(MCTRL_ID mId);
void mctrlDmpHdr(FILE *fptr,MCTRL_COM_HDR *phdr);
void mctrlDebug(MCTRL_ID mId,FILE *fptr);

MCTRL_ID mctrlInit(int netNumber);
STATUS mctrlRcvMsg(MCTRL_ID mId,char *buf,int timeoutTicks,int *pbytesDataRead,
		   int slvAdr);
STATUS mctrlSndMsg(MCTRL_ID mId,int cmd,int adr,int dataLen,char *buf,
		   int timeoutTicks);
STATUS mctrlSync(MCTRL_ID mId,int timeoutTicks);
STATUS mctrlWdCancel(MCTRL_ID mId);
STATUS mctrlWdStart(MCTRL_ID mId,int timeoutTicks);

#endif
