#ifndef incmzh  
#define incmzh
/*
 *              mz7500.h: mz7500 vxWorks Driver include file
*/
/*                      Copyright (c) 1990 Mizar EMX, Inc.
*                            All Rights Reserved
*
*
*      THIS IS UNPUBLISHED, CONFIDENTIAL AND PROPRIETARY SOURCE CODE OF
*      Mizar EMX, Inc. THE COPYRIGHT OF WHICH VESTS IN Mizar EMX, Inc.
*
*      The copyright notice above does not evidence any actual or
*      intended publication of such source code.
*
*/
/*
 * (C) Copyright 1990 Carl Cork. All rights reserved.
 * Permission to copy for any purpose is hereby granted so long
 * as this copyright notice remains intact.
 *
 * $Log: mz.h,v $
 * Revision 1.1  2006/05/19 23:18:43  jeffc
 * *** empty log message ***
 *
 * Revision 1.3  91/01/07  15:54:31  ricb
 * cleaned up
 * 
 * Revision 1.2  90/12/18  15:18:44  ricb
 * changed #defines for board status : switched ATN and LLO
 * to reflect tms address status register mapping
 * 
 * Revision 1.1  90/12/11  15:08:36  ricb
 * Initial revision
 * 
 * PURPOSE:
 *  Descriptor File,
 *  for Mizar MZ IEEE-488 (GPIB) bus controller.
 *
 * COMMENTS:
 * KNOWN BUGS:
 */
/*
*modification history
* 13feb95 pjp - moved prototypes for mzDevCreate/ mzDrv to here..
*/

/*======================== External Declarations =============================*/
#include <vxWorks.h>
#include <vwModNum.h>
#include "Base.h"

/*
 *      ioctl requests
 *   why does 0x12 not make it into the dmzver??. changed to 0x13
*/
#define MZIOCDEVREAD            0x7
#define MZIOCDEVWRITE           0x8
#define MZIOCDEVCLR             0x9
#define MZIOCDEBUG              0xa
#define MZIOCDEVTOLOC           0xb
#define MZIOCDEVSPOLL           0xd
#define MZIOCMULTILINE          0xe
#define MZIOCUNIREN             0xf
#define MZIOCDEVLLO             0x10
#define MZIOCDEVBREAD           0x11


/* IOCtl function codes */

    /* DEVICE CONTROL FUNCTIONS */

#define MZ$K_DCLEAR	    (M_mz7500Lib | 0x92)  /* Clear a device */
#define MZ$K_DLOCAL	    (M_mz7500Lib | 0x93)  /* Enable local control */
#define MZ$K_PASSCONTROL    (M_mz7500Lib | 0x96)  /* Pass control */
#define MZ$K_PPOLL	    (M_mz7500Lib | 0x97)  /* Parallel poll device */
#define MZ$K_PPOLLCONF      (M_mz7500Lib | 0x98)  /* Configure for PPOLL */
#define MZ$K_TRIGGER	    (M_mz7500Lib | 0x99)  /* Trigger a device */
#define MZ$K_SPOLL	    (M_mz7500Lib | 0x9a)  /* Serial poll device */
#define MZ$K_DCONF_RD       (M_mz7500Lib | 0xa1)  /* Return device config */
#define MZ$K_DCONF_WRT      (M_mz7500Lib | 0xa2)  /* Update device config */

    /* BOARD CONTROL FUNCTIONS */

#define MZ$K_CMD	    (M_mz7500Lib | 0x67)  /* Send GPIB commands */
#define MZ$K_ATNON	    (M_mz7500Lib | 0x68)  /* Raise ATN */
#define MZ$K_ATNOFF	    (M_mz7500Lib | 0x69)  /* Lower ATN */
#define MZ$K_SETREMOTE      (M_mz7500Lib | 0x6b)  /* Set remote enable */
#define MZ$K_SENDIFC	    (M_mz7500Lib | 0x6c)  /* Send interface clear */
#define MZ$K_PRESENT	    (M_mz7500Lib | 0x6d)  /* Rtn TRUE if board present */
#define MZ$K_ONLINE	    (M_mz7500Lib | 0x6e)  /* Place board online */
#define MZ$K_OFFLINE	    (M_mz7500Lib | 0x6f)  /* Place board offline */
#define MZ$K_BLOCAL	    (M_mz7500Lib | 0x71)  /* Rtn board to local state */
#define MZ$K_SPOLLUCHAR     (M_mz7500Lib | 0x72)  /* Change serial poll byte */
#define MZ$K_REQUEST	    (M_mz7500Lib | 0x73)  /* Set request service */
#define MZ$K_PPOLLRESP      (M_mz7500Lib | 0x74)  /* Set parallel poll status */
#define MZ$K_CLRREMOTE      (M_mz7500Lib | 0x75)  /* Clear remote enable */

    /* DEVICE AND BOARD CONTROL FUNCTIONS */

#define MZ$K_STATUS	    (M_mz7500Lib | 0x70)  /* Return board status */
#define MZ$K_BWAIT	    (M_mz7500Lib | 0x76)  /* Wait for event */
#define MZ$K_BCONF_RD       (M_mz7500Lib | 0x79)  /* Return board config */
#define MZ$K_BCONF_WRT      (M_mz7500Lib | 0x80)  /* Update board config */

/* MZ_CONFIG - device (and board) configuration structure */

typedef struct		/* MZ_BRD_IOS - MZ_BRD device IO descriptor */
{
  UINT  	timeout;	/* device timeout */
  UINT16	flags;		/* device configuration flags */
  UCHAR		priAddr;	/* device primary address */
  UCHAR		secAddr;	/* device secondary address */
  UCHAR		reosChar;	/* read EOS character */
  UCHAR		weosChar;	/* write EOS character */
} MZ_CONFIG;

typedef MZ_CONFIG* PMZ_CONFIG;

#define MZ_CONFIG$M_FLAGS	0x000f	/* flag mask */
#define MZ_CONFIG$V_EOT	        0x0008	/* enable EOI on write */
#define MZ_CONFIG$V_REOS	0x0004	/* terminate read on EOS */
#define MZ_CONFIG$V_WEOS	0x0002	/* send EOS on write (with EOI) */
#define MZ_CONFIG$V_DMA	        0x0001	/* do DMA I/O transfers */

/* MZ_STATUS - board status word */

typedef UINT32 MZ_STATUS;
typedef MZ_STATUS* PMZ_STATUS;

#define MZ_STATUS$V_RES      0x80000000	/* reserved */
#define MZ_STATUS$V_TIMEO    0x40000000	/* timeout */
#define MZ_STATUS$V_BERR     0x20000000	/* bus error */
#define MZ_STATUS$V_SYSFAIL  0x10000000	/* SYSFAIL LED is lit */
#define MZ_STATUS$V_BCRDN    0x08000000	/* byte count register done */
#define MZ_STATUS$V_FULL     0x04000000 /* FIFO full */
#define MZ_STATUS$V_EODNE    0x02000000 /* EOS/EOI received */
#define MZ_STATUS$V_EMPTY    0x01000000 /* FIFO empty */
#define MZ_STATUS$V_SC	     0x00800000	/* System Controller */
#define MZ_STATUS$V_CIC	     0x00400000	/* Controller In Charge */
#define MZ_STATUS$V_BI       0x00200000	/* Byte In */
#define MZ_STATUS$V_BO       0x00100000	/* Byte out */
#define MZ_STATUS$V_END      0x00080000	/* EOI received */
#define MZ_STATUS$V_SPAS     0x00040000	/* Serial poll received */
#define MZ_STATUS$V_RLC      0x00020000	/* Remote/Local change */
#define MZ_STATUS$V_MAC      0x00010000	/* My address change */
#define MZ_STATUS$V_GET      0x00008000	/* Group execute trigger */
#define MZ_STATUS$V_ERR      0x00004000	/* Error */
#define MZ_STATUS$V_UNC      0x00002000	/* Command error */
#define MZ_STATUS$V_APT      0x00001000	/* Address Pass Through */
#define MZ_STATUS$V_DCAS     0x00000800	/* Device clear active state */
#define MZ_STATUS$V_MA       0x00000400	/* My address */
#define MZ_STATUS$V_SRQ      0x00000200	/* Service request */
#define MZ_STATUS$V_IFC      0x00000100	/* Interface clear */
#define MZ_STATUS$V_REM      0x00000080	/* Remote state */
#define MZ_STATUS$V_LLO      0x00000040	/* Local lockout */
#define MZ_STATUS$V_ATN      0x00000020	/* Attention */
#define MZ_STATUS$V_LPAS     0x00000010	/* Listen Pri addressed state */
#define MZ_STATUS$V_TPAS     0x00000008	/* Talk pri addressed state */
#define MZ_STATUS$V_LADS     0x00000004	/* addressed to listen */
#define MZ_STATUS$V_TADS     0x00000002	/* addressed to talk */
#define MZ_STATUS$V_ULPA     0x00000001	/* LSB last address */
/*
 *      prototypes
*/
#ifdef VXWORKS
STATUS mzDrv(unsigned char *pA16,int intLevVme, int intNumVme);
STATUS mzDevCreate (char *name,int flags,int timeOut,unsigned char primaryAdr,
                    unsigned char secAdr,char reosChar,char weosChar);
#endif

#endif /*incmzh*/


