/*
 *      add file for penTeck a/d boards
 *      the global addresses startup.cmd are:
 *      ptRamSizeBytes  number bytes in memory (2 bytes/samples
 *      ptWordClock     memory resolution size
 *      pt1Adr16        start 16 bit address
 *      pt1Adr32        start 32 bit address
 *
 * history:
 * 27sep92,pjp, ptIOCVMED16,32 changed from 1,2 to 16,32
 *      
*/
#ifndef INCpth
#define INCpth
/*
 *      device names
*/
#define PT1DEVNAME "/pt1"
#define PT2DEVNAME "/pt2"
/*
 * block size values determined by jumpers jp6-7 pg 41 of ptManual.
 * This defines the wordClock. To modify,
 * - see pg 41 of manual to update jumpers. read new values and place here,
 *   then recompile driver and all software... to get new min/max sizes.
 *  20ct96 - set to word clock = 8 to give min resolution with max size.
*/
#define PT_BLKSIZE_MIN		         16
#define PT_BLKSIZE_MAX           524280
#define PT_BLKSIZE_RES                8
#define PT_BLKSIZE_DIV_RES            1
/*
 *      ioctl requests
*/
#define ptIOCSETBUFLEN          0x1
#define ptIOCSETCLOCK           0x2
#define ptIOCSETCLOCKDIV        0x3
#define ptIOCSTART              0x4
#define ptIOCWAITINT            0x5
#define ptIOCSTAT               0x6
#define ptIOCSTATINT            0x7
#define ptIOCSTATCMD            0x8
#define ptIOCLOOPBANKSWAP       0x9
#define ptIOCDEBUG              0xa
#define ptIOCENAINT             0xb
#define ptIOCSTATTRG            0xc
#define ptIOCSTATINTMSK         0xd
#define ptIOCSETTRG             0xe
#define ptIOCRESET              0xf
#define ptIOCVMEDSIZE          0x10
#define ptIOCDMACSETUP         0x11
#define ptIOCDMACENABLE        0x13
#define ptIOCEXTPORT           0x14
#define ptIOCDISINT            0x15

/*
 *      ioctl values
*/
#define ptIOCCLOCKINT           0x1
#define ptIOCCLOCKEXT           0x2
#define ptIOCTRGIMD             0x1
#define ptIOCTRGSTART           0x2
#define ptIOCTRGIPP             0x3
#define ptIOCTRGGATED           0x4
/* note trgsmp is never passed to drv. It maps to trgimd. only
   ptProg differentiates between the two*/
#define ptIOCTRGSMP             0x5
#define ptIOCVMED16              16
#define ptIOCVMED32              32
/*
 *      the following addresses are byte offsets from the a16 base
*/
#define BLKSIZE_A_OFF           0x0
#define BLKSIZE_B_OFF           0x2
#define SAMPLE_CLK_DIV_OFF      0x4
#define CTC_CONTROL_OFF         0x6
#define INT_ID_VEC_OFF          0x10
#define STATUS_REG_OFF          0x10
#define INT_MASK_REG_OFF        0x12
#define CMD_REG_OFF             0x14
#define TRIGGER_REG_OFF         0x16
#define TAG_CMD_REG_OFF         0x1a
#define TAG_DATA_REG_OFF        0x1c
#define INT_STAT_REG_OFF        0x1e
#define FIFO_PORT_OFF           0x30
/* 
 *      masks for cmd word
*/
#define VME_PATH_MSK    0x80
#define VME_XFERS_MSK   0x40
#define MEM_ENABLE_MSK  0x20
#define RESET_MODE_MSK  0x10
#define BANK_LOAD_MSK   0x08
#define TOP_CNT_B_MSK   0x04
#define TOP_CNT_A_MSK   0x02
#define INP_OUT_MSK     0x01
/*
 *      macros to set/reset bits in command reg 0x14
*/
#define VME_PATH_ON(a)          ( (a) | ( VME_PATH_MSK))
#define VME_PATH_OFF(a)         ( (a) & (~VME_PATH_MSK)) 
#define VME_32_XFERS(a)         ( (a) | ( VME_XFERS_MSK))
#define VME_16_XFERS(a)         ( (a) & (~VME_XFERS_MSK))
#define MEM_ENABLE_ON(a)        ( (a) | ( MEM_ENABLE_MSK))
#define MEM_ENABLE_OFF(a)       ( (a) & (~MEM_ENABLE_MSK))
#define RESET_MODE_OFF(a)       ( (a) | ( RESET_MODE_MSK))
#define RESET_MODE_ON(a)        ( (a) & (~RESET_MODE_MSK)) 
#define BANK_LOAD_SET(a)        ( (a) | ( BANK_LOAD_MSK)) 
#define BANK_LOAD_REL(a)        ( (a) & (~BANK_LOAD_MSK))
#define TOP_CNT_B_ON(a)         ( (a) | ( TOP_CNT_B_MSK))
#define TOP_CNT_B_OFF(a)        ( (a) & (~TOP_CNT_B_MSK))
#define TOP_CNT_A_ON(a)         ( (a) | ( TOP_CNT_A_MSK))
#define TOP_CNT_A_OFF(a)        ( (a) & (~TOP_CNT_A_MSK))
#define B_INP_A_OUT(a)          ( (a) | ( INP_OUT_MSK))
#define A_INP_B_OUT(a)          ( (a) & (~INP_OUT_MSK))
/*
 *      macros to access status information in location  0x10 readonly
*/
#define OUT_DATA_RDY(a)  ( (a) & 0x80)
#define RDY_FOR_INP(a)   ( (a) & 0x40)
#define TOP_CNT_A(a)     ( (a) & 0x20)
#define TOP_CNT_B(a)     ( (a) & 0x10)
#define WRITE_ACT_A(a)   ( (a) & 0x08)
#define WRITE_ACT_B(a)   ( (a) & 0x04)
#define READ_ACT_A(a)    ( (a) & 0x02)
#define READ_ACT_B(a)    ( (a) & 0x01)
/*
 * bits to enable interrupt bits in word 0x12
*/
#define  PAR_PORT_ENA   0x80
#define  END_RUN_ENA    0x40
#define  START_RUN_ENA  0x20
#define  BANK_SWAP_ENA  0x10
#define  TOP_CNT_A_ENA  0x08
#define  TOP_CNT_B_ENA  0x04
#define  OVERRUN_ENA    0x02
#define  OVERLOAD_ENA   0x01
/*
 *      macros to access interrupt status info reg 0x1e
*/
#define  TOP_CNT_A_IACT(a)  ( (a) & 0x20)
#define  TOP_CNT_B_IACT(a)  ( (a) & 0x10)
#define  RUN_IACT(a)        ( (a) & 0x08)
#define  BANK_SWAP_IACTL(a) ( (a) & 0x04)
#define  OVERRUN_IACTL(a)   ( (a) & 0x02)
#define  OVERLOAD_IACTL(a)  ( (a) & 0x01)
/*
 *      trigger register 0x16   read/write
 *      masks
*/
#define RUNFF_AUTO_RESET_MSK    0x40
#define TRG_RISING_MSK          0x20
#define TRG_LEVEL_MSK           0x10
#define EXT_CLK_MSK             0x08
#define EXT_TRG_MSK             0x04
#define FORCE_RESET_RUN_FF_MSK  0x02
#define FORCE_SET_RUN_FF_MSK    0x01

/*
 *      macros to set bits...
*/
#define RUNFF_AUTO_RESET(a)  ( (a) | ( RUNFF_AUTO_RESET_MSK))   
#define RUNFF_NOAUTO_RESET(a)( (a) & (~RUNFF_AUTO_RESET_MSK))   
#define TRG_RISING_HI(a)     ( (a) | ( TRG_RISING_MSK)) 
#define TRG_FALLING_LOW(a)   ( (a) & (~TRG_RISING_MSK)) 
#define TRG_LEVEL(a)         ( (a) | ( TRG_LEVEL_MSK))
#define TRG_EDGE(a)          ( (a) & (~TRG_LEVEL_MSK))
#define EXTERNAL_CLK(a)      ( (a) | ( EXT_CLK_MSK))
#define INTERNAL_CLK(a)      ( (a) & (~EXT_CLK_MSK))
#define EXT_TRG_ENABLE(a)    ( (a) | ( EXT_TRG_MSK))
#define EXT_TRG_DISABLE(a)   ( (a) & (~EXT_TRG_MSK))
#define FORCE_RESET_RUN_FF(a) (((a)| (FORCE_RESET_RUN_FF_MSK)) & \
                                     (~FORCE_SET_RUN_FF_MSK))
#define FORCE_SET_RUN_FF(a)   (((a)| (FORCE_SET_RUN_FF_MSK))   & \
                                     (~FORCE_RESET_RUN_FF_MSK))
/*
 *      CTC CONTROL REGISTER
*/
#define CTC_SEL_BANKA   0x34
#define CTC_SEL_BANKB   0x74
#define CTC_SEL_CLKDIV  0xb4
#define CTC_SEL_CLKDIV10MHZ  0x90
#define CTC_READBACK_ALL 0xce


/*
 *      info gathered at last interrput
*/
typedef struct 
        { 
        int             timeout;        /* in clock ticks*/
        unsigned char   status;         /*  status register*/
        unsigned char   intstatus;      /* int status reg*/
        unsigned int    intcount;       /* count of interrputs*/
        int             late;           /* interrupt before wait call*/
        } PT_INTINFO;

#endif   /* INCpth*/
