#ifndef incRFIRCVLIBH
#define incRFIRCVLIBH

/*
modification history:
--------------------
x.x,01nov95,pjp - changed define fltid_450_140 to 430_50
*/
/*	defines     	 */

#define RFIRCV_DEV_1    KL2500_DEV_1 
#define RFIRCV_TIMEOUT  60*5

/*
 * defines for  rfiRcvFltTbl. The data is defined in rfiRcvDat.h
 * These are indices into the table
 * Note that the ID are indices into the table,
 *           the NUM'S are ID's + 1...
*/
#define	RFIRCV_FLTID_140_1000	0
#define	RFIRCV_FLTID_165_70	1
#define	RFIRCV_FLTID_235_90	2
#define RFIRCV_FLTID_330_140	3
#define RFIRCV_FLTID_430_50 	4
#define RFIRCV_FLTID_SW3_6	5
#define RFIRCV_FLTID_550_200	6
#define RFIRCV_FLTID_725_210    7 	
#define RFIRCV_FLTID_955_290	8 
#define RFIRCV_FLTID_SW4_4      9 	
#define RFIRCV_FLTID_1075_350	10
#define RFIRCV_FLTID_1325_250	11
#define RFIRCV_FLTID_1400_60	12
#define RFIRCV_FLTID_1765_670	13
#define RFIRCV_FLTID_BYPASS 	14
#define RFIRCV_FLTID_HI_FREQ    15
#define RFIRCV_FLTID_LAST   	15
/*
 * defines for  rfiRcvAntTbl. The data is defined in rfiRcvDat.h
 * These are indices into the table
*/
#define	RFIRCV_ANTID_850_1300	  0
#define	RFIRCV_ANTID_700_1400	  1
#define	RFIRCV_ANTID_DISC_CONE    1
#define	RFIRCV_ANTID_LOAD         2 
#define	RFIRCV_ANTID_LOW_DIODE    3 
#define	RFIRCV_ANTID_1000_12500   4 
#define	RFIRCV_ANTID_HI_FREQ      5 
#define RFIRCV_ANTID_LAST         5


/*
 * defines for  rfiRcvFOAmpTbl (fiber option amp ).The data is defined in 
 * rfiRcvDat.h These are indices into the table
*/
#define	RFIRCV_FOAMPID_500_1000	        0
#define	RFIRCV_FOAMPID_1500_1000	1
#define	RFIRCV_FOAMPID_UNUSED   	2
#define	RFIRCV_FOAMPID_BYPASS       3 
#define	RFIRCV_FOAMPID_HI_FREQ      4
#define RFIRCV_FOAMPID_LAST         5
/*
 *	typedefs
*/
/*
 * 	structure holding information for a given filter. 
 *	The table rfiRcvFilterTbl is an array of these structures.
*/

typedef struct {
	int	sw2Pos;		/* for this filter grp*/
	int	sw9Pos;		/* for this filter grp*/
	int	inpSwNum;	/* for input to filter*/
	int	inpSwPos;	/* for input to filter*/
	int	outSwNum;	/* for output from filter*/
	int	outSwPos;	/* for output from filter*/
	float	cenFrqMhz;	/* center freq filter mhz*/
	float	bwMhz;		/* bw filter mhz*/
	float	noiseFigDb; 	/* noise figure for filter*/
	float	ampGainDb; 	/* amp after filter*/
	float	ampNFDb; 	/* amp after filter*/
	int	defAntNum;	/* default ant number this filter*/
	int	defFOAmpNum;	/* default fiber optic amp number*/
	} RFIRCV_FILTER;
/*
 * structure defining the various antennas on the rfiRcv system
 * the array rfiRcvAntTlb is an array of these structures.
*/

typedef struct {
	int	sw1Pos;		/* position for switch 1*/
	float	cfrMhz;		/* center frequency attenna mhz*/	
	float   bwMhz;		/* bandwidth antenna mhz*/	
	float   gainDbi; 	/* antenna gain dbi*/	
	} RFIRCV_ANT;
/*
 * structure defining the fiber optic amplifier to use
 * the array rfiRcvFOAmpTbl is an array of these structures.
*/

typedef struct {
	int	sw10Pos;	/* position for sw 10*/
	int	sw11Pos;	/* position for sw 11*/
	float	cfrMhz;		/* center frequency amp mhz*/	
	float   bwMhz;		/* bandwidth amp mhz*/	
	float   gainDb; 	/* amp gain in db*/	
	float   nfDb;  		/* amp noise figure in db*/	
	} RFIRCV_FOAMP;

typedef struct {
	int	antNum;		/* 1-4, -1--> default for filter*/
	int	antDir;		/* deg east of north. -1 --> omni*/
	int	fltNum;		/* 1 to n. -1 use cfr to pick*/
	float   cfrMhz;		/* center freq in mhz. -1--> use flt cfr*/
	float	bwMhz;		/* bandwd mhz, -1--> use filter bw*/
	int	FOAmpNum;	/* fiber optic amp Num, -1-->use deffor filter*/
	float   rbw;		/* resolution bw in hz*/
	int	dbPerDiv;	/* db per division*/
	float   refLevDb;	/* reference level in db*/
	int	intTimeSecs;	/* integration time secs. 0=none maxHold,-1*/
	}  RFISETUP_REQ;
/*
 *	protoypes
*/
int     rfiRcvOpen(void);
STATUS  rfiRcvIAnt(int antId,RFIRCV_ANT *pantI);
STATUS  rfiRcvIFilter(int filterId,RFIRCV_FILTER *pfilterI);
STATUS  rfiRcvIFOAmp(int foampId,RFIRCV_FOAMP *pfoAmpI);

STATUS  rfiRcvCnfgAll(int fd, int idAnt,int idFilter,int idFOAmp);
int	rfiRcvSelFltNum(float freqMhz);
STATUS  rfiSetupReq(RFISETUP_REQ *preq);

#endif /*incRFIRCVLIBH*/



