/* spsBg - sps buffer generating routine
 *  %W%	%G%
modification history:
--------------------
1.1 23aug94 - put under sccs control
1.2 23aug94 - added baudLen for code  in usecs (.1 accuracy).
	    - added bandWidth for code in usecs. 
		Normally bandWidth and baudLen are just inverses.  
	    - Changed codeLen to codeLenSt .. code length in states where
	      a state is a duration with constant value. turns out to be
	      the number of sps locations for the code.
	    - added codeLenBaud.. number of bauds in the code. If 0, then
	      the code has no baud length, use the bandWidth

	      The code is input in units of the baud.
	      If the user specifies a baudLen of 0, then the code will be
	      input in usecs (with fractional values allowed) and the 
	      bandwidth will determine the effective baud.
    31jan95   go rid of powertail50mHz=2 define. no longer needed.
	      leave spsBh.use50Mhz but it no longer makes any difference
	      in the buffer created.
    03feb95   before today, cal followed noise, now cal precedes noise in 
	      cal window
    13nov99   maplength multipulse sequence
    21mar02  <pjp001> use50Mhz switched to a generate a separate rf pulse
			 for 50Mhzxmter. was int, now float..0. ->no, -1 yes, > 0 
			 usecs for rf window
    24mar04  <pjp003> beam turnon usecs 25-> 10 usecs
*/
#ifndef INCspsBgh
#define INCspsBgh
#ifdef	VXWORKS
#include	<vxWorks.h>
#else
#include	"vxWorksEm.h"
#endif
#include	<stdio.h>
#include	"spsLib.h"

/*      defines         */

#define CODEFILE        "/home/online/atm/etc/codes.dat"
#define CODE_NAME_USER  "user"
#define CODE_NAME_CLP   "codedlp"

/* bits used in sps to control xmiter   */

#define BN_RI_IPP       12 
#define B_50MHZ_RF       ( 1 <<  4)
#define B_PWR_START      ( 1 <<  5)
#define B_IF_SWITCH      ( 1 <<  6)
#define B_CODED_LONG_PUL ( 1 <<  7)
#define B_TX_PHASE      ( 1 <<  8)
#define B_TX_IPP        ( 1 <<  9)
#define B_TX_BEAM       ( 1 << 10)
#define B_TX_RF         ( 1 << 11)
#define B_RI_IPP        ( 1 << BN_RI_IPP)
#define B_RCV_CAL       ( 1 << 13)
#define B_RI_SMP0       ( 1 << 14)
#define B_RI_SMP1       ( 1 << 15)

/*	some maximum lengths	*/

#define MAX_LEN_CODE_NAME       20
#define MAX_LEN_CODE_ARRAY      100
#define MAX_LEN_MP_ARRAY        100
#define MAX_NUM_RCV_WIN         10

/* tx_ipp, rf_ipp length	*/
/* WARNING.. TXIPP_TO_BEAMON must be 0. or >= to IPP_PULSE_LEN
			 The code will not start beamon in the middle of ipp_pulse_len
*/
#define IPP_PULSE_LEN		 1.
#define TXIPP_TO_BEAMON		 2.

/* beam on to rfON	 */

#if FALSE

#define BEAM_TURNON_USECS       25.
#endif 

#define BEAM_TURNON_USECS       10.

/* turn cal on this many microsecs before start of cal window */

#define CAL_TURNON_USECS        35.

/* 50mhz pwr tail. min time rf off to beam off */

/*#define PWR_TAIL_50MHZ		 2.	 */

/* duty cycle of beam in fraction of time.  */

#define MAX_BEAM_DUTY_CYCLE     .06
#define MAX_BEAM_USECS          10000.

/* minimum ipp in usecs	 */

#define MIN_IPP_TX_USECS        750.

/* round to closest .1 usecs for numbers >= 0  */

#define TO_P1(a)        ((int)((a)*10. +.5))

/* coded long pulse code name	*/



/*	typedefs		*/

/*     code structure ... pdat baud len in usec, +/- is the phase  */

	typedef struct {
           char    name[MAX_LEN_CODE_NAME];/* name of code*/
           int     lenSt;          /* current len in sps Locations*/
           int     lenBauds;       /* current len in bauds Locations*/
	   double  lenUsecs;	   /* of entire code*/
	   float   baudLenUsec;	   /* .1 usec accuracy allowed*/
	   float   bandWidthMhz;   /* usually 1/code computed*/
           double  pdat[MAX_LEN_CODE_ARRAY];/*  code array in bauds*/
        } SPS_BG_CODE;

/*      multi pulse /rf structure
 *      	pdat holds start in units
*/
typedef struct {
        int     numPulses;              /* in sequence*/
        double  unitUsec;               /* multipulse unit*/
        int     pdat[MAX_LEN_MP_ARRAY]; /* hold the data        */
        }  SPS_BG_RFMP;
/*
 *      definition of Tx window
*/
typedef struct {
        double  beamWidthUsec;/* beam width in usecs. don't include rise time*/
			      /* do include any 50mhz power tail */
        SPS_BG_CODE  code;      /* code structure*/
        SPS_BG_RFMP   rfMp;     /* single/multi pulse info*/
        int     useSP0;         /* TRUE --> use sample win 0 b14, else b15*/
        int     smpTxWin;       /* TRUE/false*/
        double  extraTxSmpUsec;/* usecs beyond tx window to sample*/
		float	use50Mhz;/* 0 no, -1 yes, > 0 usecs for 50tx rf window*/
#if FALSE
		int	use50Mhz;	/* implies 2use pwr tail end of rf window*/	
#endif
        /*
         * following are returned
        */
        int     smpInTxWin;     /* routine will fill in*/
        double  txWinUsec;      /* duration txWindow*/
        double  dutyCycle;      /* in use fraction of total*/
        double  rfToEndTxWinUsec;/* time rf to end of tx window*/
        double  txIppToRfOnUsec; /* start ipp to 1st rf on */
        } SPS_TX_WIN;
/*
 *      definition of Rcv window
*/
typedef struct {
        int     useSP0;         /* true/false.*/
        double  winStartUsec;   /* relative to start 1st rf pulse*/
        int     numSamples;     /* entire window*/
        int     numSamplesCal;  /* cal precedes noise*/
        /*
        *  returns
        */
        double  winLenUsec;     /* duration window in usecs*/
 
        } SPS_RCV_WIN;
/*
 *      definition of spsBufGen structure
*/
typedef struct  {
        int         initialized;/* has been inited if = 1234*/
        double      ippUsec;    /* ipp in usecs*/
        double      gwUsec;     /* sampling gw in usecs*/
        SPS_TX_WIN  txWin;      /* transmit window*/
        int         numRcvWin;  /* number of rcv windows in use/allocated*/
        SPS_RCV_WIN prcvWins[MAX_NUM_RCV_WIN];/* ptr to array of rcv windows*/
        /*
         returned info
        */
        SPSBUF_DATA spsBuf;     /* holds the constructed buffer*/
        int         totSmpIpp;  /* total samples we take in ipp*/
        } SPS_BG;

/*      function prototypes     */
         
STATUS       spsBgBuild(SPS_BG *pspsBg,int *pmodified);
STATUS       spsBgCleanup(SPS_BG *pspsBg);
STATUS       spsBgDisp(FILE* fp,SPS_BG *pspsBg,char comment);
STATUS       spsBgInpExact(char* filename,SPS_BG    *pspsBg);
STATUS       spsBgInpFile(char* filename,SPS_BG    *pspsBg);
STATUS	     spsBgMake(SPS_BG *pspsBg);
void         spsBgOutFile(char* filename,SPS_BG *pspsBg,int query);

#endif	/*INCsps*/  
