/*
 *	include file for spslib.h
 *
 * history:
 * 20feb93 - created
*/
#ifndef INCspsLibh
#define INCspsLibh
#ifdef	VXWORKS
#include	<vxWorks.h>
#else
#include	<vxWorksEm.h>
#endif
#include	<stdio.h>
#include "sps.h"

/*	defines	 	*/

/*	sps Buffer definition. used by spsProg	*/

#define BUFID_MAX_LEN		       20

#define BUFCTRL_CLR_ST_TRG_REC_EOB      1
#define BUFCTRL_CLR_ST_TRG_REC_TRG      2
#define BUFCTRL_CLR_ST_IMD_REC_EOB      3
#define BUFCTRL_ST_TRG_REC_EOB          4
#define BUFCTRL_ST_TRG_REC_TRG          5
#define BUFCTRL_ST_EOB_REC_EOB          6
#define FINP_COMMENT    '#'

/*	for parsing spsProg keywords	*/

#define  PARSE_PRE_SPS   "sps_"

/*	for print command. the can be ored together	*/

#define  PR_ALL			0xf
#define  PR_HDR			0x1
#define  PR_ACTIVE_BUF          0x2
#define  PR_INUSE_LIST          0x4
#define  PR_FREE_LIST           0x8

/*	typedefs	*/

/*
 *      external spsBuffers for spsProg, etc. defined via SPSBUF_INFO which
 *	holds the setup info and the state and duration buffers
*/
typedef struct {
        char    bufId[BUFID_MAX_LEN];
        int     bufLen;
        int     bufOff;         /* Where it's actually loaded*/

        int     mClkDiv;        /* master clock divisor. n where clk=20/(n+1)*/
        int     sClk0Div;       /* smp  clock0  divisor. n where clk=20/(n+1)*/
        int     sClk1Div;       /* smp  clock1  divisor. n where clk=20/(n+1)*/
        int     fixedClk;       /* true -> fixed clock*/
        int     bufCtrl;        /* how to start and recirculate buffer
                                   1 - clrsps,trig start, recirc EOB
                                   2 - clrsps,trig start, recirc trig
                                   3 - clrsps,imd  start, recirc EOB
                                   4 -        trig start, recirc EOB
                                   5 -        trig start, recirc trig
                                   6 -        EOB  start, recirc EOB
                                */
        int     trigToUse;      /*
                                   1 - 10sec tick
                                   2 - external tick
                                   3 - 1sec tick
                                */
        } SPSBUF_INFO;
/*
 *	hold state/duration memory in cpu
*/
typedef struct {
	int	maxStates;	/* that either array can hold*/ 
	int	curStates;	/* that are in array*/ 
	int    *pstate;		/* ptr to state info*/
	int    *pdur;		/* ptr to duration info*/
	} SPSBUF_DATA;		

/*
 * 	prototypes for functions
*/
STATUS  spsDAChirp(int numStates,double mClockPeriod,int mClocksPerState,
         	   double (*freqFunc)(),double phaseBalDeg,int* pstateBuf,
         	   int* pdurBuf);
STATUS	spsInpFile(char* filename,SPSBUF_INFO* pbufI,int* pstate,int* pdur);
void    spsPrintBuf(int buflen,int off,int *pstate,int *pdur);
void    spsPrintStat(FILE *fptr,int status);
STATUS  spsRtgBld(double ipp, double gw, double gd,int numGw,int ippB,
                        int useSP0,int *pstate,int *pdur,int *pnumStates);
void    spsScmdActive(int val,int *pcmd);
void    spsScmdClear(int val,int *pcmd);
STATUS  spsScmdInt(char *cval,int *pcmd);
STATUS  spsScmdStartOn(char *cval,int *pcmd);
STATUS  spsScmdSysClk(char *cval,int *pcmd);
STATUS  spsScmdTrig(char *cval,int *pcmd);
/*
 *	vxWorks only routines
*/
#ifdef	VXWORKS

STATUS spsActBufVw(int fd,int bufLen,int bufOff,int isAbsOff);
STATUS  spsBufSwDisVw(int fd);
STATUS  spsBufSwEnaVw(int fd);
STATUS  spsClearVw(int fd);
STATUS  spsDefStateVw(int fd);
int     spsGetStatVw(int fd);
void    spsGetStateVw(int fd, SPS_STATE *pspsState);
STATUS  spsLdBufVw(int fd,int buflen,int off,int *pstate,int *pdur);
STATUS spsLdBufStVw(int fd,SPS_STATE *pspsState,int *pstates,int *pdur);
STATUS  spsLdBufSwVw(int fd,int numbufs,int bufOffSet,BUFSW_INFO *pbufSw_I);
STATUS  spsLdClocksVw(int fd,int mClkDiv,int sClk0Div,int sClk1Div);
STATUS  spsLdOffLenVw(int fd,int off,int absOff,int Len);
STATUS  spsLdStateVw(int fd,SPS_STATE *pspsState);
void    spsPrAllReg(FILE *fptr,SPS_REG *preq,SPS_REG *pintf,SPS_REG *pact);
STATUS  spsRdAllRegVw(int fd,SPS_REG *preqReg,SPS_REG *pintfReg,
 		SPS_REG *pactReg);
STATUS  spsRdBufAbsVw(int fd,int buflen,int off,int *pstate,int *pdur);
STATUS  spsRdBufRelVw(int fd,int buflen,int off,int *pstate,int *pdur);
STATUS spsRtgEmVw(int fd,double ipp, double gw, double gd,int numGw,
                  int tick,int startOnTick,int fixedClk,int useSP0);
STATUS  spsSendCmdVw(int fd,int cmd);

#endif /* vxWorks*/

#endif	/*INCspsLibh*/  
