#ifndef INCterPosh
#define INCterPosh
#ifdef VXWORKS
#include	<vxWorks.h>
#else
#include	<vxWorksEm.h>
#endif
#include	<math.h>
#include	<stdio.h>
/*
 * tertiary position include file
 * nomenclature/abbreviations
 * 
 * Dc dome 	centerline coordinate system
 * Foc,F  	focus coordinate system
 * oXX      starting coordinates (0 offsets all points) in XX coordinate
 *		    system. 
 * M        matrix (3x3) 
 * V        vector
*/
/*
 * dome center line coordinate system right handed...
 *
 *
 *       z up  vertically
 *            ^
 *            |  /y toward the stair case
 *            | /
 *            |-----> x uphill on the azimuth arm
*/
/*
 *  vectors -  2-d vectors (x,z) but store as col vectors x,z,tr
 *             where tr is unity. It is used for translations..
 *
 *  matrix   - 2-d but 3x3  store in row major order (increment across
 *             colums most rapidly.
 *            ( x11    z12  dx )
 *            ( x21    z22  dz )
 *            (   0      0  1  )
 *            The last column and last row are used for translations
*/

/* tertiary connection points
 *
 *   P1 - top vertical tiedown .. fixed
 *   P2 - back horizontal tiedown fixed.
 *   P3 - where horizontal,vertical tiedowns meet.
 *   P4 - tail gunner fixed point
 *   P5 - tail gunner moveable point.
*/


/*	defines		*/



/*	macros		*/

#define TER_DIST(p1,p2) (sqrt( ((p1)->x - (p2)->x) * ((p1)->x - (p2)->x)  +\
						   ((p1)->z - (p2)->z) * ((p1)->z - (p2)->z)  ))

#define TER_MAG_V(pv) (sqrt( (pv)->x * (pv)->x + (pv)->z*(pv)->z))
 
/*   law of cosines  theta = acos (a*a + b*b - c*c)/(2ab)       */
 
#define TER_LAW_COS(a,b,c) ( acos( ((a)*(a) + (b)*(b) - (c)*(c))/(2.*(a)*(b))))



/*	typedefs	*/

/*  hold a vector. tr is for translations   */
 
typedef struct  {
    double      x;
    double      z;
    double      tr; /* translation*/
    } TER_VEC;

/*	connection points for tertiary */
typedef struct {
    TER_VEC     p1;
    TER_VEC     p2;
    TER_VEC     p3;
    TER_VEC     p4;
    TER_VEC     p5;
    } TER_COORDS;

/*
 *  strokes, distance for each actuator
*/
typedef struct {
    double  ver;        /* vertical actuator*/
    double  hor;        /* horizontal actuator*/
    double  tail;       /* tail gunner*/
    } TER_D_ACT;
/*
 * distances between fixed points
*/
typedef struct {
    /*
     * fixed distances
    */
    double  p12;        /* vertical actuator*/
    double  p53;        /* horizontal actuator*/
    } TER_D_FIXED;
/*
 * postion info
*/
typedef struct {
    TER_COORDS  oDc;    /* origin dome centerline*/
    TER_COORDS  oF;     /* origin focus coords*/
    double      dcToFoc_M[9];/* dome cenline to focus coordinates*/
    double      focToDc_M[9];/* focus to dome center coords*/
 
    TER_D_FIXED fixed_d;    /* fixed distances*/
    double      xf;     /* x offset focus coords*/
    double      zf;     /* z offset focus coords*/
    double      dphiRd;     /* phi offset focus coords rd*/
    double      thetaRd;    /* theta angle radians*/
    } TER_POS_INFO;

#endif
