/*
 *	include file for vp1 particulars (adresses etc)
 *      vicreg.h has vic chip addresses, sbp.h has sbp brd junk 
*/
#ifndef INCvp1h
#define INCvp1h
/*
 * 	mailbox registers offsets from start of a16 base
*/
#define VP1_ICR0_OFF		0x01
#define VP1_ICR1_OFF		0x03
#define VP1_ICR2_OFF		0x05
#define VP1_ICR3_OFF		0x07
#define VP1_ICR4_OFF		0x09
#define VP1_ICR5_OFF		0x0b
#define VP1_ICR6_OFF		0x0d
#define VP1_ICR7_OFF		0x0f
/*
 *	set/clear global mailbox registers
*/
#define VP1_ICGS0_CLR_OFF 	0x10
#define VP1_ICGS0_SET_OFF 	0x11
#define VP1_ICGS1_CLR_OFF 	0x12
#define VP1_ICGS1_SET_OFF 	0x13
#define VP1_ICGS2_CLR_OFF 	0x14
#define VP1_ICGS2_SET_OFF 	0x15
#define VP1_ICGS3_CLR_OFF 	0x16
#define VP1_ICGS3_SET_OFF 	0x17
/*
 *	set/clear local mailbox registers
*/
#define VP1_ICMS0_CLR_OFF 	0x20
#define VP1_ICMS0_SET_OFF 	0x21 
#define VP1_ICMS1_CLR_OFF 	0x22
#define VP1_ICMS1_SET_OFF 	0x23 
#define VP1_ICMS2_CLR_OFF 	0x24
#define VP1_ICMS2_SET_OFF 	0x21 
#define VP1_ICMS3_CLR_OFF 	0x20
#define VP1_ICMS3_SET_OFF 	0x21 
/*
 *	when downloading. lowermost two bits tell what it is
*/
#define VP1_DOWNLOAD_MASK	0x03
#define VP1_DOWNLOAD_LAST_PAIR  0x00
#define VP1_DOWNLOAD_PIRQ1_LVL  0x01
#define VP1_DOWNLOAD_PIRQ2_LVL  0x02
#define VP1_DOWNLOAD_PAIR       0x03

/* 
 *	vic register definitions
*/

#define VIC_VME_INTR_INT_CTL_REG	0x03
/*
 *	vmeBus interrupt control registers for irq1-irq7
 *	enable/disable irq1-7 and set level that they interrupt local board
*/
#define VIC_VME_INT_CTL_REG_1     	0x07	 
#define VIC_VME_INT_CTL_REG_2     	0x0b	 
#define VIC_VME_INT_CTL_REG_3     	0x0f	 
#define VIC_VME_INT_CTL_REG_4     	0x13	 
#define VIC_VME_INT_CTL_REG_5     	0x17	 
#define VIC_VME_INT_CTL_REG_6     	0x1b	 
#define VIC_VME_INT_CTL_REG_7     	0x1f	 

#define VIC_DMA_STAT_INT_CTL_REG	0x23
/*
 *	local bus interrupt control registers 
 *	enable/disable local board ints.
*/
#define VIC_LOC_INT_CTL_REG_1	0x27
#define VIC_LOC_INT_CTL_REG_2	0x2b
#define VIC_LOC_INT_CTL_REG_3	0x2f
#define VIC_LOC_INT_CTL_REG_4	0x33
#define VIC_LOC_INT_CTL_REG_5	0x37
#define VIC_LOC_INT_CTL_REG_6	0x3b
#define VIC_LOC_INT_CTL_REG_7	0x3f
/*
 *	other interup control regs 
*/
#define VIC_ICGS_INT_CTL_REG	0x43
#define VIC_ICMS_INT_CTL_REG	0x47
#define VIC_ERRGRP_INT_CTL_REG	0x4b
/*
 *	base registers for misc interrupt.s
 *	specifies the vector returned on interrup acknowledge cycle
*/
#define VIC_ICGS_INT_VEC_BASE_REG   0x4f
#define VIC_ICMS_INT_VEC_BASE_REG   0x53 
#define VIC_LOC_INT_VEC_BASE_REG    0x57
#define VIC_ERRGRP_INT_VEC_BASE_REG 0x57

#define VIC_ICMS_REG		0x5f
#define VIC_ICR_REG_0		0x63
#define VIC_ICR_REG_1		0x67
#define VIC_ICR_REG_2		0x6b
#define VIC_ICR_REG_3		0x6f
#define VIC_ICR_REG_4		0x73
/*			vic id register		*/
#define VIC_ICR_REG_5		0x77
/*			loc brd status		*/
#define VIC_ICR_REG_6		0x7b
/*			semaphores reg0-4	*/
#define VIC_ICR_REG_7		0x7f
/*
 *	vme interrupts, generate interrupt, and vectors returned on int
 *	ack cycle
*/
/*	macro to generate a vmebus int from vic 1-7
*/
#define VIC_GEN_VMEINT(n) ((1 << (n)) | 1)
#define	VIC_VME_INT_REQ		0x83
#define VIC_VME_INT_VEC_IRQ1	0x87
#define VIC_VME_INT_VEC_IRQ2	0x8b
#define VIC_VME_INT_VEC_IRQ3	0x8f
#define VIC_VME_INT_VEC_IRQ4	0x93
#define VIC_VME_INT_VEC_IRQ5	0x97
#define VIC_VME_INT_VEC_IRQ6	0x9b
#define VIC_VME_INT_VEC_IRQ7	0x9f

#define VIC_BUS_TIMEOUTS	0xa3
#define VIC_INTF_CONF_REG_1	0xa7
#define VIC_BLK_XFER_DEF_REG	0xab
#define VIC_INTF_CONF_REG_2	0xaf
#define VIC_ARB_REQ_CONF_REG	0xb3
#define VIC_AM_SRC_REG		0xb7
#define VIC_BUS_ERR_STAT_REG	0xbb
#define VIC_DMA_STAT_REG	0xbf
#define VIC_SLV0_SEL_CTL_REG_0	0xc3
#define VIC_SLV0_SEL_CTL_REG_1	0xc7
#define VIC_SLV1_SEL_CTL_REG_0	0xcb
#define VIC_SLV1_SEL_CTL_REG_1	0xcf
#define VIC_REL_CTL_REG		0xd3
#define VIC_BLK_XFER_CTL_REG	0xd7
#define VIC_BLK_XFER_LEN_MSB  	0xdb
#define VIC_BLK_XFER_LEN_LSB  	0xdf
#define VIC_SYS_RST_REG  	0xe3

#endif  /* INCvp1h */
