#ifndef INCxbLibh
#define INCxbLibh

#include	<scsiLib.h>
#include	<stdio.h>
#include	"mtio.h"
#include	"aoErrNum.h"
/*
 * much of this was stolen,borrowed, etc from:
 * The Johns Hopkins University/Applied Physics Laboratory
 *	Programmer:	PR Bade
 *	Date Created:	8/3/90
*/
/* mod pjp001... left out adtl_sense_code_qual in SENSE_DATA
 *               SENSE_DATA--> XB_SENSE_DATA
 *               added XB_SENSE_MODE
 *     pjp002    exabytes status... change reserved from 21 to 20 to fit in 32
 *		 bits
*/

/**********************************************************/
/*     Constant Definitions for the Exabyte Module        */
/**********************************************************/

/* Exabyte Status Codes
    EXB_SCSI_STAT_OTHER     0x0a is scsi_stat:
SCSI_STATUS_CONDITION_MET  
SCSI_STATUS_INTMED_GOOD 
SCSI_STATUS_INTMED_COND_MET 
SCSI_STATUS_RESERV_CONFLICT
SCSI_STATUS_QUEUE_FULL  
*/
/*
 * EXABYTE_STATUS error_code codes. The 1st 16 come from the request sense
 * sense key, the others are filled in by xbDrv.
*/
 

#define EXB_NO_ERROR		0x00
#define EXB_NOT_SUPPORTED1	0x01    /* command rejected, device busy */
#define EXB_NOT_READY		0x02
#define EXB_MEDIUM_ERROR	0x03
#define EXB_HARDWARE_ERROR	0x04
#define EXB_ILLEGAL_REQUEST	0x05
#define EXB_UNIT_ATTENTION	0x06
#define EXB_DATA_PROTECT	0x07
#define EXB_BLANK_CHECK	        0x08
#define EXB_EXABYTE_UNIQUE	0x09
#define EXB_NOT_SUPPORTED2	0x0a    /* command rejected, device busy */
#define EXB_ABORTED_COMMAND	0x0B
#define EXB_NOT_SUPPORTED3	0x0c    /* command rejected, device busy */
#define EXB_VOLUME_OVERFLOW	0x0D
#define EXB_NOT_SUPPORTED4	0x0e    /* command rejected, device busy */
#define EXB_NOT_SUPPORTED5	0x0f    /* command rejected, device busy */
#define EXB_BUSY		0x10        /* command rejected, device busy */
#define EXB_SCSI_STAT_OTHER     0x11	    /* scsi_stat,not chkcond,ok,busy*/
#define EXB_SEE_GLSENSE         0x12	    /* get global sense for error*/
#define EXB_SCSILIB_ERROR  	0x13	    /* from scsiIoctl*/
#define EXB_CHKCOND_IN_CHKCOND	0x14    /* check condition inside check cond*/

/* Exabyte Control Function Codes used when building mtop struct*/

#define TEST_READY	0x1
#define REWIND		0x2
#define SPACE		0x3
#define LOAD		0x4
#define UNLOAD		0x5
#define WRITE_FILEMARK	0x6

/*Exabyte specific ioctl's.  */  

#define XBIOCSENSEDATA  _IOR(x, 0,XB_SENSE_DATA)  /* return sense data */
#define XBIOCSENSEMODE  _IOR(x, 1,XB_SENSE_MODE)  /* return sense data */
#define XBIOCLOCK       _IOW(x, 2, int)                 /* lock/unlock door */
#define XBIOCINQ        _IOR(x, 3, struct xb_inquiry)   /* return inquiry data*/
#define XBIOCGLSENSEDATA _IOR(x,4,XB_SENSE_DATA)  /* return global sensedata*/
#define XBIOCSTAT       _IOR(x, 5,XB_STAT)  /* return stat data */
#define XBIOCLSTAT      _IOR(x, 6,XB_STAT)  /* return long stat data */

/*      macros 		*/

/*  
 *  move error struct to errno with upper 16 bits set to aoM_xbErr
*/
#define EXB_STRUCT_TO_ERRNO(pa,errnum)  \
	errnum=(((*(unsigned int *)(pa)) & 0xffff)|aoM_xbErr_BASE)

/*   move errno into error_struct	*/

#define EXB_ERRNO_TO_STRUCT(errnum,pa) \
	   *(unsigned int *)(pa) = ((errnum) & 0xffff)

/*   return true of errno hols exabyte status struct	*/

#define EXB_ERRNO_IS_STRUCT(errnum) (((errnum) & 0xffff0000)==aoM_xbErr)

/*	XB_SENSE_DATA	*/

typedef struct
{
  unsigned	info_valid	:1,	/* indicates bytes 3 thru 6 are valid */
		error_class	:3, 	/* always 7 */
		error_code	:4;	/* 0 current cmd, 1--> deferred cmd */
  unsigned char	seg_num;		/* zero unless check is result of copy*/
  unsigned	filemark	:1, 	/* read a filemark */
		end_of_media	:1,	/* past Logical EOT or BOT?? */
	 	ili		:1,	/* incorrect logical blk length req. */
		reserved	:1,
		sense_key	:4;	/* sense key gives error indication */
  unsigned char	info_bytes[4];		/*if info_valid, unprocesed bytes/blks*/
  unsigned char adtl_sense_lgth,	/* additional sense bytes..21*/ 
		reserved8,
		reserved9,
		reserved10,
		ovr_under_run_cntr,     /* times xb overrun/underrun. couldn't
					   keep it streaming*/
		adtl_sense_code,	/* additional info on sensekey app. D*/
		adtl_sense_code_qual,	/* additional info on sensekey app. D*/
		reserved14,
		reserved15,
		dataErrCnt_hb,		/* high byte of read/rewrite err cnt */
		dataErrCnt_mb,		/* middle byte of error count */
		dataErrCnt_lb;		/* low byte of error count */
  unsigned	power_fail	    :1,
		bus_parity_err      :1,
		buffer_parity_err   :1,
		media_err  	    :1,
		error_cnt_overflow  :1,	/* retry counter has overflowed*/
		tape_motion_err     :1,
		tape_not_present    :1,
		bot		    :1,	/* beginning of tape */
		rsvd1               :1,
		tape_mark_detect_err:1,
		write_protect	    :1,
		filemark_err  	    :1,
		underrun_err  	    :1,
		media_write_err     :1, /* max num retries exceeded*/
		servo_err  	    :1,
		formatter_err  	    :1,
		rsvd2               :2,
	        rvs_retry_req       :1, /*had to do retries to move tape */
		cleaned		    :1, /*1 after cleaning.reqSense clears*/
	        needs_cleaning	    :1, /*1 when need cleaning*/
		peot     	    :1,
		splice_err1	    :1,
		splice_err2	    :1;
  unsigned char reserved22,
		remain_tape_hb,		/* high byte of remaining tape */
		remain_tape_mb,		/* middle byte of remaining tape */
		remain_tape_lb;		/* low byte of remaining tape */
  unsigned char trk_retry_cnt,		/* tracking retry counter*/
	        rd_wr_retry_cnt,	/* read write retry count*/
		fault_syst_code;	/*to dbg modeselect errors.appE*/
} XB_SENSE_DATA;

/*	XB_SENSE_MODE	*/

typedef	struct
{
	unsigned char	length,		   /* sense data length excl this byte*/
		 	media_type;	   
	unsigned	write_prot     :1, /* 1--> write protected*/
			buffered_mode  :3, /* 001-> buffered mode*/
			speed	       :4; /* always 0*/
	unsigned char	blk_desc_len,	   /* block descriptor len=8*/
	                density,	   /* always zero*/
			num_blks_hb,       /* high byte # blocks in media*/
			num_blks_mb,       /* mid  byte # blocks in media*/
			num_blks_lb,       /* low  byte # blocks in media*/
	                reserved1,	   /* not used*/
	 		block_len_hb, 	   /* hbyte blck length 0--> variable*/	
	 		block_len_mb, 	   /* mbyteblock length 0--> variable*/	
	 		block_len_lb; 	   /* lbyteblock length 0--> variable*/	
	unsigned	cart_type      :1, /* 0=p6,1=p5 */
			reserved2      :1,
			no_disconnect  :1, /* 1--> no scsi disc during i/o*/ 
			reserved3      :1,
			no_busy_enable :1, /* 1-->busy status not reported*/
			even_byte_dis  :1,
			parity_enable  :1,
			no_auto_load   :1,
			reserved4      :7,
			use_as_P5_cart :1;
	unsigned char  	motion_thresh,	   /* in kybtes before i/o to tape*/
			recon_thresh,	   /* reconnect threshold in kb*/
			gap_thresh;	   /* pad blocks before stopping*/
} XB_SENSE_MODE;

			
			
/**********************************************************/
/*      Type Definitions for the Exabyte Functions        */
/**********************************************************/


typedef struct
{
  unsigned      rsvd                :16,/* aoM_xbErr value goes here*/
		sense_data_valid    :1, /* indicates if bits 1-7 are valid */
  		tape_not_present    :1,
		write_protect	    :1,
		end_of_media	    :1,	/* Logical EOT or BOT */
		bot		    :1,	/* beginning of tape */
		peot     	    :1,
  		filemark	    :1, /* read a filemark */
		ili		    :1,	/* incorrect logical blk length req. */
		reserved	    :2,/* pjp002*/
	        glSenseAvail	    :1, /* global sense available this cmd*/
		error_code	    :5;	/* see EXB_.. errors above.*/
}EXABYTE_STATUS;
/*
 *  structure for mt stat return
*/
typedef struct {
	int	fileCnt;		/* file cnt*/
	int	blkCnt;		        /* rec cnt*/
	int	blksLeftTape;	 	/* number 1k blks till leot*/
	int	lastRecLen;		/* rec length last write*/
	} XB_STAT;			/* for xb stat call*/

/*	prototypes	*/

char    *xbErrCodeStr(int errorCode);
void	xbPrExtdErr(FILE* fp,XB_SENSE_DATA* pxbsense);
void	xbPrDSense(FILE* fp,XB_SENSE_DATA* pxbsense);
void    xbPrExbStat(FILE* fp,EXABYTE_STATUS  *pst);

#endif	/*INCxbLibh*/

