//#define DEBUG

#include <vector>

const int DataSize    = 1024*1024;
const int HeaderSize  = 4096;

template<class T>
int GetTapeData(int fd, int channel, long numsamples, std::vector<T> &c_samples) {
//  fd          device that contains tape data 
//  channel     the channel (0-15) from which you wish to extract
//  numsamples  the number of samples you wish to extract
//  c_samples   a std vector of type T onto which numsamples will be pushed.
//  return val  0 == success, 1 == failure

    const int BufSize     = HeaderSize + DataSize;
    static unsigned short buf[BufSize/2];       // BufSize is in bytes
    const int BufWords = BufSize/2;
    static int buf_i=0;                         // will index an array of 16 bit words (short ints)

    size_t readbytes;
    unsigned short dataword;
    long samplecount=0, i=0, j=0;

#ifdef DEBUG
    fprintf(stderr, "entering GetTapeData....\n");
#endif

    // Keep reading BufSize bytes worth of data until we have
    // numsamples 2 bit samples.  Note that numsamples might be,
    // but does not have to be, satisfied with one read.
#ifdef DEBUG
    fprintf(stderr, "offset = %ld bufsize = %d buf_i = %d\n", lseek(fd, 0L,  SEEK_CUR), BufSize, buf_i);
#endif
    while (samplecount < numsamples) {
    
        // read the next buffer if need be
        if (buf_i == 0) {
#ifdef DEBUG
            fprintf(stderr, "Reading....\n");
#endif
            if ((readbytes = read(fd, buf, BufSize)) != BufSize) break;     // fatal - bail
            buf_i = HeaderSize;         // position buf_i on the first byte (word) past the header
#ifdef DEBUG
            fprintf(stderr, "readbytes = %ld\n", readbytes);
#endif
        }


        for (;
             buf_i < BufWords && samplecount < numsamples;
             buf_i += 1, j += 2, samplecount += 1) {

            // isolate the desired channel
            dataword = buf[buf_i] >> channel*2;
            dataword &= (unsigned short)0x0003;

            // push the complex sample onto the vector
            c_samples.push_back(dataword & (unsigned short)0x0001 ? 1 : -1);
            c_samples.push_back(dataword & (unsigned short)0x0002 ? 1 : -1);
        }

        if (buf_i >= BufWords) {
            buf_i = 0;    // we'll need to read the next buffer next time 'round
#ifdef DEBUG
            fprintf(stderr, "buf_i zeroed\n");
#endif
        }
    }

    if(samplecount < numsamples) {
#ifdef DEBUG
        fprintf(stderr, "readbytes = %d\n", readbytes);
        fprintf(stderr, "samplecount = %d\n", samplecount);
#endif
        return(1);
    } else {
#ifdef DEBUG
           fprintf(stderr, "samplecount = %d\n", samplecount);
#endif
        return(0);
    }
}

