module options

   use healpix_types

   implicit none

   CHARACTER(LEN=filenamelen) :: input_file = ''
   CHARACTER(LEN=filenamelen) :: output_file = ''
   CHARACTER(LEN=filenamelen) :: title = ''
   CHARACTER(LEN=3)   :: projection = 'MOL'
   REAL(SP)           :: usermax = 1.1e30
   REAL(SP)           :: usermin = -1.1e30
   REAL(SP)           :: lon0 = 0.0
   REAL(SP)           :: lat0 = 0.0
   REAL(SP)           :: gridresn = 2.0
   INTEGER(I4B)       :: color_table = 5
   INTEGER(I4B)       :: signal = 1         ! I/intensity measurement
   INTEGER(I4B)       :: xsize  = 800
   LOGICAL(LGT)       :: logflg = .false.
   LOGICAL(LGT)       :: bar = .false.
   LOGICAL(LGT)       :: ashflag = .false.
   REAL(SP)           :: offset = 0.0
   REAL(SP)           :: factor = 1.0

   private
   public :: input_file, output_file, &
        & title, projection, usermax, usermin, &
        & lon0, lat0, gridresn, color_table, &
        & signal, xsize, logflg, bar, &
        & ashflag, offset, factor
   public :: parseOptions

contains

   subroutine parseOptions ()

     USE extension, ONLY : getArgument, nArguments
      implicit none

      integer(I4B)       :: i,n
      character(len=4)   :: opt
      character(len=filenamelen) :: arg
      LOGICAL(LGT)       :: bad, ok
      
      n = nArguments()

      if (n < 4) then
         print *, 'usage: MAP2GIF [-inp input_file] [-out output_file]'
         print *, '               [-max usermax] [-min usermin]'
         print *, '               [-add offset] [-mul factor]'
         print *, '               [-pro projection] [-res gridresn]'
         print *, '               [-lon lon0] [-lat lat0]'
         print *, '               [-col color_table] [-sig signal]'
         print *, '               [-xsz xsize] [-log logflg]'
         print *, '               [-ash ashflg]'
         print *, '               [-bar add color bar] [-ttl title]'
         print *, '               [-hlp]'
         stop 1
      end if


      do i = 1,n,2
         call getArgument(i,opt)
         if (opt == '-hlp') then
            print 1, input_file,output_file,color_table,usermax,usermin,&
                 & signal,xsize,logflg,bar,title
            stop 1
         end if
         if (i == n) then
            print '("option ",a2," has no argument")', opt
            stop 2
         end if
         call getArgument(i+1,arg)
         select case (opt)
         case ('-inp')
            input_file = trim(arg)
         case ('-out')
            output_file = trim(arg)
         case ('-col')
            read (arg,*) color_table
         case('-sig')
            read (arg,*) signal
         case ('-xsz') 
            read (arg,*) xsize
         case ('-log')
            read (arg,*) logflg
         case ('-ash')
            read (arg,*) ashflag
         case ('-bar')
            read (arg,*) bar
         case ('-ttl')
            title = trim(arg)
         case ('-max')
            read (arg,*) usermax
         case ('-min')
            read (arg,*) usermin
         case ('-add')
            read (arg,*) offset
         case ('-mul')
            read (arg,*) factor
         case ('-pro')
            projection = trim(arg)
         case ('-lat')
            read (arg,*) lat0
         case ('-lon')
            read (arg,*) lon0
         case ('-res')
            read (arg,*) gridresn
         case default
            print '("unknown option ",a4," ignored")', opt
         end select
      end do

      inquire(file=input_file, exist=ok) ! verify input file 
      if ( .not. ok ) then
         print *,TRIM(input_file)//' not found.'
         stop
      endif

      inquire(file=output_file, exist=bad)   ! verify output file 
      if (bad) then
         print *,TRIM(output_file)//' already exists.'
         stop
      endif

      if (usermax == usermin) then
         print *, 'it is silly to set the maximum and the minimum'
         print *, 'to the same value, is it not?'
         stop
      end if

      if (ashflag .and. logflg) then
         print *, '-log and -ash cannot both be true!'
         stop
      end if

1     format(/,&
           & " -inp [] ",A,/,&
           & " -out [] ",A/,&
           & " -col [] ",I1,/,&
           & " -max [] ",e10.2,/,&
           & " -min [] ",e10.2,/,&
           & " -add [] ",e10.2,/,&
           & " -mul [] ",e10.2,/,&
           & " -pro [] ",A,/,&
           & " -lon [] ",e10.2,/,&
           & " -lat [] ",e10.2,/,&
           & " -res [] ",e10.2,/,&
           & " -sig [] ",I1,/,&
           & " -xsz [] ",I6,/,&
           & " -log [] ",L1,/,&
           & " -ash [] ",L1,/,&
           & " -bar [] ",L1,/,&
           & " -ttl [] ",A,/)

   end subroutine parseOptions

end module options
