module extension
  !
  ! defines in F90 some C commands
  ! These extensions are not completely standard in all F90/95 compilers
  !
  ! getEnvironment   : emulates getenv
  ! getArgument      : emulates getarg
  ! nArguments       : emulates iargc
  !
  ! written by Eric Hivon, Nov 2001
  !

#ifdef NAG
  USE f90_unix, ONLY : iargc, getarg
#endif
!VF  USE dflib, ONLY : nargs, getarg

  USE healpix_types, ONLY : I4B
  IMPLICIT none

#ifndef NAG
interface
  function iargc()
    integer iargc
  end function

  subroutine getarg (num, res)
    integer, intent(in) :: num
    character(len=*), intent(out) :: res
  end subroutine
end interface
#endif

  integer(kind=I4B), private :: arg_shift = 0
!VF  integer(kind=I4B), private :: arg_shift = 1

  private
  public :: getEnvironment, getArgument, nArguments

  contains
    
    ! ===========================================================
    function nArguments() result(narg)
      ! ===========================================================
      integer(kind=I4B) :: narg
      ! ===========================================================
      
      narg = iargc() - arg_shift
!VF      narg = nargs() - arg_shift

      return
    end function nArguments
    ! ===========================================================
    subroutine getEnvironment(name, value)
      ! ===========================================================
      character(len=*), intent(in) :: name
      character(len=*), intent(out) :: value
      integer(kind=I4B) :: inull, lnstr
!       character(len=200) :: name1
      ! ===========================================================
      ! call C routine after adding a trailing NULL to input
      value = ""
      call cgetenvironment(trim(adjustl(name))//char(0), value)
      ! remove trailing NULL (\0) created by C routine on output
      lnstr = len(value)
      inull = index(value, char(0), back=.true.)
      if (inull > 0 .and. inull < lnstr) value(inull:inull) = " "

      return
    end subroutine getEnvironment
    ! ===========================================================
    subroutine getArgument(index, argument)
      ! ===========================================================
      integer(kind=I4B), intent(in) :: index
      character(len=*), intent(out) :: argument
      integer(kind=I4B) :: i1
      ! ===========================================================
      i1 = index + arg_shift
      call getarg(i1, argument)

      return
    end subroutine getArgument

end module extension
