! edited to deal with underscores (needs a preprocessor)

module healpix_fft
use healpix_types
implicit none
private

public real_fft, complex_fft

interface complex_fft
  module procedure s_c_complex_fft, s_r_complex_fft, &
                   d_c_complex_fft, d_r_complex_fft
end interface

interface real_fft
  module procedure s_real_fft, d_real_fft
end interface




integer, parameter :: &
  FFTW_FORWARD         = -1, FFTW_BACKWARD        = 1, &
  FFTW_REAL_TO_COMPLEX = -1, FFTW_COMPLEX_TO_REAL = 1, &
  FFTW_ESTIMATE        =  0, FFTW_MEASURE         = 1, &
  FFTW_OUT_OF_PLACE    =  0, FFTW_IN_PLACE=8

contains 

!
! fftw_f77 routines come with 2 trailing underscores (at least when compiled with gcc)
! we have to make sure that the present code also call routines with those __
!

#ifdef ADD0US
! !----- compiler adds 2 trailing underscores by default : do nothing
subroutine my_fftw_f77_destroy_plan(plan)
  integer(i8b) :: plan
  call fftw_f77_destroy_plan (plan)
end subroutine my_fftw_f77_destroy_plan
subroutine my_fftw_f77_create_plan(plan, n, bw, direction)
  integer(i8b) :: plan
  integer      :: n, bw, direction
  call fftw_f77_create_plan (plan, n, bw, direction)
end subroutine my_fftw_f77_create_plan
subroutine my_fftw_f77_one (plan, data2, buffer)
  integer(i8b) :: plan
  real(dp), dimension(1:) :: data2, buffer
  call fftw_f77_one (plan, data2, buffer)
end subroutine my_fftw_f77_one
#endif
#ifdef ADD1US
!----- compiler adds 1 trailing underscore by default : add one
subroutine my_fftw_f77_destroy_plan(plan)
  integer(i8b) :: plan
  call fftw_f77_destroy_plan_ (plan)
end subroutine my_fftw_f77_destroy_plan
subroutine my_fftw_f77_create_plan(plan, n, bw, direction)
  integer(i8b) :: plan
  integer      :: n, bw, direction
  call fftw_f77_create_plan_ (plan, n, bw, direction)
end subroutine my_fftw_f77_create_plan
subroutine my_fftw_f77_one (plan, data2, buffer)
  integer(i8b) :: plan
  real(dp), dimension(1:) :: data2, buffer
  call fftw_f77_one_ (plan, data2, buffer)
end subroutine my_fftw_f77_one
#endif
#ifdef ADD2US
!----- compiler adds 0 trailing underscore by default : add two
subroutine my_fftw_f77_destroy_plan(plan)
  integer(i8b) :: plan
  call fftw_f77_destroy_plan__ (plan)
end subroutine my_fftw_f77_destroy_plan
subroutine my_fftw_f77_create_plan(plan, n, bw, direction)
  integer(i8b) :: plan
  integer      :: n, bw, direction
  call fftw_f77_create_plan__ (plan, n, bw, direction)
end subroutine my_fftw_f77_create_plan
subroutine my_fftw_f77_one (plan, data2, buffer)
  integer(i8b) :: plan
  real(dp), dimension(1:) :: data2, buffer
  call fftw_f77_one__ (plan, data2, buffer)
end subroutine my_fftw_f77_one
#endif
!----------------------------

function complex_plan (n, backward)
  integer, intent(in) :: n
  logical, intent(in), optional :: backward
  integer(i8b) complex_plan

  logical, save :: have_plan = .false.
  integer(i8b),save :: my_plan
  integer, save :: my_n, my_backward

  integer bw

  bw = FFTW_FORWARD
  if (present(backward)) &
    bw = merge (FFTW_BACKWARD, FFTW_FORWARD, backward)

  if (have_plan) then
    if ((n/=my_n) .or. (bw/=my_backward)) then
      call my_fftw_f77_destroy_plan (my_plan)
      have_plan= .false.
    endif
  endif
  if (.not. have_plan) then
    call my_fftw_f77_create_plan (my_plan, n, bw, FFTW_ESTIMATE+FFTW_OUT_OF_PLACE)
    have_plan = .true.
    my_n = n
    my_backward = bw
  endif

  complex_plan = my_plan
end function

subroutine d_c_complex_fft (data, backward, onlyreal)
  complex(dp), intent(inout) :: data(:)
  logical, intent(in), optional :: backward, onlyreal

  real(dp) data2(2*size(data)), buffer(2*size(data))
  integer(i8b) plan

  data2 = transfer (data, data2)
  plan = complex_plan (size(data), backward)
  call my_fftw_f77_one (plan, data2, buffer)
  data = transfer (buffer, data)
end subroutine

subroutine d_r_complex_fft (data, backward, onlyreal)
  real(dp), intent(inout) :: data(:)
  logical, intent(in), optional :: backward, onlyreal

  real(dp) buffer(size(data))
  integer(i8b) plan

  plan = complex_plan (size(data)/2, backward)
  call my_fftw_f77_one (plan, data, buffer)
  data = buffer
end subroutine

subroutine s_c_complex_fft (data, backward, onlyreal)
  complex(sp), intent(inout) :: data(:)
  logical, intent(in), optional :: backward, onlyreal

  real(dp) data2(2*size(data)), buffer(2*size(data))
  integer(i8b) plan
  integer i

  do i = 1,size(data)
    data2(2*i-1) = real(data(i))
    data2(2*i)   = aimag(data(i))
  end do
  plan = complex_plan (size(data), backward)
  call my_fftw_f77_one (plan, data2, buffer)
  do i = 1,size(data)
    data(i) = cmplx (data2(2*i-1),data2(2*i))
  end do
end subroutine

subroutine s_r_complex_fft (data, backward, onlyreal)
  real(sp), intent(inout) :: data(:)
  logical, intent(in), optional :: backward, onlyreal

  real(dp) data2(size(data)), buffer(size(data))
  integer(i8b) plan

  data2 = data
  plan = complex_plan (size(data)/2, backward)
  call my_fftw_f77_one (plan, data2, buffer)
  data = buffer
end subroutine

subroutine d_real_fft (data, backward)
  real(dp), intent(inout) :: data(:)
  logical, intent(in), optional :: backward

  real(dp) data2 (2*size(data)), buffer(2*size(data))
  logical bw
  integer n, i
  integer(i8b) plan

  n = size(data)
  bw = .false.
  if (present(backward)) bw=backward

  if (.not. bw) then
    data2 (2:2*n:2) = 0
    data2 (1:2*n-1:2) = data
    plan = complex_plan (size(data), backward)
    call my_fftw_f77_one (plan, data2, buffer)
    data(1) = buffer(1)
    data(2:n) = buffer(3:n+1)
  else
    data2(1) = data(1)
    data2(2) = 0
    data2(3:n+1) = data(2:n)
    data2(n+2:) = 0
    do i=1, n/2
      data2(2*n-2*i+1)=  data2(2*i+1)
      data2(2*n-2*i+2)= -data2(2*i+2)
    end do
    plan = complex_plan (size(data), backward)
    call my_fftw_f77_one (plan, data2, buffer)
    data = buffer(1:2*n-1:2)
  endif
end subroutine

subroutine s_real_fft (data, backward)
  real(sp), intent(inout) :: data(:)
  logical, intent(in), optional :: backward

  real(dp) data2 (2*size(data)), buffer(2*size(data))
  logical bw
  integer n, i
  integer(i8b) plan

  n = size(data)
  bw = .false.
  if (present(backward)) bw=backward

  if (.not. bw) then
    data2 (2:2*n:2) = 0
    data2 (1:2*n-1:2) = data
    plan = complex_plan (size(data), backward)
    call my_fftw_f77_one (plan, data2, buffer)
    data(1) = buffer(1)
    data(2:n) = buffer(3:n+1)
  else
    data2(1) = data(1)
    data2(2) = 0
    data2(3:n+1) = data(2:n)
    data2(n+2:) = 0
    do i=1, n/2
      data2(2*n-2*i+1)=  data2(2*i+1)
      data2(2*n-2*i+2)= -data2(2*i+2)
    end do
    plan = complex_plan (size(data), backward)
    call my_fftw_f77_one (plan, data2, buffer)
    data = buffer(1:2*n-1:2)
  endif
end subroutine

end module healpix_fft
