! -*- f90 -*-

!
! v1.0: M. Reinecke
! v1.1: 2002-09, E. Hivon, added concatnl, scan_directories, numeric_string
!                     made interactive mode more user friendly
!
module paramfile_io
  use healpix_types
  implicit none
  private

  public paramfile_handle, parse_init, parse_real, parse_double, parse_int, &
         parse_long, parse_lgt, parse_string
  public concatnl, scan_directories
!   public numeric_string

  type paramfile_handle
    character(len=filenamelen) filename
    logical interactive
  end type paramfile_handle

  character(len=*), parameter, public :: ret = achar(10)//' '

contains

!=====================================================================
subroutine notify_user (keyname, rdef, rmin, rmax, ddef, dmin, dmax, &
  idef, imin, imax, ldef, lmin, lmax, logdef, chdef, descr)
  !=====================================================================
  ! prompts user for next parameter when in interactive mode
  !=====================================================================
  character(len=*), intent(in) :: keyname
  real(sp), intent(in), optional :: rdef, rmin, rmax
  real(dp), intent(in), optional :: ddef, dmin, dmax
  integer(i4b), intent(in), optional :: idef, imin, imax
  integer(i8b), intent(in), optional :: ldef, lmin, lmax
  logical, intent(in), optional :: logdef
  character(len=*), intent(in), optional :: chdef, descr

!   print *, 'Please enter a value for the key ', keyname
!   if (present(descr)) print *,'(',descr,').'
  if (present(descr)) then
     write(*,'(a)') trim(descr)
  else
     print *, 'Please enter a value for the key ', keyname
  endif
  if (present(rmin)) print *, "min value: ", rmin
  if (present(dmin)) print *, "min value: ", dmin
  if (present(imin)) print *, "min value: ", imin
  if (present(lmin)) print *, "min value: ", lmin
  if (present(rmax)) print *, "max value: ", rmax
  if (present(dmax)) print *, "max value: ", dmax
  if (present(imax)) print *, "max value: ", imax
  if (present(lmax)) print *, "max value: ", lmax
  if (present(rdef)) print *, "default value: ", rdef
  if (present(ddef)) print *, "default value: ", ddef
  if (present(idef)) print *, "default value: ", idef
  if (present(ldef)) print *, "default value: ", ldef
  if (present(logdef)) print *, "default value: ", logdef
  if (present(chdef)) print *, "default value: ", trim(chdef)
end subroutine notify_user

!===================================================================
function parse_init (fname)
  !===================================================================
  character(len=*), intent(in) :: fname
  type(paramfile_handle) parse_init
  logical existant

  if (len(trim(fname))==0) then
    parse_init%filename = ''
    parse_init%interactive=.true.
  else
    inquire(file=fname,exist=existant)
    if (.not.existant) then
      print *, 'Parser: error: cannot open ', trim(fname)
      stop 1
    endif
    if (len(fname)>filenamelen) then
      print *,'Parser: error: file name too long'
      stop 1
    endif
    parse_init%filename = fname
    parse_init%interactive=.false.
  endif

  ! be verbose
  if (parse_init%interactive) then
     write(*,'(a)') 'Interactive mode. Answer the following questions.'
     write(*,'(a)') 'If no answer is entered, the default value will be taken'
  else
     write(*,'(a)') 'Reading run parameters from '//trim(parse_init%filename)
     write(*,'(a)') ' parameters not defined in that file will be set to their default value'
  endif
end function parse_init
!===================================================================

!===================================================================
subroutine find_param (handle,keyname,result,found,rdef,rmin,rmax, &
    ddef,dmin,dmax,idef,imin,imax,ldef,lmin,lmax,logdef,chdef,descr)
  !===================================================================
  ! extract parameter from file or read from standard input
  !===================================================================
  type(paramfile_handle), intent(in) :: handle
  character(len=*), intent(in) :: keyname
  character(len=*), intent(out) :: result
  logical, intent(out) :: found
  real(sp), intent(in), optional :: rdef, rmin, rmax
  real(dp), intent(in), optional :: ddef, dmin, dmax
  integer(i4b), intent(in), optional :: idef, imin, imax
  integer(i8b), intent(in), optional :: ldef, lmin, lmax
  logical, intent(in), optional :: logdef
  character(len=*), intent(in), optional :: chdef, descr

  character(len=filenamelen) :: line, name, value
  integer i
  !===================================================================

  found=.false.

  if (.not. handle%interactive) then
    open  (1, file=trim(handle%filename))
    do
      read (1,'(a)',end=2) line
      i=scan(line,'=')
      if (i==0 .or. line(1:1)=='#') cycle
      name=trim(adjustl(line(:i-1)))
      value=trim(adjustl(line(i+1:)))

      if (trim(name)==keyname) then
        result=trim(value)
        found=.true.
        if (trim(result)=="") then
           write(*,'(a)') ' '
           write(*,'(a)') 'ERROR: Inputs of the form '
           write(*,'(a)') trim(keyname)//' = '
           write(*,'(a)') ' (ie, defined as a blank value) are not valid'
           write(*,'(a)') 'To get the default value, comment out the keyword in '&
                &         //trim(handle%filename)
           write(*,'(a)') '# '//trim(keyname)//' = '
           write(*,'(a)') "If you mean 'No file', use"
           write(*,'(a)') trim(keyname)//" = ''   "
           write(*,'(a)') ' '
           stop 1
        endif
      end if
    end do
2   close (1)
  else
    call notify_user (keyname,rdef,rmin,rmax,ddef,dmin,dmax, &
                      idef,imin,imax,ldef,lmin,lmax,logdef,chdef,descr)
    read (*,'(a)',err=5) result
    found = (trim(result)/='')
  end if

  return

5 print*,'Parser: find_param: error reading value'
  stop 1
end subroutine find_param
!===================================================================

!===================================================================
function parse_real (handle, keyname, default, vmin, vmax, descr)
  !===================================================================
  !===================================================================
  type(paramfile_handle), intent(in) :: handle
  character(len=*), intent(in) :: keyname
  real(sp), intent(in), optional :: default, vmin, vmax
  character(len=*), intent(in), optional :: descr
  real(sp) :: parse_real

  character(len=filenamelen) :: result
  logical found
  !===================================================================

10 continue
  call find_param (handle, trim(keyname), result, found, rdef=default, &
                   rmin=vmin, rmax=vmax, descr=descr)
  if (found) then
    read (result,*,err=5) parse_real
  else
    if (present(default)) then
      print *,'Parser: warning: using default value for ',trim(keyname)
      parse_real = default
    else
      print *,'Parser: error: ',trim(keyname),' not found.'
      goto 2
    endif
  endif
  print *,'Parser: ',trim(keyname),' = ',parse_real
  if (present(vmin)) then
    if (parse_real<vmin) then
      print *,'Parser: error: value for ', trim(keyname),' too small.'
      goto 2
    endif
  endif
  if (present(vmax)) then
    if (parse_real>vmax) then
      print *,'Parser: error: value for ', trim(keyname),' too large.'
      goto 2
    endif
  endif

  return ! normal exit

5 print*,'Parser: parse_real: error reading value'
2 if (handle%interactive) goto 10 ! try again
  stop 1

end function parse_real

!===================================================================
function parse_double (handle, keyname, default, vmin, vmax, descr)
  !===================================================================
  !===================================================================
  type(paramfile_handle), intent(in) :: handle
  character(len=*), intent(in) :: keyname
  real(dp), intent(in), optional :: default, vmin, vmax
  character(len=*), intent(in), optional :: descr
  real(dp) :: parse_double

  character(len=filenamelen) :: result
  logical found
  !===================================================================

10 continue
  call find_param (handle, trim(keyname), result, found, ddef=default, &
                   dmin=vmin, dmax=vmax, descr=descr)
  if (found) then
    read (result,*,err=5) parse_double
  else
    if (present(default)) then
      print *,'Parser: warning: using default value for ',trim(keyname)
      parse_double = default
    else
      print *,'Parser: error: ',trim(keyname),' not found.'
      goto 2
    endif
  endif
  print *,'Parser: ',trim(keyname),' = ',parse_double
  if (present(vmin)) then
    if (parse_double<vmin) then
      print *,'Parser: error: value for ', trim(keyname),' too small.'
      goto 2
    endif
  endif
  if (present(vmax)) then
    if (parse_double>vmax) then
      print *,'Parser: error: value for ', trim(keyname),' too large.'
      goto 2
    endif
  endif

  return ! normal exit

5 print*,'Parser: parse_double: error reading value'
2 if (handle%interactive) goto 10 ! try again
  stop 1

end function parse_double

!==================================================================
function parse_int (handle, keyname, default, vmin, vmax, descr)
  !==================================================================
  ! parse 4 byte integer parameter
  !==================================================================
  type(paramfile_handle), intent(in) :: handle
  character(len=*), intent(in) :: keyname
  integer(i4b), intent(in), optional :: default, vmin, vmax
  character(len=*), intent(in), optional :: descr
  integer(i4b) :: parse_int

  character(len=filenamelen) :: result
  logical :: found
  !==================================================================

10 continue
  call find_param (handle, trim(keyname), result, found, idef=default, &
                   imin=vmin, imax=vmax, descr=descr)
  if (found) then
    read (result,*,err=5) parse_int
  else
    if (present(default)) then
      print *,'Parser: warning: using default value for ',trim(keyname)
      parse_int = default
    else
      print *,'Parser: error: ',trim(keyname),' not found.'
      goto 2
    endif
  endif
  print *,'Parser: ',trim(keyname),' = ',parse_int
  if (present(vmin)) then
    if (parse_int<vmin) then
      print *,'Parser: error: value for ', trim(keyname),' too small.'
      goto 2
    endif
  endif
  if (present(vmax)) then
    if (parse_int>vmax) then
      print *,'Parser: error: value for ', trim(keyname),' too large.'
      goto 2
    endif
  endif

  return ! normal exit

5 print*,'Parser: parse_int: error reading value'
2 if (handle%interactive) goto 10 ! try again
  stop 1

end function parse_int
!==================================================================

!==================================================================
function parse_long (handle, keyname, default, vmin, vmax, descr)
  !==================================================================
  ! parse 8 byte integer parameter
  !==================================================================
  type(paramfile_handle), intent(in) :: handle
  character(len=*), intent(in) :: keyname
  integer(i8b), intent(in), optional :: default, vmin, vmax
  character(len=*), intent(in), optional :: descr
  integer(i8b) :: parse_long

  character(len=filenamelen) :: result
  logical found
  !==================================================================

10 continue
  call find_param (handle, trim(keyname), result, found, ldef=default, &
                   lmin=vmin, lmax=vmax, descr=descr)
  if (found) then
    read (result,*,err=5) parse_long
  else
    if (present(default)) then
      print *,'Parser: warning: using default value for ',trim(keyname)
      parse_long = default
    else
      print *,'Parser: error: ',trim(keyname),' not found.'
      goto 2
    endif
  endif
  print *,'Parser: ',trim(keyname),' = ',parse_long
  if (present(vmin)) then
    if (parse_long<vmin) then
      print *,'Parser: error: value for ', trim(keyname),' too small.'
      goto 2
    endif
  endif
  if (present(vmax)) then
    if (parse_long>vmax) then
      print *,'Parser: error: value for ', trim(keyname),' too large.'
      goto 2
    endif
  endif

  return ! normal exit

5 print*,'Parser: parse_long: error reading value'
2 if (handle%interactive) goto 10 ! try again
  stop 1

end function parse_long

!===================================================================
function parse_lgt (handle, keyname, default, descr)
  !===================================================================
  ! parse (1 byte) logical parameter
  !===================================================================
  type(paramfile_handle), intent(in) :: handle
  character(len=*), intent(in) :: keyname
  logical, intent(in), optional :: default
  character(len=*), intent(in), optional :: descr
  logical :: parse_lgt

  character(len=filenamelen) :: result
  logical found
  !===================================================================

10 continue
  call find_param (handle, trim(keyname), result, found, logdef=default, &
                   descr=descr)
  if (found) then
    read (result,*,err=5) parse_lgt
  else
    if (present(default)) then
      print *,'Parser: warning: using default value for ',trim(keyname)
      parse_lgt = default
    else
      print *,'Parser: error: ',trim(keyname),' not found.'
      goto 2
    endif
  endif
  print *,'Parser: ',trim(keyname),' = ',parse_lgt

  return ! normal exit

5 print*,'Parser: parse_lgt: error reading value'
2 if (handle%interactive) goto 10 ! try again
  stop 1

end function parse_lgt

!===================================================================
function parse_string (handle, keyname, default, descr, filestatus)
  !===================================================================
  ! parse a character string parameter
  ! 
  ! if filestatus is 'old', look for an existing file having the name of the string
  !
  ! if filestatus is 'new', no file with the exact same name as the string should exist
  !
  !===================================================================
  type(paramfile_handle), intent(in) :: handle
  character(len=*), intent(in) :: keyname
  character(len=*), intent(in), optional :: default
  character(len=*), intent(in), optional :: descr
  character(len=*), intent(in), optional :: filestatus
  character(len=filenamelen) :: parse_string

  character(len=filenamelen) :: result
  logical :: found, there
  !===================================================================

10 continue
  call find_param (handle, trim(keyname), result, found, chdef=default, &
                   descr=descr)
  if (found) then
    parse_string = trim(result)
  else
    if (present(default)) then
      write(*,'(1x,a)') 'Parser: warning: using default value for '//trim(keyname)
      parse_string = trim(default)
    else
      write(*,'(1x,a)') 'Parser: error: '//trim(keyname)//' not found.'
      goto 2
    endif
  endif
  write(*,'(1x,a)') 'Parser: '//trim(keyname)//' = '//trim(parse_string)

  ! 0 (zero), '' and ' ' (2 single quotes with nothing or one space in between) 
  !         are interpreted as "No File"
  if (trim(adjustl(parse_string)) == "0" )  parse_string = ''
  if (trim(adjustl(parse_string)) == "''")  parse_string = ''
  if (trim(adjustl(parse_string)) == "' '") parse_string = ''

  if (present(filestatus) .and. trim(parse_string) /= '') then
     if (trim(filestatus)=='new' .or. trim(filestatus)=='NEW') then
        inquire(file=trim(parse_string),exist=there)
        if (there) then
           print *, 'Parser: error: output file ' // trim(parse_string) // &
                ' already exists!'
           goto 2
        end if
     else if (trim(filestatus)=='old' .or. trim(filestatus)=='OLD') then
        inquire(file=trim(parse_string),exist=there)
        if (.not. there) then
           print *, 'Parser: error: input file ' // trim(parse_string) // &
                ' does not exist!'
           goto 2
        end if
     else
        print *, 'Parser: error: wrong value for filestatus :',filestatus
        stop 1
     endif
  endif

  return ! normal exit

2 if (handle%interactive) goto 10 ! try again
  stop 1
  
end function parse_string



!========================================================================
function concatnl(line1,line2,line3,line4,line5,line6,line7,line8,line9,line10)
  !========================================================================
  ! concatenate line1, line2, line3,... into one string,
  ! while putting a char(10) Line Feed in between
  !========================================================================
  
  character(len=*), intent(in)           :: line1
  character(len=*), intent(in), optional ::       line2,line3,line4,line5
  character(len=*), intent(in), optional :: line6,line7,line8,line9,line10

  character(len=filenamelen) :: concatnl

  concatnl = trim(line1)
  if (present(line2)) concatnl = trim(concatnl)//ret//trim(line2)
  if (present(line3)) concatnl = trim(concatnl)//ret//trim(line3)
  if (present(line4)) concatnl = trim(concatnl)//ret//trim(line4)
  if (present(line5)) concatnl = trim(concatnl)//ret//trim(line5)
  if (present(line6)) concatnl = trim(concatnl)//ret//trim(line6)
  if (present(line7)) concatnl = trim(concatnl)//ret//trim(line7)
  if (present(line8)) concatnl = trim(concatnl)//ret//trim(line8)
  if (present(line9)) concatnl = trim(concatnl)//ret//trim(line9)
  if (present(line10)) concatnl = trim(concatnl)//ret//trim(line10)


end function concatnl
!========================================================================

!========================================================================
function scan_directories(directories, filename, fullpath)
  !========================================================================
  ! scan directories in search of filename,
  ! if found, returns .true. and the full path is in fullpath.
  ! The search is *NOT* reccursive
  !
  ! it assumes that the given directory and filename are separated by either
  !  nothing, a / (slash) or a \ (backslash)
  !
  ! if several directories are to be searched (up to 20), 
  ! concatenate them into 'directories',
  ! putting a special character (ASCII < 32) between them.
  !  see concatnl
  ! NB: a space is not a special character
  !========================================================================
  logical(LGT) :: scan_directories
  character(len=*), intent(in)  :: filename, directories
  character(len=*), intent(out) :: fullpath

  logical :: found
  integer(I4B), dimension(1:20) :: index
  integer(I4B)                  :: i, k, nc, nspecial
  character(len=1)              :: ch
  character(len=filenamelen)    :: directory
  character(len=3000)           :: string
  character(LEN=1), DIMENSION(1:3) :: separator
  character(len=*), parameter   :: code = 'scan_directories'
  !========================================================================

  ! define separators (this is the only way that works for all compilers)
  separator(1) = char(32) ! ' '
  separator(2) = char(47) ! '/'
  separator(3) = char(92) ! '\'

  ! find location of special characters
  nc = len_trim(directories)
  index(1) = 0
  nspecial = 2
  do i=1,nc
     ch = directories(i:i)
     if (iachar(ch) < 32) then
        index(nspecial) = i
        nspecial        = nspecial + 1
     endif
  enddo
  index(nspecial) = nc + 1

  ! test string between special character as potential directory
  fullpath = ''
  found = .false.
  do i = 1, nspecial-1
     directory=trim(adjustl(directories(index(i)+1:index(i+1)-1)))
        do k = 1, size(separator)
           string = trim(directory)//trim(separator(k))//trim(filename)
           inquire(&
                &  file=string, &
                &  exist=found)
           if (found) goto 10
        enddo
  enddo

10 continue
  if (found) then
     if (len(fullpath) >= len_trim(string)) then
        fullpath = trim(string)
     else
        print*,code
        print*,'variable fullpath is not large enough'
        print*,'requires ',len_trim(string),' characters'
        print*,'has only ',trim(fullpath)
        stop 1
     endif
  endif

  scan_directories = found

end function scan_directories

end module paramfile_io
