!===================================================================================
program pmlgen
  ! scalar implementation
  !===================================================================================
  ! precomputation of scalar and tensorial spherical harmonics
  ! version 1.0 : ?
  ! version 1.1 : ?
  ! version 1.2 :
  !   Aug 14, 2000, EH, Caltech, switch to CMBFAST normalisation of tensorial spherical harmonics
  !       (see Zaldarriaga, astro-ph/9709271; ApJ, 503, 1 (1998))
  !===================================================================================
  USE healpix_types
  USE utilities, ONLY: die_alloc
  USE head_fits, ONLY: add_card
  USE fitstools, ONLY: write_dbintab
  USE pix_tools, ONLY: nside2npix

  implicit none


  INTEGER(I4B) :: nsmax
  INTEGER(I4B) :: nlmax
  INTEGER(I4B) :: nmmax

  INTEGER(I4B) :: l, m, ith, scalem, scalel,lm, pp          ! alm related

  REAL(DP) :: cth, sth, dth1, dth2, dst1, OVFLOW, UNFLOW, logOVFLOW
  REAL(DP) :: a_rec, lam_mm, lam_lm, lam_0, lam_1, lam_2, par_lm, lam_lm1m
  REAL(DP) :: f2m, fm2, fl2, corfac, one_on_s2, c_on_s2,fm_on_s2, a_x, a_w, b_w
  REAL(DP) :: fl, fm

  INTEGER(I4B) :: status, nlheader, map_type
  
  REAL(DP), DIMENSION(:,:), ALLOCATABLE :: recfac, lam_fact
  REAL(DP), DIMENSION(:,:),ALLOCATABLE :: plm
  REAL(DP), DIMENSION(:), ALLOCATABLE :: normal_l
  
  LOGICAL(LGT) :: bad, polarisation
  CHARACTER(LEN=filenamelen)                  :: outfile
  CHARACTER(LEN=80), DIMENSION(160) :: header
  CHARACTER(LEN=*), PARAMETER :: code = 'PLMGEN'
  CHARACTER(LEN=*), PARAMETER :: version = '1.2.0'
  REAL(KIND=DP), PARAMETER :: FL_LARGE = 1.0e30_dp
  REAL(KIND=DP), PARAMETER :: FL_SMALL = 1.0e-30_dp
  !===================================================================================
  
3 PRINT *,' Enter the resolution parameter (Nside) '
  READ *, nsmax
  if (nside2npix(nsmax) < 0) goto 3

  PRINT *,' Enter the maximum l range for the precomputed P_lms: '
  WRITE(*,'(a,i5,a)') '  (0 <= l <= l_max <= ',3*nsmax-1,') l_max = '
  READ *, nlmax

10 PRINT *, ' Do you want to precompute for'
  PRINT *, ' 1) Temperature maps only'
  PRINT *, ' 2) Temperature + polarisation maps'
  READ *, map_type
  if (map_type .lt. 1 .or. map_type .gt. 2) goto 10
  if (map_type .eq. 1) polarisation = .false.
  if (map_type .eq. 2) polarisation = .true.
  
5 PRINT *,' Enter Output file name (eg, plm.fits) :'
  READ 100, outfile
  inquire(file=outfile, exist=bad)
  if ( bad ) then
     PRINT *,' '//code//'> '//TRIM(outfile)//' already exists'
     PRINT *,' '//code//'> choose a new output file name.'
     goto 5
  endif
  
100 FORMAT(A)
  
  nmmax=nlmax
  
  
  pp=(nlmax+1)*(nlmax+2)*nsmax

  ALLOCATE(recfac(0:nlmax,0:nmmax),stat = status)    
  if (status /= 0) call die_alloc(code,'recfac')
  
  if (polarisation) then
     
     ALLOCATE(plm(0:pp-1,1:3),stat = status)    
     if (status /= 0) call die_alloc(code,'plm')
     
     ALLOCATE(lam_fact(0:nlmax,0:nmmax),stat = status)    
     if (status /= 0) call die_alloc(code,'lam_fact')
     
     ALLOCATE(normal_l(0:nlmax),stat = status)    
     if (status /= 0) call die_alloc(code,'normal_l')
     
     
  else
     
     ALLOCATE(plm(0:pp-1,1:1),stat = status)    
     if (status /= 0) call die_alloc(code,'plm')
     
  endif
  
  
  plm=0.d0
  
  do m = 0, nmmax
     fm2 = DBLE(m) **2
     do l = m, nlmax
        fl2 = DBLE(l+1) **2
        recfac(l,m) = DSQRT( (4.d0 * fl2 - 1.d0) / (fl2-fm2) )
     enddo
  enddo

  if (polarisation) then
     lam_fact(:,:) = 0.d0
     do l = 2, nlmax
        fl  = DBLE(l)
        fl2 = fl * fl
        do m = 0, l
           fm2 = DBLE(m) **2
           lam_fact(l,m) = 2.d0 * SQRT( (2.d0 * fl + 1.d0) / (2.d0 * fl - 1.d0) * (fl2-fm2) )
        enddo
     enddo
   
     normal_l(:) = 0.d0
     do l = 2, nlmax
        fl = DBLE(l)
        normal_l(l) = SQRT( KvS / ((fl+2.0_dp)*(fl+1.0_dp)*fl*(fl-1.0_dp)) ) 
        ! either CMBFAST (KvS=1) or Kamionkowski et al. (KvS=2) definition
     enddo
  endif

  
  OVFLOW=exp(log(FL_LARGE))
  UNFLOW=exp(log(FL_SMALL))
  logOVFLOW=log(FL_LARGE)
  
  dth1 = 1.d0 / (3.d0*DBLE(nsmax)**2)
  dth2 = 2.d0 / (3.d0*DBLE(nsmax))
  dst1 = 1.d0 / (SQRT(6.d0) * DBLE(nsmax) )

  lm=0

  do ith = 1,2*nsmax      ! 0 <= cos theta < 1
     !        cos(theta) in the pixelisation scheme
     if (ith.lt.nsmax) then  ! polar cap (north)
        cth = 1.d0  - DBLE(ith)**2 * dth1
        sth = SIN( 2.d0 * ASIN( ith * dst1 ) ) ! sin(theta)
     else                   ! tropical band (north) + equator
        cth = DBLE(2*nsmax-ith) * dth2
        sth = DSQRT((1.D0-cth)*(1.D0+cth)) ! sin(theta)
     endif
     one_on_s2 = 1.d0 / sth**2 ! 1/sin^2
     c_on_s2 = cth * one_on_s2

     !        -----------------------------------------------------
     !        for each theta, and each m, computes
     !        b(m,theta) = sum_over_l>m (lambda_l_m(theta) * a_l_m) 
     !        ------------------------------------------------------
     !        lambda_mm tends to go down when m increases (risk of underflow)
     !        lambda_lm tends to go up   when l increases (risk of overflow)
     lam_mm = sq4pi_inv ! lambda_00
     scalem=1
     do m = 0, nmmax
        fm  = DBLE(m)
        f2m = 2.d0 * fm
        fm2 = fm * fm
        fm_on_s2 = fm * one_on_s2
      
        !           ---------- l = m ----------
        par_lm = 1.d0  ! = (-1)^(l+m)
        if (m .ge. 1) then ! lambda_0_0 for m>0
           lam_mm = -lam_mm*sth*dsqrt((f2m+1.d0)/f2m)
        endif
        if (abs(lam_mm).lt.UNFLOW) then
           lam_mm=lam_mm*OVFLOW
           scalem=scalem-1
        endif
        corfac=exp(scalem*logOVFLOW) !Actual lam_mm
        lam_lm = lam_mm*corfac/OVFLOW
        plm(lm,1)=lam_lm
        if (polarisation) then
           plm(lm,2)=( normal_l(m) * lam_lm * (fm - fm2) ) * ( 2.d0 * one_on_s2 - 1.d0 ) * SW1
           plm(lm,3)=( normal_l(m) * lam_lm * (fm - fm2) ) *   2.d0 *   c_on_s2
        endif
        lm=lm+1
        !           ---------- l > m ----------
        lam_0 = 0.d0
        lam_1 = 1.d0 
        scalel=0
        a_rec = recfac(m,m)
        lam_2 = cth * lam_1 * a_rec
        do l = m+1, nlmax
           par_lm = - par_lm  ! = (-1)^(l+m)
           lam_lm1m=lam_lm  ! actual lambda_l-1,m (useful for polarisation)
           lam_lm = lam_2*corfac*lam_mm/OVFLOW ! Remove OVFLOW-factors 
           plm(lm,1)=lam_lm
           if (polarisation) then
              fl  = DBLE(l)
              fl2 = fl * fl
              a_w =  2.d0 * (fl - fm2) * one_on_s2 + (fl2 - fl)
              b_w =  c_on_s2 * lam_fact(l,m)
              a_x =  2.d0 * cth * (fl-1.d0) * lam_lm
              plm(lm,2) =   normal_l(l) * ( a_w * lam_lm - b_w * lam_lm1m ) * SW1
              plm(lm,3) = - normal_l(l) * fm_on_s2 * ( a_x - lam_fact(l,m) * lam_lm1m )
           endif
           lm=lm+1
           lam_0 = lam_1 / a_rec
           lam_1 = lam_2
           a_rec = recfac(l,m)
           lam_2 = (cth * lam_1 - lam_0) * a_rec
           
           if (abs(lam_2) .gt. OVFLOW) then
              lam_0=lam_0/OVFLOW
              lam_1=lam_1/OVFLOW
              lam_2 = (cth * lam_1 - lam_0) * a_rec
              scalel=scalel+1
              corfac=exp((scalel+scalem)*logOVFLOW)
           else
              if (abs(lam_2) .lt. UNFLOW) then
                 lam_0=lam_0*OVFLOW
                 lam_1=lam_1*OVFLOW
                 lam_2 = (cth * lam_1 - lam_0) * a_rec 
                 scalel=scalel-1
                 corfac=exp((scalel+scalem)*logOVFLOW)
              endif
           endif
        enddo
     enddo
  enddo
  
  header = ' '
  call add_card(header,'NSIDE'   ,nsmax,   'Resolution parameter for HEALPIX')
  call add_card(header,'CREATOR',code,        'Software creating the FITS file')
  call add_card(header,'VERSION',version,     'Version of the simulation software')
  call add_card(header,'MAX-LPOL',nlmax      ,'Maximum multipole l used in map synthesis')
  call add_card(header)
  
  call add_card(header,'COMMENT','-----------------------------------------------')
  call add_card(header,'COMMENT','     Data Description Specific Keywords       ')
  call add_card(header,'COMMENT','-----------------------------------------------')
  call add_card(header)
  call add_card(header,'COMMENT','contains precomputed P_lms starting with P_00,')
  call add_card(header,'COMMENT','then P_10,P_11,P_20,P_21,P22, etc.....(m<0 not incl.)')
  call add_card(header,'COMMENT','synfast and anafast execute faster with precomputed P_lms')
  call add_card(header)
  call add_card(header,'TTYPE1', 'SCALAR',  'Scalar Spherical Harmonics (Y_lm)')
  call add_card(header,'TTYPE2', 'SPINSUM', 'half sum        of Spin-weighted Spherical Harmonics')
  call add_card(header,'TTYPE3', 'SPINDIFF','half difference of Spin-weighted Spherical Harmonics')
  call add_card(header,'COMMENT','spin-weighted spherical harmocis now have CMBFAST normalisation')
  call add_card(header)
  
  
  
  nlheader = SIZE(header)
  
  if (polarisation) then
     call write_dbintab(plm, pp, 3, header, nlheader, outfile,nsmax,nlmax)
  else
     call write_dbintab(plm, pp, 1, header, nlheader, outfile,nsmax,nlmax)
  endif
end program pmlgen



