pro init_healpix, verbose=verbose
;+
; defines the (structure) system variable Healpix
;
;-

; system variable name
healpix_sysvar = '!HEALPIX'

; Healpix version
version = 1.2

; release data
date = '2003-02-08'

; Healpix directory
sv = 'HEALPIX'
dsv = '$'+sv
directory = expand_path(dsv)
if (strtrim(directory,2) eq dsv) then begin
    print,' system variable '+sv+' not found '
    directory = ''
endif

; list of possible Nside's
nside = 2L^lindgen(14)  ; 1, 2, 4, 8, ..., 8192

; flag for missing values
bad_value = -1.6375e30

comment = ['This system variable contains some information on Healpix :', $
           healpix_sysvar+'.VERSION   = current version number,', $
           healpix_sysvar+'.DATE      = date of release,',$
           healpix_sysvar+'.DIRECTORY = directory containing Healpix package,',$
           healpix_sysvar+'.NSIDE     = list of all valid values of Nside parameter,',$
           healpix_sysvar+'.BAD_VALUE = value of flag given to missing pixels in FITS files,',$
           healpix_sysvar+'.COMMENT   = this description.']

; create structure
stc = {version:version, date:date, directory:directory, nside:nside, bad_value:bad_value, comment:comment}

; fill variable out
defsysv, healpix_sysvar, exists = exists
if (exists) then begin
    !Healpix = stc
endif else begin
    defsysv, healpix_sysvar, stc
endelse

if (keyword_set(verbose)) then begin

    print,'Initializing '+healpix_sysvar+' system variable'
    print
    print,comment,form='(a)'
    print
;     help,/st,healpix_sysvar
;     print
endif



return
end

