function extract_map_from_stc, stc, select_in, select=select, value=select_name, error=error, tonan=tonan


routine = 'extract_map_from_stc'
header = stc.(0)
if (error ne 0) then return,0
bad_value = -1.6375e30

junk = getsize_fits(header,/header,type = fits_type, nside = nside_head, tags=tag_names(stc))
; coverage = strtrim(strupcase(sxpar(header,'OBJECT',Count=flag_cov)),2)
; nside_head = long(sxpar(header,'NSIDE',Count=flag_nside))
print,fits_type
if (fits_type eq 3) then begin
    ; partial sky coverage
    select_pix = index_word(tag_names(stc),'PIXEL',value=select_name,err=error2)
    if undefined(select_in) then begin
        select_map = index_word(tag_names(stc),'SIGNAL',value=select_name,err=error1)
    endif else begin
        select_map = index_word(tag_names(stc),select_in,value=select_name,err=error1)
    endelse
    if (error1 ne 0 or error2 ne 0 or select_map le 0 or select_pix le 0) then begin
        print,'wrong select in '+routine
        print,select_in,tag_names(stc)
        error = 1
        return,0
    endif
    npix_head = nside2npix(nside_head)
    if keyword_set(tonan) then bad_val = !values.f_nan else bad_val = bad_value
    map = make_array(npix_head,/float,value=bad_val)
    map[stc.(select_pix)] = stc.(select_map)
    select = select_map
endif else if (fits_type eq 0 or fits_type eq 2) then begin
    ; full sky coverage (in image or binary table)
    if undefined(select_in) then begin
        select = index_word(tag_names(stc),1        ,value=select_name,err=error)
    endif else begin
        select = index_word(tag_names(stc),select_in,value=select_name,err=error)
    endelse
    if (error ne 0 or select eq -1) then begin
        print,'wrong select in '+routine
        print,select_in,tag_names(stc)
        error = 1
        return,0
    endif
    if (keyword_set(tonan)) then begin
        bad = where ( stc.(select) le (bad_value*.9), nbad)
        map = stc.(select)
        if (nbad gt 0) then map[bad] = !values.f_nan
    endif else begin
        map   = stc.(select)
    endelse
        
endif else begin
;     print,'Unrecognised format'
;     stop
    message,'Unrecognised format'
endelse

return, map
end

;==================================================================================================
pro loaddata, file_in, select_in,$
  data, pol_data, pix_type, pix_param, do_conv, do_rot, coord_in, coord_out, eul_mat, title_display, sunits, $
  SAVE=save,ONLINE=online,NESTED=nested_online,UNITS=units,COORD=coord,ROT=rot_ang,QUADCUBE=quadcube,LOG=log, $
  ERROR=error, FLIP=flip, POLARIZATION=polarization, FACTOR=factor_u, OFFSET = offset_u
;+
; LOADATA
;
; IN:
;   file_in, select_in
; OUT:
;   data, pol_data, pix_type, pix_param, do_conv, do_rot, coord_in, coord_out, eul_mat,
;   title_display, sunits 
; KEYWORDS
;   save, online, nested, units, coord, rot, quadcube, log, polarization,
;   flip, factor
; OPTIONAL OUPUT
;   error
;
;  Aug 2003 : accepts ONLINE polarized data
;-

factor = 1.
if defined(factor_u) then factor = factor_u
offset = 0.
if defined(offset_u) then offset = offset_u
; ----------------------------------------------
; check consistency of parameters
; ----------------------------------------------
error = 0
routine='loaddata'
kw_save = KEYWORD_SET(save) & kw_online = KEYWORD_SET (online)
if ((kw_save) and (kw_online)) then begin
    print,routine+' : /ONLINE and /SAVE can not be used together'
    error=1
    return
endif

if (datatype(file_in) NE 'STR' AND kw_save) then begin
    print,routine+' : an external file is expected with /SAVE'
    error=1
    return
endif

if (datatype(file_in) NE 'STR' AND NOT kw_online) then begin
    print,routine+' : a file name is expected or /ONLINE is missing'
    error=1
    return
endif

if (datatype(file_in) EQ 'STR' AND kw_online) then begin
    print,routine+' : /ONLINE can not be used with an external file'
    error=1
    return
endif

if (datatype(file_in) EQ 'STR') then begin ; looking for a file
    junk = FINDFILE(file_in, count=countfile) ; check its existence
    if (countfile eq 0) then begin
        print,file_in+' not found'
        error=1
        return
    endif
endif
if keyword_set(flip) then flipconv=1 else flipconv = -1  ; longitude increase leftward by default (astro convention)
do_polarization = keyword_set(polarization)
if not do_polarization then polarization = 0

; ----------------------------------------------
; reads in the FITS file or the SAVESET file
; ----------------------------------------------
print,'reading the file ...'
; in file_in, kw_save, kw_online, select_in, routine
; out data, title_display, header
select_name = ''
;if undefined(select_in) then select_in = 1

t_keys = ['T','I']
q_keys = ['Q-P','Q_P','QS','Q']
u_keys = ['U-P','U_P','US','U']

if (kw_save) then begin            ; read the saveset file
    RESTORE, file_in, /VERBOSE
    if (factor ne 1.) then data = temporary(data) * factor 
    if (offset ne 0.) then data = temporary(data) + factor*offset
    title_display = file_in
    if (do_polarization) then begin
        print,'No plotting of polarisation available with this kind of data set'
        error = 1
        return
    endif
    if (keyword_set(quadcube) and defined(sky_cube)) then sixpack2vector,sky_cube, data
endif else begin                  ; non structure ONLINE data
    if (kw_online and datatype(file_in) ne 'STC') then begin
        title_display = ' on line processing '
        dim1 = n_elements(file_in[*,0])
        dim2 = n_elements(file_in[0,*])
        if (do_polarization) then begin
            if (dim2 ne 3) then begin
                print,'No plotting of polarisation available with this kind of data set'
                print,' expected array of shape [Npix,3],     got : ',dim1,dim2
                error = 1
                return
            endif else begin
                case polarization of
                    1: begin
                        select_name = 'Polarisation Amplitude'
                        data = SQRT(file_in[*,1]^2 + file_in[*,2]^2)*factor ; amplitude = sqrt(U^2+Q^2)
                    end
                    2: begin
                        select_name = 'Polarisation Direction'
                                ; change sign of U if astro convention
                        data = .5*ATAN(file_in[*,2]*flipconv, file_in[*,1]) ; angle phi = 1/2 atan(U/Q) in radians
                    end
                    3: begin
                                ; amplitude = sqrt(U^2+Q^2) & angle phi = 1/2 atan(U/Q) in radians
                        pol_data = [[SQRT(file_in[*,1]^2 + file_in[*,2]^2)*factor], $
                                    [.5*ATAN(file_in[*,2]*flipconv, file_in[*,1])]]
                        if (offset ne 0.) then pol_data[*,0] = pol_data[*,0] + factor*offset
                        data   = file_in[*,0]
                        select_name = 'Temperature + Polarisation'
                    end
                endcase
            endelse
        endif else begin
            data = file_in[*,0] * factor 
            if (offset ne 0.) then data = temporary(data) + factor*offset
        endelse
    endif else begin   ; ----------- STRUCTURE : online or fits file --------------------
        if (kw_online and datatype(file_in) EQ 'STC') then begin ; structure
            header = file_in.(0)
            title_display = ' on line processing '
            pol_err_mess = 'can not find POLARISATION in selected online data set'
        endif 
        if (not kw_online and datatype(file_in) EQ 'STR') then begin                   ; fits file
            file_keep = file_in
            READ_FITS_S, file_keep, file_in, /merge ; reads in a FITS extension or an image
            header = file_in.(0)
            title_display = file_keep
            pol_err_mess = 'can not find POLARISATION in file '+file_keep
        endif
        ;                            extract data from structure
        case polarization of
            1: begin
                data_q = extract_map_from_stc(file_in, q_keys, select=select, value=select_name, error=error, /tonan)
                data   = extract_map_from_stc(file_in, u_keys, select=select, value=select_name, error=error, /tonan)
                select_name = 'Polarisation Amplitude'
                if (error ne 0) then begin
                    print,pol_err_mess
                    return
                endif
                data = temporary(SQRT(temporary(data)^2 + data_q^2))*factor ; amplitude = sqrt(U^2+Q^2)
                data_q = 0.
            end
            2: begin
                data_q = extract_map_from_stc(file_in, q_keys, select=select, value=select_name, error=error, /tonan)
                data   = extract_map_from_stc(file_in, u_keys, select=select, value=select_name, error=error, /tonan)
                select_name = 'Polarisation Direction'
                if (error ne 0) then begin
                    print,pol_err_mess
                    return
                endif
                if (flipconv lt 0) then data = temporary(data) * flipconv ; change sign of U if astro convention
                data = .5*temporary(ATAN(temporary(data), data_q)) ; angle phi = 1/2 atan(U/Q) in radians
                data_q = 0.
            end
            3: begin
                data_q = extract_map_from_stc(file_in, q_keys, select=select, value=select_name, error=error, /tonan)
                data_u = extract_map_from_stc(file_in, u_keys, select=select, value=select_name, error=error, /tonan)
                if (error ne 0) then begin
                    print,pol_err_mess
                    return
                endif
                if (flipconv lt 0) then data_u = temporary(data_u) * flipconv ; change sign of U if astro convention
                ; amplitude = sqrt(U^2+Q^2) & angle phi = 1/2 atan(U/Q) in radians
                pol_data = [[SQRT(data_u^2 + data_q^2)*factor],[.5*(ATAN(data_u, data_q))]] 
                if (offset ne 0.) then pol_data[*,0] = pol_data[*,0] + factor*offset
                data_q = 0. & data_u = 0.
                data   = extract_map_from_stc(file_in, t_keys, select=select, value=select_name, error=error)
                select_name = 'Temperature + Polarisation'
            end
            else: begin ; temperature alone
                data = extract_map_from_stc(file_in, select_in, select=select, value=select_name, error=error)
                data = temporary(data)*factor
                if (offset ne 0.) then data = temporary(data) + factor*offset
            end
        endcase
        if (not kw_online and defined(file_keep)) then begin
            file_in = file_keep
        endif
        if (error ne 0) then return
    endelse
endelse
title_display = title_display + ': '+select_name

if defined(header) then begin
   if n_elements(header) eq 1 then header =[header,' ']
endif
; ---- Pixel scheme ----
; Healpix : Nested or Ring scheme
; or Quadcube
pix_type = 'R' ; Healpix ring by default
IF KEYWORD_SET(nested_online) THEN pix_type = 'N'
IF DEFINED(header) THEN BEGIN
    ordering = strtrim(SXPAR(header,'ORDERING',COUNT=flag_order),2)
    if (flag_order ne 0 and STRUPCASE(STRMID(ordering,0,4)) eq 'NEST') then pix_type = 'N'
ENDIF 
if keyword_set(quadcube) then pix_type = 'Q'

; ---- units ----
sunits = ''
if (datatype(units) eq 'STR') then begin ; if the units given is a string, take it
    sunits = units
endif else begin ; otherwise find it in the file
    if defined(header) then begin
        key_unit = 'TUNIT'+STRTRIM(STRING(select,form='(i4)'),2) ; TUNITi
        sunits_read = SXPAR(header,key_unit,COUNT=flag_units)
        if (flag_units NE 0) then sunits = STRTRIM(sunits_read, 2)
    endif
endelse
if keyword_set(log) then sunits = 'Log ('+sunits+')'


;----------------------------------------------------
;       plot parameters defined by the user
;----------------------------------------------------
; ---- input and output coord system ----
flag_coord = 0 & coord_in = 'G'
IF DEFINED(header) THEN coord_in = STRUPCASE(STRMID(strtrim(SXPAR(header,'COORDSYS',COUNT=flag_coord),2),0,1))
if (flag_coord eq 0 or strtrim(coord_in) eq '') then coord_in = 'G' ; nothing in the header -> assume galactic
coord_out = coord_in
if N_elements(coord) EQ 1 then coord_out = STRUPCASE(STRMID(coord(0),0,1))
if N_elements(coord) EQ 2 then begin 
    coord_in  = STRUPCASE(STRMID(coord(0),0,1)) 
    coord_out = STRUPCASE(STRMID(coord(1),0,1))
endif
if coord_in  EQ 'C' then coord_in =  'Q'  ; cgis skyconv coding convention for celestial/equatorial
if coord_out EQ 'C' then coord_out = 'Q'
print,'input file : ',decode_coord(coord_in)+' coordinates'
print,'plot coord : ',decode_coord(coord_out)+' coordinates'
do_conv = (coord_in NE coord_out)

; ---- extra rotation ----
if defined(rot_ang) then rot_ang = ([rot_ang,0.,0.])(0:2) else rot_ang = [0., 0., 0.]
; eul_mat = euler_matrix(-rot_ang(0), -rot_ang(1), -rot_ang(2), /Deg, /ZYX)
eul_mat = euler_matrix_new(rot_ang(0), -rot_ang(1), rot_ang(2), /Deg, /ZYX)
do_rot = (TOTAL(ABS(rot_ang)) GT 1.e-5)

; ---- resolution parameter ----
npix = (size(data))[1]
if (keyword_set(quadcube)) then begin
    pix_param = ROUND(ALOG(npix/6.)/ALOG(4.)) + 1 ; resolution parameter
endif else begin
    nside = npix2nside(npix)
    pix_param = nside
endelse

return
end
