pro preview_file, file, ps=ps, gif=gif, png=png, landscape=landscape
;+
; NAME:
;     preview_file
;
; PURPOSE:
;     previews Gif, of Png or Postscript files created by Healpix
;     visualisation routines
;
; CATEGORY:
;     visualisation
;
;
; CALLING SEQUENCE:
;     preview_file, file, GIF=, LANDSCAPE=, PNG=, PS=
;
; INPUTS:
;     file
;
; OPTIONAL INPUTS:
;
; 
; KEYWORD PARAMETERS:
;        GIF
;        LANDSCAPE
;        PNG
;        PS
;
; OUTPUTS:
;        None
;
; OPTIONAL OUTPUTS:
;        None
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;        Spawn an external code
;
; RESTRICTIONS:
;       Requires the definition file idl_default_previewer
;
; PROCEDURE:
;
; EXAMPLE:
;
;
;
; MODIFICATION HISTORY:
;        Eric Hivon, Dec 2002, McM
;-


; ****************************************************************

; papersize : paper size used
; media : either paper size or blank (in which case BBox is used)
; ps_scom : short name of the PS viewing command
; ps_com  : full name of the PS viewing command
; gif_scom : short name of the PNG/GIF viewing command
; gif_com  : full name of the PNG/GIF viewing command


@idl_default_previewer

do_bbox = 0
if defined(media) then begin
    do_bbox = (strcompress(media,/remove_all) eq '')
endif

fullfile = file
blank=' '

; POSTSCRIPT
; ==========


case ps_scom of
    'gv' : begin
    ; gv
        ps_pps = '  -media '+papersize+blank
        ps_lsc = ' -seascape ' 
        ps_ext = ' ' ; extra options
    end
    'ghostview ' : begin
    ; ghostview
        ps_pps = '  -'+papersize+blank
        ps_lsc = ' -seascape' 
        ps_ext = ' '
    end
    'gs' : begin
    ; ghostscript
        ps_pps = ' -sPAPERSIZE='+papersize+blank
        ps_lsc = ' '
        ps_ext = ' '
    end
    else : begin
    ; unknown : neutral options
        ps_pps = ' '
        ps_lsc = ' '
        ps_ext = ' '
    end
endcase

if do_bbox then ps_pps = ' ' ; use default BBox
ps_opt = ps_ext + ps_pps
;;;print,ps_opt+ps_lsc

; PNG and GIF
; ===========

; gif_com = 'xv '
; gif_com = 'gimp '
; gif_com = 'kpaint '
; gif_com = 'kview '
; gif_opt = ' '
if (keyword_set(gif) or keyword_set(png)) then begin
  case gif_scom of
    'netscape' : begin
        cd, current=cwd ; find full path to current directory
        fullpath = filepath(file,root=cwd) ; build full path to file (OS dependent)
        junk=findfile(fullpath,count=count)
        if (count eq 0) then fullpath=file ; file is already a full path
        fullfile = 'file:'+fullpath
        gif_opt = ' '
    end
    else : begin
        gif_opt = ' '
    end
  endcase
endif


; background task
; ===============

os_family = strlowcase(!VERSION.OS_FAMILY)
case os_family of
    'unix' : begin
        bg = ' & '
    end
    'vms' : begin
        bg = '   ' ; don't know
    end
    'windows' : begin
        bg = '   ' ; don't know
    end
    else : begin
        bg = '   '
    end
endcase

; **************************************************************
;   construct command
; **************************************************************

; POSTSCRIPT
if (keyword_set(ps)) then begin
    if ( not keyword_set(landscape)) then ps_lsc = ' '
    command = ps_com + ps_opt + ps_lsc + blank + fullfile + bg
endif

; PNG and GIF
if (keyword_set(gif) or keyword_set(png)) then begin
    command = gif_com + blank + gif_opt + blank + fullfile + bg
endif

; **************************************************************
;   execute command
; **************************************************************

spawn, command

return
end

