pro vec2moll, vec, u, v, flip=flip
;+
; VEC2MOLL
;  returns u,v position on Mollweide map corresponding to 3D vector 'vec'
; 
; VEC2MOLL, vec, u, v
;
;-

if keyword_set(flip) then flipconv = +1 else flipconv = -1 ; longitude increase leftward by default (astro convention)
halfpi = !pi / 2.

common vec2mollcom, ntab, vtab, ztab
if undefined(ntab) then begin
    ntab = 180L * 10
    vtab = 2.*FINDGEN(ntab)/(ntab-1) - 1 ; in [-1,1]
    ztab = (ASIN(vtab) + vtab*SQRT(1-vtab^2))/halfpi ; in [-1,1]
endif


np1 = n_elements(vec)/3
vec = reform(vec,np1,3,/Over)

phi = ATAN(vec(*,1), vec(*,0))/ halfpi ; in [-2, 2]

v = INTERPOL(vtab,ztab,vec(*,2)/SQRT(TOTAL(vec^2,2)))
u = flipconv* phi * SQRT(1-v^2) ; minus sign : astro convention


return
end
