
/*
 * Copyright (c) 2003, 2006 Matteo Frigo
 * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* $Id: sighandler.h,v 1.1 2006/03/31 00:42:50 korpela Exp $ */

/* Module local signal handlers.  Yes, all defined static, not extern because
 * we don't want to worry about signal handlers in one module clobbering 
 * saved signal handlers in another module, and we don't want to invent a 
 * stack of signal handlers.
 */

#ifndef SIGHANDLER_H
#define SIGHANDLER_H

#ifdef ENABLE_SIGNAL_HANDLER
#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif
#ifdef HAVE_SETJMP_H
#include <setjmp.h>
#endif

#ifdef __linux__
#undef SIGRTMAX
#define SIGRTMAX 64
#endif

#ifndef SIGRTMAX
#if defined(_SIGRTMAX)
#define SIGRTMAX _SIGRTMAX
#elif defined(NSIG)
#define SIGRTMAX (NSIG-1)
#else
#define SIGRTMAX 32
#endif
#endif

static jmp_buf jb;

typedef void (*signal_handler)(int);

/* all our signal handler does is jump to a place we've set */
static void sighandler(int x) {
    UNUSED(x);
    longjmp(jb,1);
}

/* flags to mark whether we've been installed already */
static int installed[SIGRTMAX+1];

/* a place to save our the existing signal handlers   */
static signal_handler oldhandler[SIGRTMAX+1];

/* these are the signals we'll handle */
static const int install_handler[]={
#ifdef SIGILL
    SIGILL,
#endif
#ifdef SIGPRIV
    SIGPRIV,
#endif
#ifdef SIGSEGV
    SIGSEGV,
#endif
#ifdef SIGBUS
    SIGBUS,
#endif
#ifdef SIGIOT
    SIGIOT,
#endif
#ifdef SIGABRT
    SIGABRT,
#endif
#ifdef SIGSTKFLT
    SIGSTKFLT,
#endif
    0
};
#endif

/* Keep these as stubs even if signal handling is not enabled */
static inline void install_sighandler() {
#ifdef ENABLE_SIGNAL_HANDLER
  int i=0,sig;
#ifdef _WIN32
  install_windows_exception_handler();
#endif
  while ((sig=install_handler[i])!=0) {
    if (!installed[sig]) {
      oldhandler[sig]=signal(sig,sighandler);
      installed[sig]=1;
    }
    i++;
  }
#endif /* ENABLE_SIGNAL_HANDLER */
}

static inline void reinstall_sighandler() {
#ifdef ENABLE_SIGNAL_HANDLER
  int i=0,sig;
  while ((sig=install_handler[i])!=0) {
    if (installed[sig]) {
      signal(sig,sighandler);
    }
    i++;
  }
#endif /* ENABLE_SIGNAL_HANDLER */
}

static inline void uninstall_sighandler() {
#ifdef ENABLE_SIGNAL_HANDLER
  /* Don't uninstall the windows signal handler! */
  int i=0,sig;
  while ((sig=install_handler[i])!=0) {
    if (installed[sig]) {
      signal(sig,oldhandler[sig]);
      installed[sig]=0;
    }
    i++;
  }
#endif /* ENABLE_SIGNAL_HANDLER */
}

#endif /* SIGHANDLER_H */
