#ifndef _CPUID_H_
#define _CPUID_H_

#ifdef _MSC_VER
#ifndef inline
#define inline __inline
#endif
#endif

static inline int is_386() {
#ifdef _MSC_VER
    unsigned int result,test;
    _asm {
        pushfd
        pop eax
        mov edx,eax
        xor eax,40000h
        push eax
        popfd
        pushfd
        pop eax
        push edx
        popfd
        mov test,edx
        mov result,eax
    }
#else
    register unsigned int result,test;
    __asm__ (
        "pushfl\n\t"
        "popl %0\n\t"
        "movl %0,%1\n\t"
        "xorl $0x40000,%0\n\t"
        "pushl %0\n\t"
        "popfl\n\t"
        "pushfl\n\t"
        "popl %0\n\t"
        "pushl %1\n\t"
        "popfl"
    : "=r" (result), "=r" (test) /* output */
    :  /* no inputs */
    );
#endif
    return (result == test);
}

static inline int has_cpuid() {
#ifdef _MSC_VER
    unsigned int result,test;
    _asm {
        pushfd
        pop eax
        mov edx,eax
        xor eax,200000h
        push eax
        popfd
        pushfd
        pop eax
        push edx
        popfd
        mov test,edx
        mov result,eax
    }
#else
    register unsigned int result,test;
    __asm__ (
        "pushfl\n\t"
        "pop %0\n\t"
        "movl %0,%1\n\t"
        "xorl $0x200000,%0\n\t"
        "pushl %0\n\t"
        "popfl\n\t"
        "pushfl\n\t"
        "popl %0\n\t"
        "pushl %1\n\t"
        "popfl"
    : "=r" (result), "=r" (test) /* output */
    : /* no inputs */
    );
#endif
    return (result != test);
}

typedef struct {
    int eax, ebx, ecx, edx;
} cpuid_regs;

static inline cpuid_regs cpuid(int op) {
    cpuid_regs result;
#ifdef _MSC_VER
    _asm {
        push ebx
        mov eax,op
        cpuid
        mov result.eax,eax
        mov result.ebx,ebx
        mov result.ecx,ecx
        mov result.edx,edx
        pop ebx
    }
    return result;
#else
    register unsigned int dummy;
    __asm__(
#ifdef __linux__   // And any other ABI that requires ebx be preserved for PIC
	"movl %%ebx,%4\n\t"
#endif
        "cpuid"
#ifdef __linux__ 
        "\n\tmov %%ebx,%1\n\t"
	"movl %4,%%ebx"
#endif
    : "=a" (result.eax), 
#ifdef __linux__
      "=r" (result.ebx),
#else
      "=b" (result.ebx),
#endif
      "=c" (result.ecx), 
      "=d" (result.edx),
      "=g" (dummy)
    : "a" (op));
    return result;
#endif
}

#define cpuid_eax(x) cpuid(x).eax
#define cpuid_ebx(x) cpuid(x).ebx
#define cpuid_ecx(x) cpuid(x).ecx
#define cpuid_edx(x) cpuid(x).edx

#endif
