#include <cstdio>
#include "healpix_base.h"
#include "setilib.h"

void get_disc(double ra, double decl, double radius, std::vector<int> &listpix);

static const double D2R=(M_PI/180.0);  // decimal degrees to radians 

Healpix_Base sky(2048, NEST, SET_NSIDE);  // a pixelated sky

int main (int argc, char ** argv) {

    double radius=0.1;                        // disk radius in degrees
    std::vector<int> listpix;
    double ra=8.0, decl=18.0, theta, phi;
    int i;

    for (i=1; i<argc; i++) {
        if (!strcmp(argv[i], "-ra")) {
            ra = atof(argv[++i]);
        } else if (!strcmp(argv[i], "-decl")) {
            decl = atof(argv[++i]);
        } else if (!strcmp(argv[i], "-radius")) {
            radius = atof(argv[++i]);
        } else {
            fprintf(stderr, "bad cmdline arg: %s\n", argv[i]);
            exit(1);
        }
    }

    get_disc(ra, decl, radius, listpix);

    printf("disc contains %d elements\n", listpix.size());
    for(i=0; i < listpix.size(); i++) {
        co_pix2radecl(ra, decl, listpix[i]);
        printf("%3d  %lf  %lf  %d\n", i, ra, decl, listpix[i]);
    }
}

void get_disc(double ra, double decl, double radius, std::vector<int> &listpix) {

    double theta, phi;

    theta = D2R * (90.0 - decl);
    phi   = D2R * (ra * 15.0);

    pointing coord(theta, phi);

    sky.query_disc (coord, D2R*radius, listpix);
}
