//____________________________________________________________________
// Source Code Name    : s4freq.cc
// Programmer          : Donnelly/Cobb et al
//                     : Project SERENDIP, University of California, Berkeley CA
// Compiler            : GNU gcc
// Date Opened         : 10/94
// Purpose             : frequency calculation functions
// History             :
// jeffc     12/97     Added 2 routines : fq_GetBrdFrq() and 
//                     fq_CalcBirdie().
//____________________________________________________________________
// Modification History:
//   -brendan 12/94 - added fq_s3fcalc() function
//   -jeffc   12/98 - board table is now 2d
//____________________________________________________________________

#include <stdio.h>
#include "SIREN.h"
#include "s4boardtable.h"
#include "s4tel.h"

//____________________________________________________________________
// fq_s4fcalc()
// calculate frequency for serendip 4
//____________________________________________________________________

double fq_s4fcalc(double        FirstLO, 
                  double        SecondLO,
                  unsigned long FftBdFreq, 
                  long          BinNumber,
                  char          TID)
 
  {
  if (TID == AO_430 || TID == AO_1420) 
       return( (1e6 * FirstLO)      + 
               (1e6 * SecondLO)     + 
               (1000.0 * FftBdFreq) +
               (0.596046447 * BinNumber)) ;
  else return 0;
  } 

//____________________________________________________________________
// fq_GetBrdFrq()
// Get board center frequency for serendip 4
//____________________________________________________________________

unsigned long fq_GetBrdFrq(long BoardTab, unsigned char BoardId)
  {
  return(BoardTable[BoardTab][BoardId].lSubBandingLo +
         BoardTable[BoardTab][BoardId].lFilterBoardLo);
  }

//____________________________________________________________________
// fq_s3fcalc()
// calculate frequency for serendip 3. Used by io_GetNextRecord to 
// populate a data structure from a record type 'B' (s3 data).
//____________________________________________________________________

double fq_s3fcalc(double lo, long synth, long binno)
  {
  return((1e6 * lo) +
         (1000.0 * (double) synth) +
         (0.596046447 * (double) binno));
  }
 
//____________________________________________________________________
// fq_CalcBirdie()
// Given s4 synth setting and board number, calculates bin number
// where birdie should be found.
//____________________________________________________________________

void fq_CalcBirdie(double Synth, long * BirdieBoard, double * BirdieBin,
                   long BoardTab)
  {
  double FirstBoardCenter;
  double BinWidth, Offset;

  FirstBoardCenter = fq_GetBrdFrq(BoardTab, 0) * 1e3;
  BinWidth = (double)BOARD_WIDTH / (double)NUM_FFT_BINS;

  Offset = Synth - FirstBoardCenter;
  *BirdieBoard = nlong(Offset / BOARD_WIDTH);
  *BirdieBin = (Offset - (*BirdieBoard * BOARD_WIDTH)) / BinWidth;
  }

