// Source Code Name  : s4boardtable.cc
// Programmer        : Jeff Cobb
//                   : Project SERENDIP, University of California, Berkeley CA
// Compiler          : GNU gcc
// Date Opened       : 12/97
//
// History:
// 12/18/99  v1.1    jeffc	Made the board table 2d to handle different
//				receiver setups at AO.  Added routine
//				GetBoardTab() to obtain the index to the
//				correct board table.
// 3/5/99    v1.2    jeffc      removed bd_GetBoardTab()
// This table contains values indexed by board number.

#define BOARD_TABLE
#include "SIREN.h"
#include "s4boardtable.h"
#include "s4tel.h"

#define POLARIZATION 'L'
#define AO430_SUB_BANDING_LO_0   180000
#define AO1420_SUB_BANDING_LO_0  220000
#define SUB_BANDING_LO_1         240000
#define SUB_BANDING_LO_2         260000
#define SUB_BANDING_LO_3         280000
#define SUB_BANDING_LO_4         300000
#define FILTER_BOARD_LO_0         -7500
#define FILTER_BOARD_LO_1         -5000
#define FILTER_BOARD_LO_2         -2500
#define FILTER_BOARD_LO_3             0 
#define FILTER_BOARD_LO_4          2500
#define FILTER_BOARD_LO_5          5000
#define FILTER_BOARD_LO_6          7500
#define FILTER_BOARD_LO_7         10000

BoardInfo BoardTable[NUM_BOARD_TABLES][NUM_BOARDS_IN_TABLE] =
  {
  {
  {0,    AO430_SUB_BANDING_LO_0, FILTER_BOARD_LO_0, 0, POLARIZATION},
  {1,    AO430_SUB_BANDING_LO_0, FILTER_BOARD_LO_1, 0, POLARIZATION},
  {2,    AO430_SUB_BANDING_LO_0, FILTER_BOARD_LO_2, 0, POLARIZATION},
  {3,    AO430_SUB_BANDING_LO_0, FILTER_BOARD_LO_3, 0, POLARIZATION},
  {4,    AO430_SUB_BANDING_LO_0, FILTER_BOARD_LO_4, 0, POLARIZATION},
  {5,    AO430_SUB_BANDING_LO_0, FILTER_BOARD_LO_5, 0, POLARIZATION},
  {6,    AO430_SUB_BANDING_LO_0, FILTER_BOARD_LO_6, 0, POLARIZATION},
  {7,    AO430_SUB_BANDING_LO_0, FILTER_BOARD_LO_7, 0, POLARIZATION},

  {8,    SUB_BANDING_LO_1, FILTER_BOARD_LO_0, 1, POLARIZATION},
  {9,    SUB_BANDING_LO_1, FILTER_BOARD_LO_1, 1, POLARIZATION},
  {10,   SUB_BANDING_LO_1, FILTER_BOARD_LO_2, 1, POLARIZATION},
  {11,   SUB_BANDING_LO_1, FILTER_BOARD_LO_3, 1, POLARIZATION},
  {12,   SUB_BANDING_LO_1, FILTER_BOARD_LO_4, 1, POLARIZATION},
  {13,   SUB_BANDING_LO_1, FILTER_BOARD_LO_5, 1, POLARIZATION},
  {14,   SUB_BANDING_LO_1, FILTER_BOARD_LO_6, 1, POLARIZATION},
  {15,   SUB_BANDING_LO_1, FILTER_BOARD_LO_7, 1, POLARIZATION},

  {16,   SUB_BANDING_LO_2, FILTER_BOARD_LO_0, 2, POLARIZATION},
  {17,   SUB_BANDING_LO_2, FILTER_BOARD_LO_1, 2, POLARIZATION},
  {18,   SUB_BANDING_LO_2, FILTER_BOARD_LO_2, 2, POLARIZATION},
  {19,   SUB_BANDING_LO_2, FILTER_BOARD_LO_3, 2, POLARIZATION},
  {20,   SUB_BANDING_LO_2, FILTER_BOARD_LO_4, 2, POLARIZATION},
  {21,   SUB_BANDING_LO_2, FILTER_BOARD_LO_5, 2, POLARIZATION},
  {22,   SUB_BANDING_LO_2, FILTER_BOARD_LO_6, 2, POLARIZATION},
  {23,   SUB_BANDING_LO_2, FILTER_BOARD_LO_7, 2, POLARIZATION},
 
  {24,   SUB_BANDING_LO_3, FILTER_BOARD_LO_0, 3, POLARIZATION},
  {25,   SUB_BANDING_LO_3, FILTER_BOARD_LO_1, 3, POLARIZATION},
  {26,   SUB_BANDING_LO_3, FILTER_BOARD_LO_2, 3, POLARIZATION},
  {27,   SUB_BANDING_LO_3, FILTER_BOARD_LO_3, 3, POLARIZATION},
  {28,   SUB_BANDING_LO_3, FILTER_BOARD_LO_4, 3, POLARIZATION},
  {29,   SUB_BANDING_LO_3, FILTER_BOARD_LO_5, 3, POLARIZATION},
  {30,   SUB_BANDING_LO_3, FILTER_BOARD_LO_6, 3, POLARIZATION},
  {31,   SUB_BANDING_LO_3, FILTER_BOARD_LO_7, 3, POLARIZATION},
 
  {32,   SUB_BANDING_LO_4, FILTER_BOARD_LO_0, 4, POLARIZATION},
  {33,   SUB_BANDING_LO_4, FILTER_BOARD_LO_1, 4, POLARIZATION},
  {34,   SUB_BANDING_LO_4, FILTER_BOARD_LO_2, 4, POLARIZATION},
  {35,   SUB_BANDING_LO_4, FILTER_BOARD_LO_3, 4, POLARIZATION},
  {36,   SUB_BANDING_LO_4, FILTER_BOARD_LO_4, 4, POLARIZATION},
  {37,   SUB_BANDING_LO_4, FILTER_BOARD_LO_5, 4, POLARIZATION},
  {38,   SUB_BANDING_LO_4, FILTER_BOARD_LO_6, 4, POLARIZATION},
  {39,   SUB_BANDING_LO_4, FILTER_BOARD_LO_7, 4, POLARIZATION},
  },
  {
  {0,    AO1420_SUB_BANDING_LO_0, FILTER_BOARD_LO_0, 0, POLARIZATION},
  {1,    AO1420_SUB_BANDING_LO_0, FILTER_BOARD_LO_1, 0, POLARIZATION},
  {2,    AO1420_SUB_BANDING_LO_0, FILTER_BOARD_LO_2, 0, POLARIZATION},
  {3,    AO1420_SUB_BANDING_LO_0, FILTER_BOARD_LO_3, 0, POLARIZATION},
  {4,    AO1420_SUB_BANDING_LO_0, FILTER_BOARD_LO_4, 0, POLARIZATION},
  {5,    AO1420_SUB_BANDING_LO_0, FILTER_BOARD_LO_5, 0, POLARIZATION},
  {6,    AO1420_SUB_BANDING_LO_0, FILTER_BOARD_LO_6, 0, POLARIZATION},
  {7,    AO1420_SUB_BANDING_LO_0, FILTER_BOARD_LO_7, 0, POLARIZATION},

  {8,    SUB_BANDING_LO_1, FILTER_BOARD_LO_0, 1, POLARIZATION},
  {9,    SUB_BANDING_LO_1, FILTER_BOARD_LO_1, 1, POLARIZATION},
  {10,   SUB_BANDING_LO_1, FILTER_BOARD_LO_2, 1, POLARIZATION},
  {11,   SUB_BANDING_LO_1, FILTER_BOARD_LO_3, 1, POLARIZATION},
  {12,   SUB_BANDING_LO_1, FILTER_BOARD_LO_4, 1, POLARIZATION},
  {13,   SUB_BANDING_LO_1, FILTER_BOARD_LO_5, 1, POLARIZATION},
  {14,   SUB_BANDING_LO_1, FILTER_BOARD_LO_6, 1, POLARIZATION},
  {15,   SUB_BANDING_LO_1, FILTER_BOARD_LO_7, 1, POLARIZATION},

  {16,   SUB_BANDING_LO_2, FILTER_BOARD_LO_0, 2, POLARIZATION},
  {17,   SUB_BANDING_LO_2, FILTER_BOARD_LO_1, 2, POLARIZATION},
  {18,   SUB_BANDING_LO_2, FILTER_BOARD_LO_2, 2, POLARIZATION},
  {19,   SUB_BANDING_LO_2, FILTER_BOARD_LO_3, 2, POLARIZATION},
  {20,   SUB_BANDING_LO_2, FILTER_BOARD_LO_4, 2, POLARIZATION},
  {21,   SUB_BANDING_LO_2, FILTER_BOARD_LO_5, 2, POLARIZATION},
  {22,   SUB_BANDING_LO_2, FILTER_BOARD_LO_6, 2, POLARIZATION},
  {23,   SUB_BANDING_LO_2, FILTER_BOARD_LO_7, 2, POLARIZATION},

  {24,   SUB_BANDING_LO_3, FILTER_BOARD_LO_0, 3, POLARIZATION},
  {25,   SUB_BANDING_LO_3, FILTER_BOARD_LO_1, 3, POLARIZATION},
  {26,   SUB_BANDING_LO_3, FILTER_BOARD_LO_2, 3, POLARIZATION},
  {27,   SUB_BANDING_LO_3, FILTER_BOARD_LO_3, 3, POLARIZATION},
  {28,   SUB_BANDING_LO_3, FILTER_BOARD_LO_4, 3, POLARIZATION},
  {29,   SUB_BANDING_LO_3, FILTER_BOARD_LO_5, 3, POLARIZATION},
  {30,   SUB_BANDING_LO_3, FILTER_BOARD_LO_6, 3, POLARIZATION},
  {31,   SUB_BANDING_LO_3, FILTER_BOARD_LO_7, 3, POLARIZATION},
 
  {32,   SUB_BANDING_LO_4, FILTER_BOARD_LO_0, 4, POLARIZATION},
  {33,   SUB_BANDING_LO_4, FILTER_BOARD_LO_1, 4, POLARIZATION},
  {34,   SUB_BANDING_LO_4, FILTER_BOARD_LO_2, 4, POLARIZATION},
  {35,   SUB_BANDING_LO_4, FILTER_BOARD_LO_3, 4, POLARIZATION},
  {36,   SUB_BANDING_LO_4, FILTER_BOARD_LO_4, 4, POLARIZATION},
  {37,   SUB_BANDING_LO_4, FILTER_BOARD_LO_5, 4, POLARIZATION},
  {38,   SUB_BANDING_LO_4, FILTER_BOARD_LO_6, 4, POLARIZATION},
  {39,   SUB_BANDING_LO_4, FILTER_BOARD_LO_7, 4, POLARIZATION},
  }
  };
