/*
 * Source Code Name  : s4summary.cc
 * Programmer        : Jeff Cobb
 *                   : Project SERENDIP, University of California, Berkeley CA
 * Compiler          : GNU gcc
 * Common database routines for serendip 4 offline programs.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SIREN.h"
#include "s4time.h"
#include "s4database.h"

//===========================================================
unsigned long db_FileNameToKey(char * sDbaseFileName)
//===========================================================
  {

  char Day[3];
  char Year[3];
  char Letter1;
  char Letter2;
  double DummyHour = 0.0;
  unsigned long lDay;
  unsigned long lMonth;
  unsigned long lYear;
  unsigned long MJD;      // modified julian day

  memcpy(&Day, sDbaseFileName+0, 2);
  Day[2] = NULL;
  lDay = (unsigned long)atol(Day);
 
  lMonth = (unsigned long)tm_MonToI(sDbaseFileName);
 
  memcpy(&Year, sDbaseFileName+4, 2);
  Year[2] = NULL;
  lYear = (unsigned long)tm_CentYear(atol(Year));

  // The "+ 0.5" is because julian day starts at noon and we want MJD
  // the fit in a single 24 hour civil day.  Files are named with civil days.
  MJD = (unsigned long)(tm_ToJul(lMonth, lDay, lYear, DummyHour) + 0.5) - 2400000;
 

  Letter1 = sDbaseFileName[6];
  Letter2 = sDbaseFileName[7];
 
  return((MJD                        << 16) +
         ((unsigned long)Letter1     <<  8) +
         (unsigned long)Letter2);
  }

//===========================================================
void db_KeyToFileName(unsigned long Key, char * FileName)
//===========================================================
  {
  unsigned long JD; 
  long lMonth, lDay, lYear;
  double DummyHour;
  char Letter1, Letter2;

  JD      = (Key >> 16) + 2400000;
  Letter1 = (Key >> 8) & 0x000000ff;
  Letter2 = Key & 0x000000ff;

  // We  subtract 0.5 to get JD back to being a real julian day.
  tm_FromJul((double)JD - 0.5, &lMonth, &lDay, &lYear, &DummyHour);

  lYear = tm_TwoDigitYear(lYear);

  sprintf(FileName, "%02ld%s%02ld%c%c", lDay, 
                                        stMonths[lMonth], 
                                        lYear, 
                                        Letter1, 
                                        Letter2); 
  }
