/* ________________________________________________________________________
 programmer : Jeff Cobb
 source name: CmpCandidates.cc
 compiler   : AT&T C++ from SUN
___________________________________________________________________________
 modification history:
 4/96    jeffc - added new fields to header output
 7/98    jeffc - major revision in subset and duplicate group detection.
___________________________________________________________________________
*/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "SIREN.h"
#include "s4freq.h"
#include "s4coord.h"
#include "s4io.h"

//#define DEBUG
//#define OUTPUT_REJECTS

#define MAX_CANDIDATE_GROUPS     10000
#define MAX_CANDIDATES_PER_GROUP 100

long CompareCandidates(double SkyRes,
                      CandidateRec * candidates1, long NumCandidates1,
                      CandidateRec * candidates2, long NumCandidates2);
BOOLEAN IsEventAlgo(unsigned short usAlgo);
int SameCandidate(CandidateRec Candidate1, CandidateRec Candidate2);
BOOLEAN SubSet(long Groups[][MAX_CANDIDATES_PER_GROUP], long Current);
void OutputHeaders(void);
void OutputTop(void);
void OutputGroup(CandidateRec * candidates1, CandidateRec * candidates2,
                 long Groups[][MAX_CANDIDATES_PER_GROUP], long Current);
void OutputRejects(CandidateRec * candidates1, CandidateRec * candidates2,
                   long Groups[][MAX_CANDIDATES_PER_GROUP], long Current);
void PrintGroups(long Groups[][MAX_CANDIDATES_PER_GROUP]);

int TwoFiles = FALSE;

short EOD = -1;

short EventAlgos[8] = {1,2,3,4,5,7,8,-1};

//--------------------------------------------------------------
int main (int argc, char * argv[])
//--------------------------------------------------------------
  {
  FILE * File1;
  FILE * File2;

  char * usage = "usage : s3CmpCandidate sky_res file1 [file2] ";

  int i;
  long CandInFile1;
  long CandInFile2;

  long NumGroups;

  double SkyRes;

  CandidateRec * candidates1;
  CandidateRec * candidates2;

  if(argc != 3 && argc != 4)
    {
    printf("%s\n", usage);
    exit(1);
    }

  SkyRes = atof(argv[1]);

  // Open and count records in file 1.
  if ((File1 = fopen(argv[2],"r")) == NULL)
    {
    fprintf(stderr, "Can't open input file %s.\n", argv[2]);
    exit(1);
    }
  else
    {
    /* Get number of candidate records */ 
    if (!(CandInFile1 = io_CountRecords(File1, CANDIDATE_REC)))
      {
      fprintf(stderr, "No candidate records in input file %s.\n", argv[2]); 
      exit(1);
      }
    else
      if(CandInFile1 == -1)
        {  
         fprintf(stderr, "Can't count records in input file %s.\n", argv[2]);
         exit(1);
         }  
    }


  // Open and count records in file 2, if there is a second file.
  if (argc == 4)
    {
    TwoFiles = TRUE;
    if ((File2 = fopen(argv[3],"r")) == NULL)
      {
      fprintf(stderr, "Can't open input file %s.\n", argv[3]);
      exit(1);
      }
    else
      {
      /* Get number of candidate records */ 
      if (!(CandInFile2 = io_CountRecords(File2, CANDIDATE_REC)))  
        { 
        fprintf(stderr, "No candidate records in input file %s.\n", argv[3]);
        exit(1);
        }
      else 
        if(CandInFile1 == -1)
          {   
          fprintf(stderr, "Can't count records in input file %s.\n", argv[3]); 
          exit(1); 
          }
      }
    }

  /* Allocate space for events array */
  if ((candidates1 =
     (CandidateRec *) calloc((unsigned)CandInFile1, sizeof(CandidateRec))) == NULL)
    {
    fprintf(stderr, "Bad candidate array allocation.\n");
    exit(1);
    }
  if(TwoFiles)
     if ((candidates2 =
        (CandidateRec *) calloc((unsigned)CandInFile2, sizeof(CandidateRec))) == NULL)
      {
      fprintf(stderr, "Bad candidate array allocation.\n");
      exit(1);
      }

  for (i = 0; i < CandInFile1; i++)
    if (! io_GetNextRecord(File1, &candidates1[i], CANDIDATE_REC))
      {
      printf("%s : error in reading cadidate file %s\n", argv[0], argv[2]);
      exit(1);
      }
 
  if(TwoFiles)
    for (i = 0; i < CandInFile2; i++)
      if (! io_GetNextRecord(File2, &candidates2[i], CANDIDATE_REC))
        {
        printf("%s : error in reading cadidate file %s\n", argv[0], argv[3]);
        exit(1);
        }

  OutputTop();
  OutputHeaders();

  if(TwoFiles)
    NumGroups = CompareCandidates(SkyRes, candidates1, CandInFile1, candidates2, CandInFile2);
  else
    NumGroups = CompareCandidates(SkyRes, candidates1, CandInFile1, 
                      candidates1, CandInFile1);

// Output summary
  fprintf(stdout, "# Number of candidates in %s: %ld\n", argv[2], CandInFile1);
  if(TwoFiles)
    {
    fprintf(stdout, "# Number of candidates in %s: %ld\n", argv[3], CandInFile2);
    }
  fprintf(stdout, "# Total number of groups output = %ld\n", NumGroups);

//   for(i = 0; i < CandInFile1; i++)
//     OutputCandidate(candidates1[i]); 

  } // end of main()



//--------------------------------------------------------------
long CompareCandidates(double SkyRes,
                      CandidateRec * candidates1, long NumCandidates1,
                      CandidateRec * candidates2, long NumCandidates2)
//--------------------------------------------------------------
  {
  long i, j, k, Match;
  long NumGroups;

  NumGroups = 0;

  long Groups[MAX_CANDIDATE_GROUPS][MAX_CANDIDATES_PER_GROUP];
  long Current = 0;

  BOOLEAN EventAlgo;

  // Fill in end-of-group indicator (-1)
  for(i = 0; i < MAX_CANDIDATE_GROUPS; i++)
    {
    for(j = 0; j < MAX_CANDIDATES_PER_GROUP; j++)
      Groups[i][j] = -1;
    }

  /* For each candidate in the first file, we do some intializing
     and then loop through all the events in the second file (which
     may in fact be the same as the first file).  In this inner
     loop we see if the candidates being compared are within SkyRes
     degrees of each other on the sky (taking care that we are not
     comparing a candidate against itself).  A positive match results
     in the formation of a "current group".  The current group is stored
     by adding all matches from candidates2 to the Groups[][] array.  The
     base candidate (ie from candidates1) is then added as the final
     candidate of the current group.  EventAlgo tracks whether or not at 
     least 1 of the current group is an "event" oriented candidate, (a signal
     rather than a star or planetary system).  Once TRUE, EventAlgo stays
     TRUE for the duration of the current group.
     NOTE: one might be tempted, in the "1 file" case, to start comparing
           forward of the base candidate, ie not go back and start comparing
           from the beginning.  This would not be correct, because we are
           looking for canadidates clustered around the base candidate.
           This demands a complete search for each and every base
           candidate.  Example:
           c1-----------c2---------------c3
           Suppose c2 is within SkyRes of c1, and c3 is not with SkyRes
           of c1, but both c1 and c3 are within SkyRes od c2.  The group
           c2c1c3 will only be found if we search from the beginning
           when c2 is the base candidate.
  */
  for(i = 0; i < NumCandidates1; i++)
    {
    EventAlgo = IsEventAlgo(candidates1[i].usAlgo);
    Match = FALSE;

    for(j = TwoFiles ? 0 : 0, k = 0; j < NumCandidates2; j++)
      {
      if(co_SkyAngle(candidates1[i].dbRa, candidates1[i].dbDec,
                   candidates2[j].dbRa, candidates2[j].dbDec) <= SkyRes
        &&
         !SameCandidate(candidates1[i], candidates2[j]))
        {
        Match = TRUE;
        EventAlgo = IsEventAlgo(candidates2[j].usAlgo) ? TRUE : EventAlgo;
#ifdef DEBUG
        fprintf(stderr,"Match.  EventAlgo = %d  Current = %ld\n", EventAlgo, Current);
#endif
        Groups[Current][k]  = j;
        k++;
        }
      }

    if(Match)
      {
      if(EventAlgo)
        {
        Groups[Current][k] = i;      // Base event to current group
        Current++;                   // Next group
        }
      else
        {
        // No match, so "erase" current group, space ready to re-use
        for(j = 0; Groups[Current][j] != -1; j++)
          Groups[Current][j]  = -1;
        }  // End !EventAlgo
      }  // End Match
    }  // End loop through NumCandidates1

#ifdef DEBUG
  PrintGroups(Groups);
#endif

  for(Current = 0; Groups[Current][0] != -1; Current++)
    {
    if(TwoFiles || !SubSet(Groups, Current))   // Two files cannot have subsets
      {
      OutputGroup(candidates1, candidates2, Groups, Current);
      NumGroups++;
      if((NumGroups % 10) == 0)
        OutputHeaders();
      }
    else
      OutputRejects(candidates1, candidates2, Groups, Current);
    }
    
  return NumGroups;
  }

//--------------------------------------------------------------
BOOLEAN SubSet(long Groups[][MAX_CANDIDATES_PER_GROUP], long Current)
//--------------------------------------------------------------
/* NOTE: we do not, nor is there need to, call this
         routine if we are processing 2 files.  With 2 files,
         each base candidate is a unique candidate from file 1. 
         Thus, there can be no subsets.  If you *were* to call this
         routine with 2 files, replace the line:
         for(j = 0; Groups[Check][j] != -1; j++)        // level 3
         with:
         for(j = 0; Groups[Check][TwoFiles ? j+1 : j] != -1; j++) // level 3
         to stop one short of the unique "file 1" candidate.
*/
  {
  BOOLEAN ReturnValue;
  long i, j, maxj, Check;

  ReturnValue = FALSE;


 /* See if the current group is a complete or partial
    subset of any other group.  "Level 1" loops
    through all groups.  "Level 2" loops through
    all events of the curent group.  "Level 3" loops
    through all events of each "other" group. If
    we find a match in level 3, we go back to level2
    for the next event of the current group.  If we
    fall out of level 3 because of end-of-group (-1),  
    we go back to level 1 for the next subsequent group.
    If we *finish* level 2 *and* have landed at the 
    current group's end-of-group, that means we have 
    found another group that is a duplicate or 
    superset of the current group.  If this "check"
    group is a superset of the current group OR if
    the current group is not the *1st* duplicate, we
    return TRUE.  Otherwise, we return FALSE.
 */
 for(Check = 0; Groups[Check][0] != -1 && 
                ReturnValue != TRUE;      Check++)    // level 1
   if(Check != Current)      // Don't check a group against itself!                
     {
     for(i = 0; Groups[Current][i] != -1; i++)        // level 2
       {
       for(j = 0; Groups[Check][j] != -1; j++)        // level 3
         if(Groups[Check][j] == Groups[Current][i])
           break;                                     // back to level 2
       if(Groups[Check][j] == -1)                     
         break;                                       // back to level 1
       }
     if(Groups[Current][i] == -1)
       {
       for(maxj = 0; Groups[Check][maxj] != -1; maxj++); // get size of chk grp
       if(i != maxj || Check < Current)  // subset IF check is superset OR 
                                         // current NOT 1st dup (1st dup OK)
         {
         ReturnValue = TRUE;
         break;                                       // fini
         } // End i != maxj ...
       } // End Groups[Current][i] == -1
     } // End Check != Current (end of block for each check group)

#ifdef DEBUG
  fprintf(stderr,"Check = %ld  %ld\n", Check, Groups[Check][0]);
#endif

  return ReturnValue;
  }

//--------------------------------------------------------------
int SameCandidate(CandidateRec Candidate1, CandidateRec Candidate2)
//--------------------------------------------------------------
  {
  if(Candidate1.usAlgo        == Candidate2.usAlgo         &&
     Candidate1.dbRa          == Candidate2.dbRa           &&
     Candidate1.dbDec         == Candidate2.dbDec          &&
     Candidate1.dbSkySep      == Candidate2.dbSkySep       &&
     Candidate1.dbFreq        == Candidate2.dbFreq         &&
     Candidate1.lFreqSep      == Candidate2.lFreqSep       &&
     Candidate1.lHiPower      == Candidate2.lHiPower       &&
     Candidate1.lMaxSpectra1  == Candidate2.lMaxSpectra1   &&
     Candidate1.lMaxSpectra2  == Candidate2.lMaxSpectra2   &&
     Candidate1.lNumObs       == Candidate2.lNumObs        &&
     Candidate1.lLineNum      == Candidate2.lLineNum)
    return TRUE;
  else
    return FALSE;
  }


//--------------------------------------------------------------
void OutputHeaders(void)
//--------------------------------------------------------------
  {
  fprintf
    (stdout, "# A            Dec               Freq.             Power  Max  #Detect      Bfit           Distance      #Obs\n");
  fprintf
    (stdout, "#       Ra            Sky Sep.            Freq. Sep.    Spectra      Line#         Object           SpC         Score\n");
  fprintf
    (stdout, "#                    (degrees)   (MHz)      (Hz)        One Two                            (Parsecs)\n");
  }

//--------------------------------------------------------------
void OutputTop(void)
//--------------------------------------------------------------
  {
  fprintf(stdout, "# Algorithms:\n");
  fprintf(stdout, "# 00 : The ________ star catalog in candidate form.\n");
  fprintf(stdout, "# 01 : exdate\n"); 
  fprintf(stdout, "# 02 : multhit\n");
  fprintf(stdout, "# 03 : multhit.exdate\n");
  fprintf(stdout, "# 04 : limpwr\n");
  fprintf(stdout, "# 05 : limpwr.exdate\n");
  fprintf(stdout, "# 06 : Globular clusters\n");
  fprintf(stdout, "# 07 : beamfit\n");
  fprintf(stdout, "# 08 : beamfit.exdate\n");
  fprintf(stdout, "# 09 : Planet search stars\n");
  fprintf(stdout, "# 10 : Planetary systems\n");
  fprintf(stdout, "#\n");
  }


//--------------------------------------------------------------
void OutputGroup(CandidateRec * candidates1, CandidateRec * candidates2, 
                 long Groups[][MAX_CANDIDATES_PER_GROUP], long Current)
//--------------------------------------------------------------
  {
  long i;

  for(i = 0; Groups[Current][i] != -1; i++)
    {
    if(TwoFiles)      // If we have 2 files, the last candidate in the current 
                      // group is from file 1.
      if(Groups[Current][i+1] == -1)
        {
        io_WriteCandidate(stdout, &candidates1[Groups[Current][i]]);
        continue;
        }
    io_WriteCandidate(stdout, &candidates2[Groups[Current][i]]);
    }
    
  io_WriteComment(stdout, "");
  }

//--------------------------------------------------------------
void OutputRejects(CandidateRec * candidates1, CandidateRec * candidates2,
                   long Groups[][MAX_CANDIDATES_PER_GROUP], long Current)
//--------------------------------------------------------------
  {
#ifdef OUTPUT_REJECTS
  long i;
 
  for(i = 0; Groups[Current][i] != -1; i++)
    io_WriteCandidate(stderr, &candidates2[Groups[Current][i]]);
     
  io_WriteComment(stderr, "");
#endif
  }

//--------------------------------------------------------------
BOOLEAN IsEventAlgo(unsigned short usAlgo)
//--------------------------------------------------------------
  {
  long i;

  BOOLEAN Rtn = FALSE;

  for(i = 0; EventAlgos[i] != EOD; i++)
    if(usAlgo == EventAlgos[i])
      Rtn = TRUE;

  return Rtn;
  }

//--------------------------------------------------------------
void PrintGroups(long Groups[][MAX_CANDIDATES_PER_GROUP])
//--------------------------------------------------------------
  {
  long i, j;
 
  for(i = 0; Groups[i][0] != -1; i++)
    {
    for(j = 0; Groups[i][j] != -1; j++)
      fprintf(stderr, "%5ld", Groups[i][j]);
    fprintf(stderr, "\n");
    }
  }
