/*
Programmer : Jeff Cobb
Source name: s4NetParce.c
Compiler   : gcc 
 
Usage : 
 
Description: 

Programming notes:

*/

/*
#define DEBUG
*/

#include	<unistd.h>
#include	<stdlib.h>
#include	<stdio.h>
#include        <signal.h>
#include        <sys/types.h>
#include        <sys/timeb.h>
#include        <fcntl.h>
#include        <errno.h>
#include	"s4Net.h"

#define DUMMY "dummy"

// Buffer Control
typedef struct
  {
  long  Current;
  long  SeekStop;
  long  WriteStop;
  long  NewData;
  long  NumNew;
  long  BufEnd;
  } BufCtrlBlock;

/* signal responders */
#define If_SIGHUP_THEN_BREAK if(SigHupFlag) break

/* Internal function prototypes */
void sighup_func(void);
void myperror(char * str);
void print_time(void);
void WriteBuffer(unsigned char  *Buf,
                 BufCtrlBlock   *BufCtrl,
                 FILE           *fpo);
BOOLEAN IsNewFileName(unsigned char  *Buf,
                    BufCtrlBlock   *BufCtrl);
void SwitchFiles(FILE * fp, unsigned char *NewFileName);
void sigio_func(void);

/* global variables */
int SigHupFlag;
int SigIoFlag;
long DataLeftInBuf;
BOOLEAN FileNameComplete;
double gMyTime = 0.0;             /* for net timing tests */
double gInterval;                 /* for net timing tests */

unsigned char ucSyncWord[8] = {SYNC_WORD_BYTE_0,
                               SYNC_WORD_BYTE_1,
                               SYNC_WORD_BYTE_2,
                               SYNC_WORD_BYTE_3,
                               SYNC_WORD_BYTE_4,
                               SYNC_WORD_BYTE_5,
                               SYNC_WORD_BYTE_6,
                               SYNC_WORD_BYTE_7};

/* ------------------------------------------------------------- */
main(int argc, char *argv[])
/* ------------------------------------------------------------- */
  {
  int readrtn;

  char * pname;
      
  FILE *fpo;
  FILE *fpi;

  BufCtrlBlock BufCtrl;

  unsigned char Buf[NET_BUFSIZE + SYNC_WORD_LEN + FILENAME_LEN];

  pname = argv[0];                          // prog name for msgs

  if ((fpo = fopen(DUMMY,"w")) == NULL)   // open dummy output
    myperror("server: can't open output file ");
  setbuf(fpo, NULL);                        // unbuffered output

  fpi = stdin;                              // input thru stdin

  BufCtrl.Current  = 0;                     // Init buffer control
  BufCtrl.NewData  = 0;

  SigHupFlag = 0;
  signal(SIGHUP, sighup_func);              // catch HUP signals

  /* ------------------------------------------------------ */
  /* Set up for asynch I/O for input file */
  /* This must also set the file descriptor to be */
  /* non-blocking?  This means that a read will return */
  /* immediately with the number of bytes available, which */
  /* may well be less than the buffer size */
  /* signal(SIGIO, (SIG_PF)sigio_func); */
  signal(SIGIO, sigio_func);
  if(fcntl(fileno(fpi), F_SETOWN, getpid()) < 0)
    myperror("F_SETOWN error");
  if(fcntl(fileno(fpi), F_SETFL, FASYNC) < 0)
    myperror("F_SETFL FASYNC error");

  while(1)
    {
    If_SIGHUP_THEN_BREAK;      // allow graceful exit on "kill -HUP"

    sigblock(sigmask(SIGIO));  // blocks (delays) SIGIO's until
                               // sigsetmask(0), EXCEPT during
                               // sigpause(0).
 
    while(SigIoFlag == 0)      // Wait on a SIGIO.  The (0) in the
      sigpause(0);             // sigpause(0) sez to enable all
                               // signals, including SIGIO.  On return
                               // from sigpause(0), old mask is restored,
                               // thus SIGIO's are once again blocked.
 
    if((BufCtrl.NumNew = read(fileno(fpi), 
                               &Buf[BufCtrl.NewData], 
                               NET_BUFSIZE))             > 0)
      WriteBuffer(Buf, &BufCtrl, fpo);
    else    
      myperror("read error");

    SigIoFlag = 0;
    sigsetmask(0);  // make sure all signals are now allowed
    }

  fclose(fpo);   // close output file

  } /* end main() */


/* ------------------------------------------------------------- */
void WriteBuffer(unsigned char  *Buf,
                 BufCtrlBlock   *BufCtrl,
                 FILE           *fpo)
/* ------------------------------------------------------------- */
  {
  char NewFileName[FILENAME_LEN+1];

  BufCtrl->BufEnd   = BufCtrl->NewData +
                      BufCtrl->NumNew;

  BufCtrl->SeekStop = BufCtrl->NewData +
                      BufCtrl->NumNew  -  
                      SYNC_WORD_LEN    -  
                      FILENAME_LEN;

  while(BufCtrl->Current < BufCtrl->SeekStop)
    {    
    if(IsNewFileName(Buf, BufCtrl))
      {  
      if(BufCtrl->WriteStop > BufCtrl->Current)
        write(fileno(fpo),  
              &Buf[BufCtrl->Current], 
              BufCtrl->WriteStop - BufCtrl->Current);
      memcpy(NewFileName, &Buf[BufCtrl->WriteStop + SYNC_WORD_LEN], FILENAME_LEN);
      NewFileName[FILENAME_LEN] = '\0';
      SwitchFiles(fpo, NewFileName);
      BufCtrl->Current = BufCtrl->WriteStop + SYNC_WORD_LEN + FILENAME_LEN;
      }  
    else
      { 
      if(fpo)
         write(fileno(fpo),  
               &Buf[BufCtrl->Current],
               BufCtrl->SeekStop - BufCtrl->Current);
      BufCtrl->Current = BufCtrl->SeekStop;
      }  
    }    
 
  BufCtrl->Current  = 0; 
  BufCtrl->NewData  = BufCtrl->BufEnd - BufCtrl->SeekStop;
 
  memcpy(Buf,   
         &Buf[BufCtrl->SeekStop],
         BufCtrl->NewData);
  }



/* ------------------------------------------------------------- */
BOOLEAN IsNewFileName(unsigned char  *Buf, 
                    BufCtrlBlock   *BufCtrl)
/* ------------------------------------------------------------- */
  {
  long i;

  for(i = BufCtrl->Current; i < BufCtrl->SeekStop; i++)
    if(Buf[i]   == ucSyncWord[0] &&
       Buf[i+1] == ucSyncWord[1] &&
       Buf[i+2] == ucSyncWord[2] &&
       Buf[i+3] == ucSyncWord[3] &&
       Buf[i+4] == ucSyncWord[4] &&
       Buf[i+5] == ucSyncWord[5] &&
       Buf[i+6] == ucSyncWord[6] &&
       Buf[i+7] == ucSyncWord[7])
      {
      BufCtrl->WriteStop = i;
      return TRUE;
      }

  return FALSE;
  } 

/* ------------------------------------------------------------- */
void SwitchFiles(FILE * fp, unsigned char *NewFileName)
/* ------------------------------------------------------------- */
  {
  fclose(fp);
  if ((fp = fopen(NewFileName,"w")) == NULL)
    myperror("server: can't open NEW file ");
  }


/* ------------------------------------------------------------- */
void myperror(char * str)
/* ------------------------------------------------------------- */
  {
  perror(str);
  exit(1);
  }

/* ------------------------------------------------------------- */
void print_time()
/* ------------------------------------------------------------- */

  {
  double mytime;

  struct timeb mytimeb;

  ftime(&mytimeb);

  mytime = mytimeb.time;
  mytime += mytimeb.millitm / 1000.0;

/*
  fprintf(stderr, "time = %lf\n", mytime - gMyTime);
*/
  gInterval = mytime - gMyTime;
  gMyTime = mytime;

  }


/* ------------------------------------------------------------- */
void sighup_func()
/* ------------------------------------------------------------- */
  {
  SigHupFlag = 1;
  }

/* ------------------------------------------------------------- */
void sigio_func()
/* ------------------------------------------------------------- */
  {
  SigIoFlag = 1;
  }

