/* ________________________________________________________________________
 programmer : Matt Lebofsky
 source name: s4addsky.cc
 compiler   : gcc
___________________________________________________________________________
 modification history:
   version 1.0 finished 6/29/98
___________________________________________________________________________
 usage: s4addskyplot dbase_file
 scans dbase files and adds observation data to s4skyplot dbase.
___________________________________________________________________________
notes/bugs:
___________________________________________________________________________
*/


#include <stdio.h>
#include "SIREN.h"
#include "s4database.h"
#include "s4io.h"
#include "s4utils.h"
#include "s4skyplot.h"

// ------------------------------------------------------------------------
int main (int argc, char *argv[])
// ------------------------------------------------------------------------
{

SkyplotElement *stSkyArray;
int iSkyplotFileNo;
long lArraySize;
FILE *fDbaseFile;
double dbThisSpectra;
long lThisPixel, lTestPixel;
StdEventRec stThisEvent;
char cDbaseFileName[20];

// Why the # 51800? Well, there is at least one dbase file per day, which
// is 86400 seconds long, which means 86400/1.67 = 51736 spectra, and in
// the worst case scenario, it is slewing so fast each spectra is in a
// different pixel. 

double dbRas[51800];
double dbDecs[51800];
long lTotalPixels;

if (argc != 2) {
  fprintf(stderr, "Usage: s4addskyplot dbase_file\n");
  exit(1);
  }

if ((fDbaseFile = fopen(argv[1],"r")) == NULL) {
  fprintf(stderr,"Can't read dbase file: %s\n",argv[1]);
  exit(1);
  }

ut_GetLastField('/',argv[1],cDbaseFileName); /* strip off path to dbase file */

dbThisSpectra = 99999999;      /* to start with - no time could be this */
lThisPixel = -1;
lTotalPixels = 0;

while (io_GetNextRecord(fDbaseFile,&stThisEvent,STDEVENT_REC)) {

  if (dbThisSpectra != stThisEvent.dbJulDate) {
    dbThisSpectra = stThisEvent.dbJulDate;
    lTestPixel = sp_RaDectoElement(stThisEvent.dbRa,stThisEvent.dbDec);
    if (lThisPixel != lTestPixel) {
      lThisPixel = lTestPixel;
      dbRas[lTotalPixels] = stThisEvent.dbRa;
      dbDecs[lTotalPixels] = stThisEvent.dbDec;
      lTotalPixels++;
      }
    }     
  }

if ((lArraySize = sp_ReadSkyplotFile(&stSkyArray,&iSkyplotFileNo)) == -1)
  exit(1);

for (lThisPixel = 0; lThisPixel < lTotalPixels; lThisPixel++) 
  sp_AddObs(dbRas[lThisPixel],dbDecs[lThisPixel],cDbaseFileName,
                &stSkyArray,&lArraySize,iSkyplotFileNo);

fclose(fDbaseFile);
sp_CloseSkyplotFile(iSkyplotFileNo);
}
