// Source Code Name  : s4pcdbase.cc
// Programmer        : Jeff Cobb
//                   : Project SERENDIP, University of California, Berkeley CA
// Compiler          : GNU gcc
/*
 modification history:
 1/10/99	v1.0	jeffc	New program : calculates birdie freq at given synth setting
 3/5/99	    	v1.1	jeffc	changed bd_GetBoardTab() to tel_GetTelTab()
*/

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include "SIREN.h"
#include "s4freq.h"
#include "s4boardtable.h"
#include "s4tel.h"

unsigned char GetTid(char * sReceiverID);

// ---------------------------------------------------------
int main(int argc, char ** argv)
// ---------------------------------------------------------
  {

  long BoardId, BoardTab, BoardFreq;
  double Synth, Freq, Lo1, Lo2, BinNum;
  unsigned char Tid;

  if(argc != 5)
    {
    fprintf(stderr, "Usage : s4calcbirdie Tid Lo1 Lo2 synth\n");
    exit(1);
    }

  Lo1     = atof(argv[2]);
  Lo2     = atof(argv[3]);
  Synth   = atof(argv[4]) * 1e6;

  if((Tid = tel_GetTid(argv[1])) == UNKNOWN_TEL)
    {
    fprintf(stderr, "Bad Tid\n");
    fprintf(stderr, "Usage : s4calcbirdie Tid Lo1 Lo2 synth\n");
    exit(1);
    }

  BoardTab = tel_GetTelTab(Tid);
  fq_CalcBirdie(Synth, &BoardId, &BinNum, BoardTab);
  if(BoardId < 0) {
    fprintf(stdout, "Synth %5.1lf is out of band.\n", Synth/1e6);
    exit(1);
  } 
  BoardFreq = fq_GetBrdFrq(BoardTab, BoardId);
  Freq = fq_s4fcalc(Lo1, Lo2, BoardFreq, (long)BinNum, Tid);

  fprintf(stdout, 
          "synth = %5.1lf MHz, board = %3ld, bin = %11.2lf  freq = %11.6lf MHz\n",
          Synth/1e6, BoardId, BinNum, Freq/1e6);
  }


