/*********************************************************************************/
/* Programmer        : Jeff Cobb
 * Source code name  : s4dupfilt.cc
 * Compiler          : Sun C++
 * Purpose           : reads a file in the dbas format and filters out
 *                     duplicate records.  for each record read in, we
 *                     write it to the output file only if we have not
 *                     already written an exactly matching record.
 *
 *********************************************************************************
 *    modifications:
 *      11/30/93   chuck fixed locos from float to double
 *      2/28/95    scott converted to ANSI-C and S4 standards
 ********************************************************************************* 
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "SIREN.h"
#include "s4io.h"
#include <malloc.h>


int main(int argc, char *argv[])

{

  StdEventRec * events;
  long events_in;
  int i, j;
  FILE * fin_p;

			/* open input file */

  if ((fin_p = fopen(argv[1], "r")) == NULL)
	 {
	 fprintf(stderr, "Can't open input file - %s.\n", argv[3]);
	 exit(1);
	 }

		/* find number of events */

  events_in = (io_CountRecords(fin_p,STDEVENT_REC));

  if (events_in < 0)
          {
          fprintf(stderr, "Trouble counting events in file\n");
          exit(1);
          }

  if (events_in == 0)
          {
          fprintf(stderr, "Empty File\n");
          exit(1);
          }


	  /* determine if too many events */

  if (events_in >= MAX_EVENTS)
	 {
	 fprintf(stderr, "Too many events.\n");
	 exit(1);
	 }

         /* allocate memory for ALL events in file */

  if ((events =
        (StdEventRec *) calloc((unsigned)events_in, sizeof(StdEventRec))) == NULL)
		  {
		  fprintf(stderr, "Bad event memory allocation.\n");
		  exit(1);
		  }

	/* populate event memory with ALL events*/

  for(i=0;i < events_in; i++)
	  if ((io_GetNextRecord(fin_p,&events[i],STDEVENT_REC)) != 1)
	     {
             fprintf(stderr, "Trouble reading events in file.\n");
             exit(1);
	     }

	/* compare event(1) to all other events */

       for(i = 0; i < events_in; i++)
	 {
		for(j = 0; j <= i; j++)
		if((strcmp(events[j].stUTime, events[i].stUTime) == 0) &&
	 (events[j].dbRa == events[i].dbRa) &&
	 (events[j].dbDec == events[i].dbDec) &&
	 (events[j].dbFirstLo == events[i].dbFirstLo) &&
	 (events[j].dbSecondLo == events[i].dbSecondLo) &&
	 (events[j].ulFftBdFreq == events[i].ulFftBdFreq) &&
	 (events[j].lBinNumber == events[i].lBinNumber) &&
	 (events[j].dbBirdieSynth == events[i].dbBirdieSynth) &&
	 (events[j].ucBid == events[i].ucBid) &&
	 (events[j].usPower == events[i].usPower) &&
	 (events[j].usSigmaThreshold == events[i].usSigmaThreshold) &&
	 (events[j].usActualThreshold == events[i].usActualThreshold) &&
	 (events[j].cPolarization == events[i].cPolarization) &&
	 (events[j].usAdc == events[i].usAdc) &&
	 (events[j].usSpctResLog == events[i].usSpctResLog)&&
	 (strcmp(events[j].stDate, events[i].stDate) == 0) &&
	 (events[j].ucTid == events[i].ucTid))
	 break;

	 if(i == j)
		io_WriteStdEvent(stdout,&events[i]);
	}

		/* examine for file close error */

	if (fclose(fin_p) != 0)
		{
		fprintf(stderr,"Error closing file\n");
		exit(1);
		}

		return (0);
}
