//____________________________________________________________________
// Source File Name   : s4fakedata.cc
// Programmer         : Jeff Cobb
// Date opened        : March, 2001
// Purpose            : Generates synthetic s4 data in dbase format. 
//                     
//                      
//____________________________________________________________________

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "SIREN.h"
#include "s4io.h"

#define ROUNDUP a ((a - int(a)) == 0 ? a : (int(a + 1)))

#define NUM_EVENTS_PER_SPECTRA 5
#define THRESHOLD 14
#define TID 1
#define SPECTRALRES 0
#define POLARIZATION 'L'

typedef struct {
	double		StartFreq;		// MHz
	double		DriftRate;		// Hz/s
	double		BandWidth;		// Hz
	long		BinSkip;		// bins
	double		Power;			// units of mean
	double		TimeOffsetStart;	// decimal hours
	double		TimeOffsetStop;		// decimal hours
} Signal_t;

typedef struct {
	double		Threshold;
	double 		StartTime;		// decimal hours
	double 		Duration;		// decimal hours
	double		SlewRate;		// degrees/s
	long		NumSignals;
	Signal_t *	Signal;			// array of signals
} SyntheticData_t;
	 

void GetParameters(char * ParameterFile, SyntheticData_t * SyntheticData);
void PrintParameters(SyntheticData_t SyntheticData);
void InitSynth(SyntheticData_t *  SyntheticData);
void InitSignals(SyntheticData_t SyntheticData);

main(int argc, char ** argv) {

  StdEventRec 		Event;
  SyntheticData_t 	SyntheticData;

  long NoiseContigBin, NoiseBin, NoiseBoard;
  long SignalContigBin, SignalBin, SignalBoard;
  //double NumHours	= 2.0;
  //long NumSpectra 	= (long)((NumHours * 3600) / INTEGRATION_TIME); 
  long NumEventsPerSPectra;
  long SignalStart;
  long SignalStop;
  long SpectraNum;		// spectra loop inc
  long i,j; 
  double DecHour	= 0;
  long Hour 		= 0;
  long Minute 		= 0;
  long Second 		= 0;
  long CentaSecond 	= 0;
  long FftBdFreq	= 212500;
  long SignalWidthInBins;
  double BinZeroFreq	= 1371.25;
  //double SignalFreq 	= 1420.0;
  double RAStart	= 1.0;
  double ThisRA 	= RAStart;
  //double SlewRate	= 0.004;
  //double Threshold	= 14.0;

  if(argc != 2) {
	fprintf(stderr, "Usage: s4fakedata parameter_file\n");
	exit(1);
  }

  InitSynth(&SyntheticData);
  GetParameters(argv[1], &SyntheticData);
  PrintParameters(SyntheticData);

  DecHour = SyntheticData.StartTime;

  NumEventsPerSPectra 	= (long)(exp(-SyntheticData.Threshold) * NUM_BINS * NUMBOARDS);

  srand48(time(NULL));

  for(SpectraNum=0; DecHour - SyntheticData.StartTime < SyntheticData.Duration; SpectraNum++) { 

	// time of new spectra
   	Hour = (long)DecHour;
	Minute = (long)((DecHour - (long)DecHour) * 60);
	Second = (long)((DecHour - (long)DecHour) * 3600) - (Minute * 60);
	CentaSecond = (long)((DecHour - (long)DecHour) * 360000) - (Second * 100 + Minute * 6000); 

	// generate noise
	for(i=0; i<NumEventsPerSPectra; i++) {
  		NoiseContigBin = (long)(drand48() * (HIGH_CONTIGBIN + 1));
		NoiseBoard = NoiseContigBin / (long)NUM_FFT_BINS;
		NoiseBin = (NoiseContigBin % (long)NUM_FFT_BINS) - abs(LOW_BIN);
		Event.dbRa 		= ThisRA;
		Event.dbDec  		= 5.0;
 		Event.dbFirstLo		= 1160.0;
		Event.dbSecondLo	= 0.0;
		Event.ulFftBdFreq	= FftBdFreq + (2500 * NoiseBoard);
		Event.lBinNumber	= NoiseBin;
		Event.dbBirdieSynth	= 310;
		Event.ucBid		= NoiseBoard;
		Event.usPower		= 123;
		Event.usSigmaThreshold	= (short)SyntheticData.Threshold;
		Event.usActualThreshold	= 75;
		Event.cPolarization	= POLARIZATION;
		Event.usAdc		= 10;
		Event.usSpctResLog	= SPECTRALRES;
		Event.ucTid		= TID;
		sprintf(Event.stUTime, "%02d%02d%02d%02d", Hour, Minute, Second, CentaSecond);
		strcpy(Event.stDate, "10ja01");
		io_WriteStdEvent(stdout,&Event);
	}

	// add signals
	for(i=0; i<SyntheticData.NumSignals; i++) {

		if(SyntheticData.Signal[i].TimeOffsetStart != 0.0 &&
		   SyntheticData.Signal[i].TimeOffsetStop  != 0.0) {
			if(SyntheticData.Signal[i].TimeOffsetStart > DecHour - SyntheticData.StartTime ||
		   	   SyntheticData.Signal[i].TimeOffsetStop  < DecHour - SyntheticData.StartTime) {
				continue;
			}
		}  // end checking for signal time bounds

		SignalContigBin 	= (long)(((SyntheticData.Signal[i].StartFreq - BinZeroFreq) * 
						  	1000000) / DELTA_FQ);  // start bin
		SignalContigBin 	+= (long)((SyntheticData.Signal[i].DriftRate / DELTA_FQ) * 
							((DecHour - SyntheticData.StartTime) * 3600));  // drift it

		SignalWidthInBins	= (long)(SyntheticData.Signal[i].BandWidth/DELTA_FQ);

		if(SignalWidthInBins == 1) {
			SignalStart = 0;
		} else {
			SignalStart = -SignalWidthInBins/2;
		}
		SignalStop = SignalStart + SignalWidthInBins;
		for(j=SignalStart ; j<SignalStop; j += 1 + SyntheticData.Signal[i].BinSkip) {
			SignalBoard 		= (SignalContigBin + j) / (long)NUM_FFT_BINS;
        		SignalBin 		= ((SignalContigBin + j) % (long)NUM_FFT_BINS) - abs(LOW_BIN);
        		Event.dbRa              = ThisRA;
        		Event.dbDec             = 5.0;
        		Event.dbFirstLo         = 1160.0;
        		Event.dbSecondLo        = 0.0;
        		Event.ulFftBdFreq       = FftBdFreq + (2500 * SignalBoard);
        		Event.lBinNumber        = SignalBin;
        		Event.dbBirdieSynth     = 310;
        		Event.ucBid             = SignalBoard;
        		Event.usPower           = 456;
			Event.usSigmaThreshold	= (short)SyntheticData.Threshold;
        		Event.usActualThreshold = 75;
        		Event.cPolarization     = POLARIZATION;
        		Event.usAdc             = 10;
        		Event.usSpctResLog      = SPECTRALRES;
        		Event.ucTid             = TID;
        		sprintf(Event.stUTime, "%02d%02d%02d%02d", Hour, Minute, Second, CentaSecond);
        		strcpy(Event.stDate, "10ja01");
			io_WriteStdEvent(stdout,&Event);
	 	}  // end adding signals for width of 1 signal
  	}  // end signals per spectra loop

   	// setup for next spectra
   	DecHour += (INTEGRATION_TIME/ 3600.0);
   	ThisRA = RAStart + SpectraNum * ((SyntheticData.SlewRate * INTEGRATION_TIME)/15.0);

   }  // end synthesizing data for 1 spectra  (main loop)

}  // end main()

void GetParameters(char * ParameterFile, SyntheticData_t * SyntheticData) {

	FILE * ParameterFile_p;
	char PLine[256];
	BOOLEAN Eof = FALSE;
	//long i = 0;

	PLine[0] = '#';


	if((ParameterFile_p = fopen(ParameterFile, "r")) == NULL) {
		fprintf(stderr, "cannot open %s!\n", ParameterFile);
		exit(1);
	}


	while(!Eof) {
		if(fgets(PLine, 255, ParameterFile_p) == NULL) { 
			Eof = TRUE;
			continue;
		}
		if(PLine[0] == '#') continue;

		if(!Eof) {

			if(strstr(PLine, "SYNTH_DATA")) {
				sscanf(PLine, "SYNTH_DATA %lf %lf %lf %lf", 	
					&SyntheticData->Threshold,
					&SyntheticData->StartTime, 
					&SyntheticData->Duration, 
					&SyntheticData->SlewRate); 

			}


			if(strstr(PLine, "SIGNAL")) {
				if(SyntheticData->Signal == NULL) {
					SyntheticData->Signal = (Signal_t *)malloc(sizeof(Signal_t));
				} else {
					SyntheticData->Signal = 
						(Signal_t *)realloc(SyntheticData->Signal, sizeof(Signal_t)*(SyntheticData->NumSignals+1));
				}
				if(SyntheticData->Signal == NULL) {
                                	fprintf(stderr, "could not (re)alloc signal array\n");
                                        exit(1);
				}
				sscanf(PLine, "SIGNAL %lf %lf %lf %ld %lf %lf %lf", 	
					&SyntheticData->Signal[SyntheticData->NumSignals].StartFreq,
					&SyntheticData->Signal[SyntheticData->NumSignals].DriftRate,
					&SyntheticData->Signal[SyntheticData->NumSignals].BandWidth,
					&SyntheticData->Signal[SyntheticData->NumSignals].BinSkip,
					&SyntheticData->Signal[SyntheticData->NumSignals].Power,
					&SyntheticData->Signal[SyntheticData->NumSignals].TimeOffsetStart,
					&SyntheticData->Signal[SyntheticData->NumSignals].TimeOffsetStop);
				SyntheticData->NumSignals++;
				if(SyntheticData->NumSignals > 100) {
					fprintf(stderr, "too many SIGNALs!\n");
                                        exit(1);
				}
			}
		}
	}
}


void PrintParameters(SyntheticData_t SyntheticData) {

	long i;


	fprintf(stderr, "SYNTH_DATA %lf %lf %lf %f %ld %p\n", 
			SyntheticData.Threshold,
			SyntheticData.StartTime, 
			SyntheticData.Duration, 
			SyntheticData.SlewRate,
			SyntheticData.NumSignals,
			SyntheticData.Signal);

	for(i=0; i<SyntheticData.NumSignals; i++) {
		fprintf(stderr, "SIGNAL %lf %lf %lf %lf %lf %lf\n",
				SyntheticData.Signal[i].StartFreq,
				SyntheticData.Signal[i].DriftRate,
				SyntheticData.Signal[i].BandWidth,
				SyntheticData.Signal[i].BinSkip,
				SyntheticData.Signal[i].Power,
				SyntheticData.Signal[i].TimeOffsetStart,
				SyntheticData.Signal[i].TimeOffsetStop);
	}
}

void InitSynth(SyntheticData_t * SyntheticData) {

	SyntheticData->Threshold	= 0.0;
	SyntheticData->StartTime	= 0.0;
        SyntheticData->Duration		= 0.0;
        SyntheticData->SlewRate		= 0.0;
        SyntheticData->NumSignals	= 0;
        SyntheticData->Signal		= NULL;
}

void InitSignals(SyntheticData_t SyntheticData) {

	long i;


	if(SyntheticData.Signal == NULL) return;

       	for(i=0; i<SyntheticData.NumSignals; i++) {
                SyntheticData.Signal[i].StartFreq	= 0.0;
                SyntheticData.Signal[i].DriftRate	= 0.0;
                SyntheticData.Signal[i].BandWidth	= 0.0;
                SyntheticData.Signal[i].BinSkip		= 0;
                SyntheticData.Signal[i].Power		= 0.0;
                SyntheticData.Signal[i].TimeOffsetStart	= 0.0;
                SyntheticData.Signal[i].TimeOffsetStop	= 0.0;
	}
}
