/**************************************************************************
 Programmer : Jeff Cobb
 Date       : 6/27/93
 Source Code: s4findofile
 Compiler   : cc on sun workstation
 Purpose    : this program reads 2 files - a 'time' file, output by
	      program s4timerng, and a dbas file containing records
	      from a number of original dbas files.  an 'original'
	      file is one in date/time order, such as all the clean
	      files surviving rfi rejection.  the dbas file that this
	      program is designed to run on is one of mixed time/date,
	      for example as output by program exdate.  this program
	      will then find the filename of the original file that
	      each input dbas record came from.


              each array element corresponds to an original
	      file, and contains the time & date of the first and last
	      record, as well as the original filename.  allowance is
	      made for the fact that the input dbas file may contain
	      blank lines.

	      see also: s4timerng.cc
**************************************************************************
 modifications :
	11/30/93   chuck fixed locos from float to double
	3/28/95    scott converted to ANSI and SERENDIP IV
**************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <malloc.h>
#include "SIREN.h"
#include "s4io.h"
#include "s4time.h"


#define TIME_FILE_LINE_LEN 51

/* this structure defines a record as output by s4timerng */

typedef struct {
  char ftime[8];        /* first record time */
  char fdate[7];        /* first record date */
  char ltime[8];        /* last record time  */
  char ldate[7];        /* last record date  */
  char filename[21];    /* original file name */
  } time_rec;

int main(int argc, char *argv [])

  {
  time_rec * time;
  StdEventRec event;

  int times_in;

  double event_time;
  double time_time;

  int i;

  FILE * fin_p;
  FILE * ftime_p;

  struct stat statbuf;

		  /* Open dbas file. */

  if ((fin_p = fopen(argv[1],"r")) == NULL)
	 {
	 fprintf(stderr, "Can't open input file.\n");
	 exit(1);
	 }

		 /* Open times range file. */

  if ((ftime_p = fopen(argv[2],"r")) == NULL)
	 {
	 fprintf(stderr, "Can't open time range file.\n");
	 exit(1);
	 }


		 /* Check times file size and get number of events. */

  (void) fstat(fileno(ftime_p),&statbuf);
  if (statbuf.st_size % TIME_FILE_LINE_LEN != 0)
	 {
	 fprintf(stderr, "Unexpected file size.\n");
	 exit(1);
	 }
	 times_in = statbuf.st_size / TIME_FILE_LINE_LEN;

		 /* Allocate space for times array */

  if ((time =
	  (time_rec *) calloc((unsigned)times_in, sizeof(time_rec))) == NULL)
	 {
	 fprintf(stderr, "Bad event allocation.\n");
	 exit(1);
	 }

		 /* Scan time file into times array */

  for(i = 0; i < times_in; i++)
	fscanf(ftime_p, "%s %s %s %s %s", time[i].ftime, time[i].fdate,
			 time[i].ltime, time[i].ldate, time[i].filename);

		 /* Input dbas file single event */

  while(io_GetNextRecord(fin_p,&event,STDEVENT_REC))
	 {
	 for(i = 0; i < times_in; i++)
	  {
	  if((strcmp(event.stDate, time[i].ldate)) == 0)
		 {

		 /* Assigns similarly formatted times to module constants */

		 event_time = event.dbUTime;
		 tm_HmsToDec(time[i].ltime, &time_time);

		 if(event_time <= time_time)
			{
			io_WriteStdEvent(stdout,&event);
			printf("#: file of origin: %s\n", time[i].filename);
			break;
			}
		 }
	         else
		 if((strcmp(event.stDate, time[i].fdate) == 0) &&
		  (strcmp(event.stDate, time[i].ldate) != 0))
			{
			io_WriteStdEvent(stdout,&event);
			printf("#: file of origin: %s\n", time[i].filename);
			break;
			}
		  } /* ends for i = 0 to times_in */
		
                if(i == times_in)
		 {
		 io_WriteStdEvent(stdout,&event);
		 printf("#: file of origin: UNKNOWN\n");
		 }

	 } /* ends while */

	 if (!feof(fin_p))
	 {
	 fprintf(stderr, "Bad read of event file.\n");
	 exit(1);
	 }

  return 0;
  }           /* ends main */











