// Source Code Name  : s4rddat.cc
// Programmer        : Cobb
//                   : Project SERENDIP, University of California, Berkeley CA
// Compiler          : GNU gcc

/* This progra is a striped down version of s4rddat.  It quickly parses
   through the file and outputs the last engineering string in the file.
*/

//#define DEBUG 
// #define IBMPC
#define SUN
#include <stdio.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>


#include "SIREN.h"

#define FOREVER 1

// cross platform types
#ifdef IBMPC
#define U16 unsigned
#define U32 unsigned long
#define S16 int
#define S32 long
#define BYTE unsigned char
#endif

#ifdef SUN
#define U16 unsigned short
#define U32 unsigned long
#define S16 short int
#define S32 long
#define BYTE unsigned char
#endif

// data block types for data files
#define SCSTR  0x0400  // telescope string
#define CMMT   0x0500  // comment
#define HITS   0x0A00  // hits
#define SYSCFG 0X0B00  // system configuration info
#define ERR    0X1000  // error
#define ENG    0X2000  // engineering record
#define THROTTLE        0x4000  // data thinning
                                // additional data: 1 double


#define ERR_SYNTH        0X01
#define ERR_VECT_WARN    0X02
#define ERR_OVER_RUN     0X03
#define ERR_ARITH_OVFL   0X04
#define ERR_OTHER        0X05
#define ERR_BD_NOT_READY 0X06
#define BUFFA 0X01
#define BUFFB 0X02

#define SIZEOF_HITS 10
#define HITS_OVERHEAD 15


char *usage = "s4lasteng [-s] S4_binary_file";

char CommentStr[80];
char ConfigStr[128];
char ScopeStr[80];
unsigned char EngStr[80];
char UtilStr255[256]; // one extra for terminator

U32 u32Synthesizer;
U32 u32Time;
U16 u16ADC;
BYTE btBuffer;
double dRA;
double dDec;

// globals
FILE *fInputDataFile;
FILE *fLogFile;
long FilePosition, NumToRead, NumAvailable;
off_t CurrentFileSize;
char InputDataFileName[512]; // shouldn't be bigger'n this! 
char LogFileName[512];
BOOLEAN RecordStarted;
U16 u16BlockType;
U16 u16BlockSize;
U16 u16Reader;
BYTE btReader;
U32 u32Reader;
unsigned uNumberOfHits;
long LastEngString;
long LastEngSize;

 
struct stat statbuf;


// function prototypes
BOOLEAN ReadNext(char *buf,long n);
void ReadNextLine(char *buf,long max_n,char eol);
void Init(char * filename);
void ErrExit(char *msg);
void ChangeEndian(unsigned char *var,int size);
void Fini();

//____________________________________________________________________
// MAIN
//--------------------------------------------------------------------
int main (int argc,char **argv)
  {

  BOOLEAN bSwapBytes;

  if ((argc != 2) && (argc != 3))
    ErrExit("illegal number of parmameters");
  bSwapBytes = FALSE;
  if (argc == 3)
    {
    if (strcmp(argv[1],"-s") == 0)
       bSwapBytes = TRUE;
    }

  Init(argv[argc-1]); 

  while (FOREVER)
     {


#ifdef DEBUG
     fprintf(stderr,"reading next block--------\n");
#endif
     RecordStarted = FALSE;

     if(ReadNext((char *)&u16BlockSize,sizeof(U16)));
       RecordStarted = TRUE;


     if (bSwapBytes)
	  ChangeEndian((unsigned char*)&u16BlockSize,sizeof(U16));

     ReadNext((char *)&u16BlockType,sizeof(U16));

     if (bSwapBytes)
	  ChangeEndian((unsigned char *)&u16BlockType,sizeof(U16));

       switch (u16BlockType)
	 {
         case THROTTLE :
		   fseek(fInputDataFile, (off_t)sizeof(double), SEEK_CUR);
                   break;
	 case HITS :
#ifdef DEBUG
		   printf("hits: blocksize %u\n ", u16BlockSize);
#endif
		   uNumberOfHits = (u16BlockSize - HITS_OVERHEAD) / SIZEOF_HITS;

		   fseek(fInputDataFile, (off_t)sizeof(U32) +
					 (off_t)sizeof(U32) +
					 (off_t)sizeof(U16) +
					 (off_t)sizeof(BYTE), 
					  SEEK_CUR);

		   fseek(fInputDataFile, uNumberOfHits * ( (off_t)sizeof(U32) +
							   (off_t)sizeof(U16) +	
							   (off_t)sizeof(U16) +	
							   (off_t)sizeof(BYTE) +	
							   (off_t)sizeof(BYTE) ),
					  SEEK_CUR); 
		   break;
	 case CMMT :   
		   fseek(fInputDataFile, (off_t)u16BlockSize-4, SEEK_CUR);
		   break;
	 case SYSCFG : 
		   int bytesRead;
		   bytesRead = 0;
		   while (bytesRead < u16BlockSize-4)
		     {
                     ReadNextLine(ConfigStr,128,(char)'\0');
		     bytesRead += strlen(ConfigStr) + 1;
		     }
		   break;
         case ENG : //ReadNext(UtilStr255,u16BlockSize-4); 
		   fseek(fInputDataFile, (off_t)sizeof(U32), SEEK_CUR);
		   LastEngString = ftell(fInputDataFile);
		   LastEngSize = u16BlockSize;
		   fseek(fInputDataFile, (off_t)u16BlockSize-8, SEEK_CUR);
                   break;
	 case ERR : 
                   ReadNext((char *)&btReader,sizeof(BYTE));

		   if(btReader != ERR_SYNTH) {
			fseek(fInputDataFile, (off_t)sizeof(BYTE), SEEK_CUR);
		   }		

#ifdef DEBUG
		   printf("err: blocksize %u\n ", u16BlockSize);
#endif
		   break;
	 case SCSTR : 
		   fseek(fInputDataFile, (off_t)u16BlockSize-4, SEEK_CUR);
		      break;

	 default : printf("Unknown.....\nblock size: %x\n", u16BlockSize);
                 ErrExit("Unexpected block type"); 
		 break;
       }

     }

  }


//--------------------------------------------------------------------
void ErrExit(char *msg)
//--------------------------------------------------------------------
  {
  fprintf(stderr,"%s\n", msg);
  fprintf(stderr, "usage: %s\n",usage );
  exit(1);
  }


//--------------------------------------------------------------------
void ChangeEndian(unsigned char *var,int size)
//--------------------------------------------------------------------
  {
  unsigned char myByteStr[8];
  int i;

  // simple, but it even works for odd sizes!
  if (size <= 8)
    {
    for (i=0;i<size;i++)
      myByteStr[i] = var[size-1-i];
    for (i=0;i<size;i++)
      var[i] = myByteStr[i];
    }

  }

//--------------------------------------------------------------------
void Init(char *filename)
//--------------------------------------------------------------------
  {
  strcpy(InputDataFileName,filename);
  fInputDataFile = fopen(InputDataFileName,"r");
  if (fInputDataFile == NULL)
    ErrExit("Can't open data file");
  fstat(fileno(fInputDataFile),&statbuf);
  CurrentFileSize = statbuf.st_size;
  FilePosition = 0;
  }

//--------------------------------------------------------------------
BOOLEAN ReadNext(char *buf,long n)
//--------------------------------------------------------------------
  {
  FilePosition = ftell(fInputDataFile);
  NumAvailable = CurrentFileSize - FilePosition;
  if(NumAvailable < n)
    Fini();
  else
    { 
    fread(buf,sizeof(char),(int) n,fInputDataFile);
    FilePosition = ftell(fInputDataFile);
    return TRUE;
    }
  return FALSE;   // to silence compile warning - will never get to this line
  }  

//--------------------------------------------------------------------
// reads next line, or up to n characters (whichever comes first)
// from global file stream, End of line is eol.
void ReadNextLine(char *buf,long max_n,char eol)
//--------------------------------------------------------------------
   {
   unsigned char NextChar;
   int NumCharsRead;
   NumCharsRead = 1;
   ReadNext((char *)&NextChar,1);
   buf[NumCharsRead-1] = NextChar;
   while((NextChar != eol) && (NumCharsRead < max_n)) 
      {
      NumCharsRead++;
      ReadNext((char *)&NextChar,1);
      buf[NumCharsRead-1] = NextChar;
      }
   }


//--------------------------------------------------------------------
void Fini()
//--------------------------------------------------------------------
  { 
  int i, rtn;

  fprintf(stderr, "Exiting... ");
    if(RecordStarted)
      {  
      fprintf(stderr, "file ends in record fragment!\n");
      rtn = 1;
      }  
    else
      { 
      fprintf(stderr, "file ends on record boundry\n");   // normal exit
      rtn = 1;
      }
 
     fseek(fInputDataFile, LastEngString, SEEK_SET);
     ReadNext((char *)EngStr,LastEngSize);
        for (i=0;i<LastEngSize-8;i++)
           printf(" [%hu] ", EngStr[i]);
     printf("\n");
  exit(rtn);
  }

