/***********************************************************
 Programmer  :Jeff Cobb
 Source      :s4order.cc
 Compiler    :cc on Sun  Workstations under UNIX
 Purpose:    :This program reads a standard dbase format file,
             :onel event at a time, and compares the time with
             :the previous event.
             :
             :Out-of-order events are reported along with the
             :line number.  Date changes are also reported
             :and not considered out of oreder, unless the date
             :changes to an earlier date.  Also considered
             :out-of-order is a time which is greater than
             :24 decimal hours.
             :
             :Any out-of-order event results in the program
             :exiting with a "1".  This can then be checked
             :for example, by another running program.
             :
             :The report can also be redirected to a log
             :file. 
 ************************************************************
 modifications:
 1/14/93     Beppe - Exits upon finding out-of-order event.
 11/30/93    chuckd - fixed locos from float to double
 06/15/95    scottr - converts from s3 to s4
 **********************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "SIREN.h"
#include "s4io.h"
#include "s4time.h"


int main (int argc, char * argv[])


{
  FILE * ascii_in;

  StdEventRec event;

  int  errorFound = 0;

  char * usage = "usage: s3order file_name";
  char file_name[20];
  char prog_name[20];

  char this_date[7];               /* used for comparison to last_date */
  char last_date[7];

  double last_time  = 0.0;        /* for decimal hours comparison to... */
  double this_time  = 0.0;        /* this_time   */

  unsigned long event_count = 0;  /* total number of events in file */
  long flag_count  = 0;           /* total # of out-of-order
                                     and date-change events */	
	
  int badtime_count = 0;

  long day1, day2, month1, month2, year1, year2;



  if (argc != 2)
	 {
	 fprintf(stderr, "illegal number of arguments: %s\n", usage);
	 exit(1);
	 }

  strcpy(prog_name, argv[0]);    /* save program and file names  */
  strcpy(file_name, argv[1]);


  if((ascii_in = fopen(file_name, "r")) == NULL)
	 {
	 fprintf(stderr, "error - cannot open file %s\n%s\n", file_name, usage);
	 exit(1);
	 }

			/* message to user; long running program */

  fprintf(stderr, "%s: reading file %s...\n", prog_name, file_name);

			/* read first record, save the time, initialize for
				following loop */

	if ((io_GetNextRecord(ascii_in,&event,STDEVENT_REC)) != 1)
	  fprintf(stderr, "error - empty file\n");
	else
	  {
	  event_count++;
	  strcpy(last_date, event.stDate);
	  last_time = event.dbUTime;
	  }

  fprintf(stdout, "%s: data for %s:\n", prog_name, file_name);

			/* loop through remaining records in file and output msg
				for each out of order event */

	while (io_GetNextRecord(ascii_in,&event,STDEVENT_REC))
	  {
	  event_count++;
	  strcpy(this_date, event.stDate);
	  this_time = event.dbUTime;

	  if(this_time > 24.0)   /* check for invalid time */
		 {
		 if(badtime_count)    /* more than 1 bad time - count but no message */
			{
			badtime_count++;
			flag_count++;
			}
		 else
			{
			fprintf(stdout, "Bad time.     Line number = %ld.\n", event_count);
			badtime_count++;
			flag_count++;
			errorFound = 1;
			}
		 }
	  else
		 {
		 if(badtime_count)    /* did previous event have a bad time? */
			{
			fprintf(stdout, "Number of consecutive bad times:  %d\n",
					  badtime_count);
			badtime_count = 0;
			}
		 if(strcmp(this_date, last_date) != 0) /* time valid, now check date */
			{
			fprintf(stdout, "Date change.   Line number = %ld.  ",
			event_count);

			fprintf(stdout, "From %s to %s.  ", last_date, this_date);
			fprintf(stdout, "Time is %s.\n", event.stUTime);

				  /* Convert string date to day, month, year */

			tm_DateToI(this_date, &day1, &month1, &year1);
			tm_DateToI(last_date, &day2, &month2, &year2);



			/* determine if last_date is earlier than this_date  */

			if (tm_CompareDate(&month1,
				           &day1,
					   &year1,
					   &month2,
					   &day2,
			                   &year2) < 0)
			 { 
			  fprintf(stdout, "Date change goes BACKWARDS!\n");
			  flag_count++;
			  errorFound = 1;
			  }

			strcpy(last_date, this_date);  /* update last_date on date change */
			}
		 else
			if(this_time < last_time)     /* no date change, check time order */
			  {
			  fprintf(stdout, "Out of order event.  Line number is %ld. ",
			  event_count);
			  fprintf(stdout, "Time is %s.\n", event.stUTime);
			  flag_count++;
			  errorFound = 1;
			  }
			last_time = this_time;  /* update last_time on each iteration */
		 }
	  }                       /* end of while loop */

	  fprintf(stdout, "Total number of events in %s is %ld.  ",
				file_name, event_count);
	  fprintf(stdout, "Total IRIG errors: %ld.\n", flag_count);

	  fprintf(stderr, "%s: processing complete.  %ld IRIG errors.\n",
				 prog_name, flag_count);

	  fclose(ascii_in);

	if (errorFound)
		return (1);
	return(0);
}

