// Source Code Name  : s4precess.cc
// Programmer        : Cobb/Donnelly
//                   : Project SERENDIP, University of California, Berkeley CA
// Compiler          : GNU gcc

/*
History:   2/24/98   jeffc  Ported to SERENDIP IV / ANSII C compliance
                            Program will now work with the record types
                            A, B, C.  It will NOT work with old style
                            s3 event records.
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "SIREN.h"
#include "s4io.h"
#include "s4time.h"
#include "s4coord.h"

main(int argc, char ** argv)
  {
  FILE *fin;
  char *usage = "usage: s4precess s3|s4|c from_epoch to_epoch infile > outfile";
  double e1;   /* from epoch */
  double e2;   /* to epoch */
  int    epoch_of_day1; /* flag indicates use epoch of day for e1 */
  int    epoch_of_day2; /* flag indicates use epoch of dar for e2 */
  double pos1[3]; /* position of input event */
  double pos2[3]; /* position of event after conversion to precessed
                   * coordinate */
  double dbRa;      /* right ascension */
  double dbDec;     /* declination */
  long    day;     /* day of event - not used, but needed 
                      for a tm_DateToI() call */
  long    mon;     /* month of event, for epoch of day */
  long    year;    /* for computing epoch of day */


  StdEventRec s4Event;
  s3_StdEventRec s3Event;
  CandidateRec Candidate;
  void * ThisRecord;
  char RecordType;
  char stDate[7];

  /* process command line */

  if (argc != 5)
    {
    fprintf(stderr,"%s\n",usage);
    exit(1);
    }

  if(strcmp(argv[1], "s3") == 0)  // s3 or s4 or candidates?
    {
    RecordType = S3_STDEVENT_REC;
    ThisRecord = (s3_StdEventRec *) &s3Event;
    }
  else if(strcmp(argv[1], "s4") == 0)
    {
    RecordType = STDEVENT_REC;
    ThisRecord = (StdEventRec *) &s4Event;
    }
  else if(strcmp(argv[1], "c") == 0)
    { 
    RecordType = CANDIDATE_REC;
    ThisRecord = (CandidateRec *) &Candidate;
    }
  else
    {
    fprintf(stderr,"%s\n",usage);
    exit(1);
    }

  if (sscanf(argv[2],"%lf",&e1) != 1)
    {
    fprintf(stderr,"%s\n",usage);
    exit(1);
    }

  if (sscanf(argv[3],"%lf",&e2) != 1)
    {
    fprintf(stderr,"%s\n",usage);
    exit(1);
    }

    /* set flag if to use epoch of the day */   

    epoch_of_day1 = (e1 == 0) ? 1 : 0; 
    epoch_of_day2 = (e2 == 0) ? 1 : 0; 

 /* open input file */ 
 
  if ((fin = fopen(argv[4],"r")) == NULL)
    {
    fprintf(stderr,"cannot open input file\n");
    exit(1);
    }

 /* now process file */

  while(io_GetNextRecord(fin, ThisRecord, RecordType))
    {
    if(RecordType == S3_STDEVENT_REC)
      {
      dbRa  = ((s3_StdEventRec *)ThisRecord)->dbRa;
      dbDec = ((s3_StdEventRec *)ThisRecord)->dbDec;
      strcpy(stDate, ((s3_StdEventRec *)ThisRecord)->stDate);
      }
    else if(RecordType == STDEVENT_REC)
      {
      dbRa  = ((StdEventRec *)ThisRecord)->dbRa;
      dbDec = ((StdEventRec *)ThisRecord)->dbDec;
      strcpy(stDate, ((StdEventRec *)ThisRecord)->stDate);
      }
    else if(RecordType == CANDIDATE_REC)
      {
      dbRa  = ((CandidateRec *)ThisRecord)->dbRa;  
      dbDec = ((CandidateRec *)ThisRecord)->dbDec; 
      } 

    /* compute epoch of the day for "from" and/or "to" epoch, if user
     * entered 0 for e1 and/or e2 */ 

    if (epoch_of_day1 || epoch_of_day2)
       {
       if(RecordType == CANDIDATE_REC)
         {
         fprintf(stderr,"cannot use epoch of the day for candidate records\n");
         exit(1);
         }
       tm_DateToI(stDate, &day, &mon, &year);
       mon = tm_MonToI(stDate);      /* get integer month */
       year = tm_CentYear(year);
       if (epoch_of_day1)
         e1 = year + (float) mon/12;
       if (epoch_of_day2)
         e2 = year + (float) mon/12;
       }

    co_EqToXyz(dbRa, dbDec, pos1);      
    co_Precess(e1,pos1,e2,pos2);
    co_XyzToEq(pos2, &dbRa, &dbDec);

    if(RecordType == S3_STDEVENT_REC)
      {
      ((s3_StdEventRec *)ThisRecord)->dbRa  = dbRa;
      ((s3_StdEventRec *)ThisRecord)->dbDec = dbDec;
      io_WriteS3StdEvent(stdout, (s3_StdEventRec *)ThisRecord);
      }
    else if(RecordType == STDEVENT_REC)
      { 
      ((StdEventRec *)ThisRecord)->dbRa  = dbRa;
      ((StdEventRec *)ThisRecord)->dbDec = dbDec;
      io_WriteStdEvent(stdout, (StdEventRec *)ThisRecord);
      }
    else if(RecordType == CANDIDATE_REC)
      { 
      ((CandidateRec *)ThisRecord)->dbRa  = dbRa;
      ((CandidateRec *)ThisRecord)->dbDec = dbDec;
      io_WriteCandidate(stdout, (CandidateRec *)ThisRecord);
      }

    }

    fclose(fin);

  }
