// Source Code Name  : s4pwrhst.cc
// Programmer        : Cobb/Donnelly
//                   : Project SERENDIP, University of California, Berkeley CA
// Compiler          : GNU gcc
 
/*
   2/3/99	jeffc	Initialize OverFlowCnt.
*/

#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <fcntl.h>
#include "SIREN.h"
#include "s4io.h"
#include "s4power.h"
#include "s4summary.h"
#include "s4utils.h"

#define MAXP            10000

void GetOps(int argc,
            char ** argv,
            BOOLEAN         * Before,
            BOOLEAN         * After,
            BOOLEAN         * UpdateDb);

main(int argc, char ** argv)
  {
  double Version = 1.0;

  long i, TotalEvents, OverFlowCnt;
  unsigned long PowerInt;

  long            SummDbIndex;
  int             SummIndexFile;
  DbaseSumRcd_t   DbaseSumRcd;
  char            FileNameRoot[64];

  BOOLEAN         Before = FALSE;
  BOOLEAN         After = FALSE;
  BOOLEAN         UpdateDb = FALSE;

  long Powers[MAXP];

  StdEventRec Event;

  FILE * fp_InFile;

  OverFlowCnt = 0;

  GetOps(argc, argv, &Before, &After, &UpdateDb);

  if ((fp_InFile = fopen(argv[argc-1],"r")) == NULL) 
    {
    printf("file: %s  does not exist\n",argv[1]);
    exit(1);
    }
     
  memset(Powers,0,sizeof(long)*MAXP);  // init power dist array

  TotalEvents = 0;

  while(io_GetNextRecord(fp_InFile, &Event, STDEVENT_REC))
    {
    if((PowerInt = (unsigned long)pw_PwrCalc(&Event, STDEVENT_REC)) > MAXP - 1)
      {
      OverFlowCnt++;
      PowerInt = MAXP - 1;
      }
    Powers[PowerInt]++;
    TotalEvents++;
    }

  // update summary DB
  if(UpdateDb == TRUE) {
    ut_GetRootFilename(argv[argc - 1], &FileNameRoot[0]);
    SummIndexFile = sum_GetDataBase(O_RDWR);
    SummDbIndex  = sum_GetRcd(FileNameRoot, &DbaseSumRcd, SummIndexFile, O_RDWR);
    if(Before) {
      DbaseSumRcd.fPwrHstBeforeVers = Version;
      DbaseSumRcd.lMaxPowerHistogrammedBefore = MAXP-1;
      DbaseSumRcd.lNumEventsOverMaxPowerBefore = OverFlowCnt;
    } else {
      DbaseSumRcd.fPwrHstAfterVers = Version;
      DbaseSumRcd.lMaxPowerHistogrammedAfter = MAXP-1;
      DbaseSumRcd.lNumEventsOverMaxPowerAfter = OverFlowCnt;
    }
    sum_PutRcd(SummDbIndex, &DbaseSumRcd, SummIndexFile);
    sum_ReleaseDataBase(SummIndexFile);
  }

  for(i=0;i<MAXP - 1;i++)
    if (Powers[i] != 0)
      printf("%d %d\n",i,Powers[i]);
  }


//--------------------------------------------------------------------------
void GetOps(int argc,
            char ** argv,
            BOOLEAN         * Before,
            BOOLEAN         * After,
            BOOLEAN         * UpdateDb)
//--------------------------------------------------------------------------
{

  long i;
  char * usage = "Usage:  s4pwrhst -before|-after [-db] infile_name";

  if (!(argc == 3 || argc == 4)) {
    fprintf(stderr, "%s %d\n", usage, argc);
    exit(1);
  }

  for (i = 1; i < argc - 1; i++) {

    if (strcmp(argv[i], "-db") == 0) {
        *UpdateDb = TRUE;
    }
    else if (strcmp(argv[i], "-before") == 0) {
        *Before = TRUE;
    } 
    else if (strcmp(argv[i], "-after") == 0) {
        *After = TRUE;
    }
    else {
        fprintf(stderr, "%s\n", usage);
        exit(1);
    }
  }
}
