/**************************************************************************/
/* Programmer        : ????
 * Source code name  : s4recordcount.cc
 * Compiler          : Sun C++
 * Purpose           : reads a dbas file and compares the number of events in
 *                     in the file with the number specified on the command
 *                     on the command line.  It returns a 2 if the number
 *                     of events in the file is greater than the number
 *                     "num_lines".  The program is written to allow
 *                     segments of shell scripts to be skipped if a file
 *                     is beyond a certain number of events.
 *                     Must select record type to be counted.
 *
 * Note              : s4recordcount.cc was converted from s3linecount.cc
 * 
 **************************************************************************
 *   modifications:
 *   2/28/95       scott converted to S4 standards and allowed record
 *                 type event selection.
 *   9/30/03       mattl cleaned up gcc 3.0 compiler warnings
 **************************************************************************
 */


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "SIREN.h"
#include "s4io.h"


int main(int argc, char **argv)

{
	char *usage = "Usage: s4recordcount filename num_lines record_type";
	long num_lines;
	long events_in;
	unsigned char record_type;
	FILE *fin_p;


	if (argc != 4)
	  {
	  fprintf(stderr,"%s\n",usage);
	  exit(1);
	  }

	if ((fin_p = fopen (argv[1],"r")) == NULL)
		 {
		 fprintf(stderr,"Error opening input file: %s\n",argv[1]);
		 fprintf(stderr,"%s\n",usage);
		 exit(1);
		 }

	 if (sscanf(argv[2],"%lu",&num_lines) != 1)
		 {
		 fprintf(stderr,"Error reading num_lines: %s\n",argv[1]);
		 fprintf(stderr,"%s\n",usage);
		 exit(1);
		 }

	 if (sscanf(argv[3],"%s",&record_type) != 1)
		 {
		 fprintf(stderr,"Error reading record_type: %s\n",argv[1]);
		 fprintf(stderr,"%s\n",usage);
		 exit(1);
		 }


			/* find number of events */

                        /* Note:  for incorrect record type, a count of zero is returned */

  events_in = (io_CountRecords(fin_p,record_type));

  if (events_in < 0)
		 {
		 fprintf(stderr, "Trouble counting events in file\n");
		 exit(1);
		 }


  fprintf(stdout,"%lu events in file \n",events_in);


		 /* output to shell */

  if (events_in > num_lines)
	 
         exit(2);
  else
	 exit(0);

     return (0);
  }



