//#define DEBUG_MAIN
//#define DEBUG_SLOWSLEW
//#define DEBUG_FASTSLEW
//#define DEBUG_PASSBAND
//#define DEBUG_SSPECTRUM
//#define DEBUG_CALCPARMS
//#define DEBUG_GETOPS
/* ===============================================================




This program is the SERENDIP IV port of s3rfi.c.


Function renames:

rfi1 - SpreadSpectrum - ported
rfi2 - PassBand       - ported
rfi6 - SlowSlew       - ported
rfi7 - FastSlew       - ported

rfi3,4 : both were baseband filters and are left out of this version

Changes from s3 version:
- threshold is gotten from first event in file, rather than
  from the command line.


Assumptions:
- threshold will not change mid-file (also made in s3)
=============================================================== */

#define VERSION 0.1

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <math.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "SIREN.h"   
#include "s4io.h"   
#include "s4freq.h"  
#include "s4coord.h"  
#include "s4summary.h"
#include "s4utils.h"

#define BSZE 256             /* Size of input buffer.               */
#define FAST  1              /* FAST mode is determined by mode = 1 */
#define STAT  0              /* STATistics mode is determined by mode = 1 */
/*#define debug */    /*used for debuging program, conditional compilation */
/* #define debug2 */

#define SPREADSPECTRUM_FLAG 0X08
#define PASSBAND_FLAG 0X04
#define RFI3_FLAG 0X01
#define RFI4_FLAG 0X02
#define SLOWSLEW_FLAG 0X10
#define FASTSLEW_FLAG 0X20

// ==================================================================================
typedef struct {
        BOOLEAN         DoIt;
        BOOLEAN         OutputRFI;    // Set to true if to output a file of rfi
                                      // rejected by SlowSlew
        long            DeltaSecs;    // num seconds over which to check slew rate 
        long            BaSecs;       // before/after group rejection time 
        double          Limit;        // slew rate limit.  If slower, reject
        unsigned        NumRej;       // number of events rejected by SlowSlew
        unsigned        NumNewRej;    // number of new events rejected by SlowSlew
        unsigned        SpectraLost;  // number of spectra rejected by SlowSlew
        char            *foutName;
        FILE            *fout;        // Output file for all rfi rejected by SlowSlew
} SlowSlew_t;
// ==================================================================================
typedef struct {
        BOOLEAN         DoIt;
        BOOLEAN         OutputRFI;    // Set to true if to output a file of rfi
                                      // rejected by FastSlew
        double          Dps;          // telescope slew limit in degrees per second
                                      // if telescope slews faster, reject events
        unsigned        NumRej;       // number of events rejected by FastSlew */
        unsigned        NumNewRej;    // number of new events rejected by FastSlew */
        unsigned        SpectraLost;  // number of spectra rejected by FastSlew */
        char            *foutName;
        FILE            *fout;        // Output file for all rfi rejected by FastSlew */
} FastSlew_t;
// ==================================================================================
typedef struct {
	BOOLEAN		DoIt;
	BOOLEAN		OutputRFI;    // set to true if to output a file of rfi
                                      // rejected by Passband
	double         	FreqRes;      // frequency resolution for PassBand 
	double         	NumBeams;     // number of telescope beams that an event
                                      // is allowed to occure and still be considered
                                      // to come from the same location in sky 
	unsigned long  	Limit;        // If 'PassBand_limit' number of events are       
                                      //  outside the beam width then flag event.  
                                      //  Used only for scanning by frequency.    
	unsigned long  	NumRej;       // number of events rejected 
	unsigned long  	NumNewRej;    // number of new events rejected 
	double       	DeadFreqBand; // dead frequency band lost 
 	char        	*foutName;    // Output file for all frequency correlated
  	FILE 		*fout;        // rfi (i.e., rfi rejected by PassBand) 
} PassBand_t;
// ==================================================================================
typedef struct {
  	BOOLEAN        	DoIt;         // Set to true if SpreadSpectrum is to be run.         
  	BOOLEAN       	OutputRFI;    // Set to true if to output a file of rfi
                                      // rejected by SpreadSpectrum 
  	double        	EntropyLimit; // Signal entropy as discussed for SpreadSpectrum      
  	long 	  	EventLimit;   // If 'SpreadSpectrum_limit' events occured at same
                                      // fft, then check signal entropy 
  	double  	SigmaLimit;   // 
	unsigned       	NumRej;       // number of events rejected by SpreadSpectrum 
	unsigned       	NumNewRej;    // number of new events rejected by SpreadSpectrum 
	long		SpectraRej;
	long		MaxInSpectra;
  	char        	*foutName;    // file for outputting all events rejected
  	FILE 		*fout;        // by SpreadSpectrum (-SpreadSpectrum option) 
} SpreadSpectrum_t;
// ==================================================================================


// --------------------- function prototypes -------------------------
void 	SpreadSpectrumFilter	(SpreadSpectrum_t *      SpreadSpectrum,
                          	unsigned               	events_in, 
                          	StdEventRec *          	events);
long 	CountThisSpectra	(StdEventRec * 		events,
                     		unsigned    		index,
                     		unsigned    		events_in);
double 	SignalEntropy		(StdEventRec * 		events,    
                      		unsigned    		index,    
                      		unsigned    		n);
void 	PassBandFilter		(PassBand_t *    	PassBand,
                    		char           		mode,
                    		unsigned long *  	index,
                    		StdEventRec *   	 events,
                    		unsigned long    	events_in,
                    		unsigned        	tel_diameter);
void 	OutputEvents		(SlowSlew_t           	SlowSlew,
                   		FastSlew_t           	FastSlew,
                   		PassBand_t           	PassBand,
                   		SpreadSpectrum_t     	SpreadSpectrum,
                   		FILE *               	fin_p,
                   		FILE *               	fout_p,
                   		FILE *               	fout_rfi,
                   		FILE *               	fout_comb,
                   		BOOLEAN              	output_rfi,
                   		BOOLEAN              	output_combination,
                   		char                 	combination,
                   		StdEventRec *        	events,
                   		unsigned long        	events_in,
                   		unsigned long *      	events_out);
void 	Report			(StdEventRec * 		events,
				unsigned long 		events_in, 
				unsigned long * 	ndx);
void 	StatisticsReport	(StdEventRec *     	events,
                       		unsigned long    	events_in,
                       		unsigned long *  	ndx);
void 	CalcPassBandParms	(long 			num_events, 
				double 			FreqCoverage, 
				double 			p, 
				double * 		bin_res, 
				unsigned long * 	limit);
void 	output_intermediate	(FILE *       		fin_p,
                         	FILE *       		fout_i,
                         	unsigned     		events_in,
                         	StdEventRec *  		events);
void 	SlowSlewFilter		(SlowSlew_t *    	SlowSlew,
                    		unsigned              	events_in,
                    		StdEventRec *         	events);
void 	FastSlewFilter		(FastSlew_t *          	FastSlew,
                    		unsigned long           events_in,
                    		StdEventRec *         	events);
long 	CountObsSeconds		(StdEventRec * 		events, 
				unsigned 		num_events);
void 	CountRejSpectra		(unsigned *       	rcount,
                        	StdEventRec *      	events,
                        	unsigned         	num_events,
                        	unsigned char    	rfimask);
void 	GetOps			(int               	argc,
            			char **           	argv,
            			SlowSlew_t *        	SlowSlew,
            			FastSlew_t *        	FastSlew,
            			PassBand_t *        	PassBand,
            			SpreadSpectrum_t *  	SpreadSpectrum,
            			unsigned *        	tel_diameter,
            			long *              	mode,
            			char **           	fout_i_name,
            			char **           	fout_rfi_name,
            			char **           	fout_summary_name,
            			double *         	p1s,
            			double *          	p2s,
            			long *             	two_pass,
            			long *             	auto_set,
				BOOLEAN *		UpdateDB);
BOOLEAN	OpenFiles		(FILE ** 		fin_p,        
				char 	* 		fin_name,
      				FILE ** 		fout_p,       
				char * 			fout_name,
      				FILE ** 		fout_i,       
				char * 			fout_i_name,         
				BOOLEAN * 		output_i,
      				FILE ** 		fout_rfi,     
				char * 			fout_rfi_name,       
				BOOLEAN * 		output_rfi,
      				FILE ** 		fout_SlowSlew,    
				char * 			fout_SlowSlew_name,      
				BOOLEAN * 		output_SlowSlew,
      				FILE ** 		fout_FastSlew,    
				char * 			fout_FastSlew_name,      
				BOOLEAN * 		output_FastSlew,
      				FILE ** 		fout_PassBand,    
				char * 			fout_PassBand_name,      
				BOOLEAN * 		output_PassBand,
      				FILE ** 		fout_SpreadSpectrum,    
				char * 			fout_SpreadSpectrum_name,      
				BOOLEAN * 		output_SpreadSpectrum,
      				FILE ** 		fout_summary, 
				char * 			fout_summary_name,   
				BOOLEAN * 		output_summary,
      				FILE ** 		fout_comb,    
				char * 			fout_comb_name);
long 	GetExpNumEvents		(double 		NumFftBins, 
				double 			NumSpectra, 
				double 			thresh);
long CountFftBoards		(long 			events_in);
static  int FreqCompare		(const void *		i, 
				const void *		j);
static  int BinCompare		(const void *		i, 
				const void *		j);
static  int FloatCompare	(const void *		i, 
				const void *		j);
void output_cntrl_structs	(SlowSlew_t *  		SlowSlew,
                                FastSlew_t *  		FastSlew,      
				PassBand_t *  		PassBand,
                                SpreadSpectrum_t *  	SpreadSpectrum);
	
// Main events array.  The global declaration is made neccessary by the
// qsort() calls on index arrays.
StdEventRec   	*events;         // 'events' will be assigned the address   
                               	 //  of the first record in the dynamically 
                               	 //  allocated memory list of events.       

const char * ProgName = "s4rfifilter";


// ================================= MAIN ======================================

//------------------------------------------------------------------------
main(int argc, char **argv)
//------------------------------------------------------------------------
  {

  DbaseSumRcd_t DbaseSumRcd;       // Summary database record
  char DbaseFileNameRoot[256];
  int SumIndexFile;
  unsigned long SummDbIndex;

  // Filter control structures
  SlowSlew_t 		SlowSlew;
  FastSlew_t 		FastSlew;
  PassBand_t 		PassBand;
  SpreadSpectrum_t 	SpreadSpectrum;

  // non-filter specific variables
  BOOLEAN       UpdateDB;
  unsigned      tel_diameter;  	 	 // diameter of telescope in feet 
  unsigned long  events_in;      	 // Number of events read in                
  unsigned long  goodtime_events;      	 // Number of events read with good slew times
  unsigned long  events_out;     	 // Number of events output                 
  unsigned long  i;              	 // Index variable                          
  struct stat   statbuf;        	 // Record to keep file status information  
  unsigned long  *bindex;         	 // 'bindex[i]' will polong to the ith event 
                                 	 //  in the list of events sorted by lBinNumber. 
  unsigned long  *findex;         	 // 'findex[i]' will polong to the ith event 
                                 	 //  in the list of events sorted by freq.  
  char          txt_buf[BSZE];   	 // Buffer for storing one input data line. 
  double        StartJd;
  double        StopJd;

  char          fout_comb_name_string[256];

  long           mode;           	 // Program run mode--"fast" or "statistics." 
                                	 //  Zero value indicates "statistics" mode.  
  BOOLEAN       output_i;       	 // Set to true if to output intermediate results
                                 	 // between pass 1 and pass 2 
  unsigned short threshold;      	 // realtime threhsold 
  BOOLEAN       output_rfi;     	 // Set to true if to output an rfi file.     
  BOOLEAN       output_summary;  	 // Set to true if to output a summary report
  BOOLEAN       output_combination; 	 // set to true if to output a fuke if rfi
                                     	 // rejected by "a combination" of algs.
                                     	 // this is useful to find overlap in
                                     	 // algorithms. 
  char          combination;    	 // this is the set of flags for
                                 	 // output_combiantion 
  long           two_pass;       	 // true if to run do two pass reduction 
  double        p1s;            	 // sensitivity of pass 1 
  double        p2s;            	 // sensitivity of pass 2 
  double	FreqCoverage;		 // total frequency coverage of this file in Hz 
  long           auto_set;       	 // true if to do auto parameter setting in 
                                 	 //   pass 2 

  unsigned       exp_num_events; 	 // from observing time and threshold, we
                                  	 // can compute the expected number of events 
  unsigned       exp_events_per_spectra; // from observing time and threshold, we
                                  	 // can compute the expected number of events 
  unsigned       exp_num_events_out; 	 // this value actually accounts for time
                                      	 // lost due to SlowSlew and FastSlew 
  unsigned       num_events;     	 // = events_in minus num_rejected, used for 
                                 	 //   setting parameters for pass 2 
  double          pass1_fres=0;     	 // frequency resolution for pass 1 
  double          pass1_bres=0;     	 // bin resolution for pass 1 
  unsigned       pass1_flimit=0;   	 // frequency event limit for pass 1 
  unsigned       pass1_blimit=0;   	 // bin event limit for pass 1 


  char        	*fout_i_name;
  char        	*fout_rfi_name;
  char        	*fout_summary_name;
  char        	*fout_comb_name;
  FILE 	      	*fin_p, *fout_p;         // Input and output files                   
  FILE        	*fout_rfi;               // Output file for all rejected events    
  FILE 		*fout_i;                 // For outputting intermediate results between
                                    	 // pass 1 and pass 2 
  FILE 		*fout_summary;           // Output file for summary report 
  FILE 		*fout_comb;

  long total_observ_time, GoodTime;

  /* Default values */

  SlowSlew.DoIt     			= FALSE;
  SlowSlew.OutputRFI   			= FALSE;    
  SlowSlew.DeltaSecs    		= 100;
  SlowSlew.Limit     			= 0.0025;
  SlowSlew.BaSecs       		= 20;
  SlowSlew.NumRej			= 0;
  SlowSlew.NumNewRej			= 0;
  SlowSlew.SpectraLost			= 0;

  FastSlew.DoIt     			= FALSE;
  FastSlew.OutputRFI			= FALSE;     
  FastSlew.Dps     			= 0.0090;
  FastSlew.NumRej			= 0;
  FastSlew.NumNewRej			= 0;
  FastSlew.SpectraLost			= 0;

  PassBand.DoIt     			= FALSE;
  PassBand.OutputRFI   			= FALSE;  
  PassBand.Limit      			= 2;
  PassBand.FreqRes        		= 10;
  PassBand.NumBeams       		= 4;
  PassBand.NumRej			= 0;             
  PassBand.NumNewRej			= 0;             
  PassBand.DeadFreqBand			= 0.0;      

  SpreadSpectrum.DoIt         		= FALSE;  
  SpreadSpectrum.OutputRFI    		= FALSE;    
  SpreadSpectrum.EntropyLimit   	= 10000;
  SpreadSpectrum.SigmaLimit       	= 0.0;
  SpreadSpectrum.NumRej			= 0;
  SpreadSpectrum.NumNewRej		= 0;
  SpreadSpectrum.SpectraRej		= 0;

  UpdateDB				= FALSE;
  mode            			= FAST; 
  output_i        			= 0;   // do not output intermdiate results by default 
  output_rfi      			= 0;   // do not output a file of all rfi by default  
  output_summary  			= 0;    	 // do not output a summary file by default 
  p1s             			= 0.00001; 	 // pass 1 sensitivity 
  p2s             			= 0.02;    	 // pass 2 sensitivity, 7% 
  two_pass        			= 1;   // default to 2-pass program, 1=true, 0=false 
  auto_set        			= 1;   // use auto parameter setting in pass 2: 1=true, 
							 //   0=false 

  long MaxFftBoards 			= 40;
  long NumFftBoards 			= 0;

  combination 				= SLOWSLEW_FLAG | FASTSLEW_FLAG;

  fout_i_name          			= NULL;
  fout_rfi_name        			= NULL;
  PassBand.foutName	    		= NULL;
  SpreadSpectrum.foutName		= NULL;
  SlowSlew.foutName	    		= NULL;
  FastSlew.foutName	    		= NULL;
  fout_summary_name    			= NULL;

  output_combination = 0;  /* change and recompile to use */
  if(output_combination)
    {
    strcpy(fout_comb_name_string, argv[argc - 2]);
    strcat(fout_comb_name_string, ".comb");
    fout_comb_name = fout_comb_name_string;
    }
  else
    fout_comb_name = NULL;

#ifdef DEBUG_MAIN
  fprintf(stderr, "==================== pre GetOps =======================================\n");
  output_cntrl_structs(&SlowSlew, &FastSlew, &PassBand, &SpreadSpectrum);
  fprintf(stderr, "===========================================================\n");
#endif

  //-----------------------------------------------------------
  GetOps(argc, 
         argv,
	 &SlowSlew,
	 &FastSlew,
	 &PassBand,
	 &SpreadSpectrum,
         &tel_diameter,
         &mode,
         &fout_i_name,
         &fout_rfi_name,
         &fout_summary_name,
         &p1s,
         &p2s,
         &two_pass,
         &auto_set,
	 &UpdateDB);
         

  //-----------------------------------------------------------


#ifdef DEBUG_MAIN
  fprintf(stderr, "==================== post GetOps =======================================\n");
output_cntrl_structs(&SlowSlew, &FastSlew, &PassBand, &SpreadSpectrum);
fprintf(stderr, "auto_set = %d\n", auto_set);
  fprintf(stderr, "===========================================================\n");
#endif

  /* Open input and output files. */
  if(!OpenFiles(&fin_p,                  argv[argc - 2],
                &fout_p,                 argv[argc - 1],
                &fout_i,                 fout_i_name,                &output_i,
                &fout_rfi,               fout_rfi_name,              &output_rfi,
                &SlowSlew.fout,          SlowSlew.foutName,          &SlowSlew.OutputRFI, 
                &FastSlew.fout,          FastSlew.foutName,          &FastSlew.OutputRFI, 
                &PassBand.fout,          PassBand.foutName,          &PassBand.OutputRFI, 
                &SpreadSpectrum.fout,    SpreadSpectrum.foutName,    &SpreadSpectrum.OutputRFI, 
                &fout_summary,           fout_summary_name,          &output_summary,
                &fout_comb,              fout_comb_name))
    exit(1);


  //-----------------------------------------------------------
  /* Check input file size and get number of records. */

  events_in = io_CountRecords(fin_p, STDEVENT_REC);
  if (events_in >= MAX_EVENTS)
    {
    perror("Too many events.");
    exit(1);
    }

#ifdef DEBUG_MAIN
  fprintf(stderr,"events_in = %ld\n", events_in);
#endif

  /* Allocate space for events and index arrrays. */

  if ((events = (StdEventRec *) calloc(events_in,sizeof(StdEventRec))) == 
     NULL)
    {
    perror("Bad event allocation.");
    exit(1);
    }
  if ((bindex=(unsigned long *) calloc(events_in,sizeof(unsigned long))) == NULL)
    {
    perror("Bad index allocation.");
    exit(1);
    }
  if ((findex=(unsigned long *) calloc(events_in,sizeof(unsigned long))) == NULL)
    {
    perror("Bad index allocation.");
    exit(1);
    }

#ifdef DEBUG_MAIN
  fprintf(stderr,"done allocating arrays\n");
#endif

  //-----------------------------------------------------------
  /* Output mode status and information about which algorithms are running. */

  if (mode == FAST)
    (void) fprintf(stderr,"%s : program running in fast mode\n", ProgName);
  else
    (void) fprintf(stderr,"%s : program running in statistics mode\n", ProgName);

  //-----------------------------------------------------------
  /* Read event information from input file. */

  (void) fprintf(stderr, "%s : Reading %u events\n", ProgName, events_in);

  for(i = 0; i < events_in; i++)
    {
    if (!io_GetNextRecord(fin_p, &events[i], STDEVENT_REC))
      {
      if (ferror(fin_p))
        perror("Bad file read.");
      else
        perror("premature end of file");
      exit(1);
      }
   if (i == 0) /* first event */
     {
     StartJd    = events[i].dbJulDate;
     threshold  = events[i].usSigmaThreshold;
     }
          
   /* Initialize event-flag and index arrays. */

   events[i].ucFlag = 0;
   bindex[i] = i;
   findex[i] = i;
   } /* end event reading */

  StopJd = events[i-1].dbJulDate;

#ifdef DEBUG_MAIN
  fprintf(stderr, "julstart = %lf   julend = %lf  thresh = %hu\n", StartJd, StopJd, threshold);
#endif

  /* Reset input file. */

  (void) rewind(fin_p);

  //-----------------------------------------------------------
  // Sort input data by bin (needed for stats reports and
  // by frequency if needed. 

  qsort((char *)bindex, events_in, sizeof(long), BinCompare);

  if (PassBand.DoIt)
    {
    (void) fprintf(stdout, "Sorting by frequency.\n");
    qsort((char *)findex, events_in, sizeof(long), FreqCompare); 
    }

#ifdef DEBUG_MAIN
   fprintf(stderr, "findex 0 = %ld bin = %ld  board = %ld freq = %lf\n", 
           findex[0], events[findex[0]].lBinNumber, events[findex[0]].ucBid, 
           events[findex[0]].dbFrequency);
   fprintf(stderr, "findex last = %ld bin = %ld  board = %ld freq = %lf\n", 
        findex[events_in-1], events[findex[events_in-1]].lBinNumber, 
        events[findex[events_in-1]].ucBid, events[findex[events_in-1]].dbFrequency);
/*
   for(i = 0; i < events_in; i++) {
	fprintf(stderr, "%lf\n", events[findex[i]].dbFrequency);
   }
*/
#endif


  //-----------------------------------------------------------
  /* pre pass */


  // Get expected number of events out (num expected from noise alone)
  num_events = events_in;	// num_events will be decremented as we reject
  NumFftBoards = CountFftBoards(events_in);
  FreqCoverage			= NumFftBoards * NUM_FFT_BINS * DELTA_FQ;  
  total_observ_time 		= CountObsSeconds(events,events_in);
  exp_events_per_spectra 	= GetExpNumEvents(NUM_FFT_BINS * NumFftBoards,
                               	    (double)1.0,
                               	    threshold);
  SpreadSpectrum.EventLimit	= exp_events_per_spectra +
				  (long)(SpreadSpectrum.SigmaLimit * sqrt(exp_events_per_spectra));	


  if (SlowSlew.DoIt)
    {
    fprintf(stderr,"rfi5: pre pass running SlowSlew\n");
    SlowSlewFilter(&SlowSlew, events_in, events);
    CountRejSpectra(&SlowSlew.SpectraLost,events,
                       events_in,(unsigned char) SLOWSLEW_FLAG);
    num_events -= SlowSlew.NumRej;   // NOTE : events rejected here
                                     // are not counted in
                                     // CalcPassBandParms().  Is this a problem for
                                     // stsat mode?
#ifdef DEBUG_MAIN
  fprintf(stderr, "slow slew spectra rej = %ld  slow slew events rej = %ld %ld num events still good = %ld\n", 
         SlowSlew.SpectraLost, SlowSlew.NumRej, SlowSlew.NumNewRej, num_events); 
#endif
    }

  if (FastSlew.DoIt)
    {
    FastSlewFilter(&FastSlew,events_in,events);
    CountRejSpectra(&FastSlew.SpectraLost,events,
                       events_in,(unsigned char) FASTSLEW_FLAG);
    fprintf(stderr,"rfi5: pre pass done running FastSlew\n");
    num_events -= FastSlew.NumNewRej;   // NOTE : events rejected here 
                                     // are not counted in
                                     // CalcPassBandParms().  Is this a problem for 
                                     // stsat mode?

#ifdef DEBUG_MAIN
  fprintf(stderr, "fast slew spectra rej = %ld  fast slew events rej = %ld %ld num events still good = %ld\n", 
          FastSlew.SpectraLost, FastSlew.NumRej, FastSlew.NumNewRej, num_events); 
#endif
    }

    // done with slew filters so get goodtime stats
    goodtime_events          = num_events;
    GoodTime                 = (GoodTime = (long)(total_observ_time            -
                                                  1.7 * SlowSlew.SpectraLost   -
                                                  1.7 * FastSlew.SpectraLost)) >= 0 ? GoodTime : 0;
    exp_num_events           = GetExpNumEvents(NUM_FFT_BINS * NumFftBoards,
                                               (double)GoodTime / INTEGRATION_TIME,
                                               threshold);

#ifdef DEBUG_MAIN
  fprintf(stderr, "
		   events_in		     = %ld\n
		   NumFftBoards              = %ld\n
                   total_observ_time         = %ld\n
                   GoodTime                  = %ld\n
                   expected num out          = %ld\n
                   per spectra               = %ld\n
                   SpreadSpectrum.EventLimit = %ld\n", 
         events_in, NumFftBoards, total_observ_time, GoodTime, exp_num_events, exp_events_per_spectra, SpreadSpectrum.EventLimit);
#endif


  if(SpreadSpectrum.DoIt)
    {
    (void) fprintf(stdout,"running SpreadSpectrum\n");
    SpreadSpectrumFilter(&SpreadSpectrum,
                         events_in,
                         events);
    num_events -= SpreadSpectrum.NumNewRej;
#ifdef DEBUG_MAIN
  fprintf(stderr, "SS events rej = %ld new = %ld num events still good = %ld\n", 
          SpreadSpectrum.NumRej, SpreadSpectrum.NumNewRej, num_events); 
#endif
    } 

  // We run Passband on good time event only, so we insist that there are some.  If
  // we run CalcPassBandParms() with no events, an infinite frequency widening loop
  // occurs.
  if(PassBand.DoIt && goodtime_events)
    {
    if (auto_set)
      CalcPassBandParms(goodtime_events, FreqCoverage, p2s, &PassBand.FreqRes, &PassBand.Limit);
    PassBandFilter(&PassBand,
		   mode, 
                   findex, 
                   events, 
                   events_in, 
                   tel_diameter);
    num_events -= PassBand.NumNewRej;	
#ifdef DEBUG_MAIN
  fprintf(stderr, "PB num events still good = %ld\n", num_events); 
  output_cntrl_structs(NULL, NULL, &PassBand, NULL);
#endif
    }   

//================   Output events to output file(s)  ================

  OutputEvents(SlowSlew,
                FastSlew,
		PassBand,
                SpreadSpectrum,
                fin_p,
                fout_p, 
                fout_rfi,
                fout_comb,
                output_rfi,
                output_combination,
                combination, 
                events,
                events_in,
                &events_out);

//=======================  Issue Reports  ============================


  if (mode == FAST) {
    Report(events,events_in,bindex);
  } else {
    StatisticsReport(events,events_in,bindex);
  }

//=====================  Update Summary DB  ==========================

  if(UpdateDB) {
 
    /* strip off path to dbase file */
    ut_GetLastField('/', argv[argc - 2], DbaseFileNameRoot);

    SumIndexFile = sum_GetDataBase(O_RDWR);
    SummDbIndex  = sum_GetRcd(DbaseFileNameRoot, &DbaseSumRcd, SumIndexFile, O_RDWR);

    DbaseSumRcd.fRfiFilterVers			= VERSION;
printf(">>> %f  %f\n", VERSION, DbaseSumRcd.fRfiFilterVers);
    DbaseSumRcd.ulRfiFilterEventsIn		= events_in;
    DbaseSumRcd.ulRfiFilterEventsOut		= events_out;
    DbaseSumRcd.ulRfiFilterExpectedOut		= exp_num_events; 
    DbaseSumRcd.ulGoodTime			= GoodTime;  
    if(SlowSlew.DoIt) {
    	DbaseSumRcd.ulFlags     		|= RFI_SLOWSLEW_RUN;        
    	DbaseSumRcd.ulSlowSlew_DeltaSecs 	= SlowSlew.DeltaSecs;      
    	DbaseSumRcd.ulSlowSlew_BaSecs 		= SlowSlew.BaSecs;        
    	DbaseSumRcd.dbSlowSlew_Limit 		= SlowSlew.Limit;     
    	DbaseSumRcd.ulSlowSlew_NumRej 		= SlowSlew.NumRej;      
    	DbaseSumRcd.ulSlowSlew_NumNewRej 	= SlowSlew.NumNewRej;      
    	DbaseSumRcd.ulSlowSlew_SpectraLost 	= SlowSlew.SpectraLost;
    }
    if(FastSlew.DoIt) {
    	DbaseSumRcd.ulFlags      		|= RFI_FASTSLEW_RUN;        
    	DbaseSumRcd.dbFastSlew_Dps 		= FastSlew.Dps;                
    	DbaseSumRcd.ulFastSlew_NumRej 		= FastSlew.NumRej;            
    	DbaseSumRcd.ulFastSlew_NumNewRej 	= FastSlew.NumNewRej;            
    	DbaseSumRcd.ulFastSlew_SpectraLost 	= FastSlew.SpectraLost; 
    }
    if(PassBand.DoIt) {
    	DbaseSumRcd.ulFlags      		|= RFI_PASSBAND_RUN;        
    	DbaseSumRcd.dbPassBand_FreqRes 		= PassBand.FreqRes;    
    	DbaseSumRcd.dbPassBand_NumBeams 	= PassBand.NumBeams;  
    	DbaseSumRcd.ulPassBand_Limit 		= PassBand.Limit ;  
    	DbaseSumRcd.ulPassBand_NumRej  		= PassBand.NumRej;        
    	DbaseSumRcd.ulPassBand_NumNewRej  	= PassBand.NumNewRej;        
    	DbaseSumRcd.dbPassBand_DeadFreqBand 	= PassBand.DeadFreqBand;  
    }
    if(SpreadSpectrum.DoIt) {    
    	DbaseSumRcd.ulFlags 		       	|= RFI_SSPECTRUM_RUN;        
    	DbaseSumRcd.dbSpreadSpectrum_EntropyLimit = SpreadSpectrum.EntropyLimit;  
    	DbaseSumRcd.ulSpreadSpectrum_EventLimit = SpreadSpectrum.EventLimit;    
    	DbaseSumRcd.dbSpreadSpectrum_SigmaLimit	= SpreadSpectrum.SigmaLimit;    
    	DbaseSumRcd.ulSpreadSpectrum_NumRej 	= SpreadSpectrum.NumRej;      
    	DbaseSumRcd.ulSpreadSpectrum_NumNewRej	= SpreadSpectrum.NumNewRej;      
    }

    sum_PutRcd(SummDbIndex, &DbaseSumRcd, SumIndexFile);
    sum_ReleaseDataBase(SumIndexFile);
  }


} // End main()


//===============================================================================
//============================== Slew Filters ===================================
//===============================================================================

//------------------------------------------------------------------------------
/*
 * SlowSlew()
 * function detects if telescope hasn't moved for
 * delta_seconds and flags events during that period.
 * also flags events before and after the period if
 * they occurred within b_a_seconds.
 */

void SlowSlewFilter(SlowSlew_t		* SlowSlew,
          	    unsigned     	  events_in,
          	    StdEventRec  	* events)
//------------------------------------------------------------------------------
  {
  long h,i,k,l;
  double SecDiff;
  unsigned char in_group; /* boolean flag, true if currently in a
                           * group of events detected while telescope
                           * is not moving */
  in_group = 0;
  
#ifdef DEBUG_SLOWSLEW
  fprintf(stderr, "SlowSlew ops: %d %d %lf %u %u\n",
          SlowSlew->DeltaSecs, SlowSlew->BaSecs, SlowSlew->Limit, events_in, SlowSlew->NumRej); 
#endif

  // h jumps ahead 1 spectra on each iteration
  for (h = 0; h < events_in - 1; h = i+1)
    {
    // march i to last event of this spectra
    for(i = h; 
        i < events_in - 1 && 
        events[i+1].dbJulDate == events[h].dbJulDate; 
        i++);
    // march k just past delta seconds from i
    for(k = i; 
        k < events_in - 1 &&
        (SecDiff = SECDIFF(events[i].dbJulDate,events[k].dbJulDate)) < SlowSlew->DeltaSecs;
        k++);

#ifdef DEBUG_SLOWSLEW
  fprintf(stderr, "h is now %d, i is now %d  k is now %ld secdiff is %lf sky angle is %lf\n", 
          h, i, k, SECDIFF(events[i].dbJulDate,events[k].dbJulDate),
          co_SkyAngle(events[i].dbRa, events[i].dbDec, events[k].dbRa, events[k].dbDec)/SecDiff);
 #endif

    // has the telescope moved far enough?
    if ((k < events_in - 1) &&
        (co_GetSlewRate(events[i].dbRa,
                     events[i].dbDec,
                     events[i].dbJulDate,
                     events[k].dbRa,
                     events[k].dbDec,
		     events[k].dbJulDate) < SlowSlew->Limit))
   // ... nope, telescope has not moved far enough
             {
#ifdef DEBUG_SLOWSLEW
  	     fprintf(stderr, "slow slew detected : %lf dps at time %s\n", 
		     co_GetSlewRate(events[i].dbRa,
                                    events[i].dbDec,
                                    events[i].dbJulDate,
                                    events[k].dbRa,
                                    events[k].dbDec,
		                    events[k].dbJulDate),
		     events[i].stUTime);
#endif
             // flag all events in this spectra
             for(l = h; l <= i; l++)
               {
               if (!events[l].ucFlag)
                 (SlowSlew->NumNewRej)++;
               if (!(events[l].ucFlag & SLOWSLEW_FLAG)) 
		 {
                 (SlowSlew->NumRej)++;
               	 events[l].ucFlag |= SLOWSLEW_FLAG;
		 }
               }
             if (!in_group)
                {
                // if we are just starting a slow slew section of data,  
                // then scan back and blitz 
                for (l=h-1;
                     (l>0) && 
                     (SECDIFF(events[l].dbJulDate,events[i].dbJulDate)
                      < SlowSlew->BaSecs);
                     l--)
                    {
                    if (!events[l].ucFlag)
                      (SlowSlew->NumNewRej)++;
                    if (!(events[l].ucFlag & SLOWSLEW_FLAG)) 
		      {
                      (SlowSlew->NumRej)++;
                      events[l].ucFlag |= 0x10;
		      }
                    }
                in_group = 1;     // indicate that we are in a slow section
                }
             }
    // ... yup, telescope has moved far enough
    else if (in_group)
        {
        // if we have just left a slow slew section of data, then we 
        // scan_forward (starting w/ 1st event of current spectra) and blitz 
        for (l=h;
             (l<events_in) && 
             (SECDIFF(events[i].dbJulDate,events[l].dbJulDate)
              <  SlowSlew->DeltaSecs + SlowSlew->BaSecs);
             l++)
               { 
               if (!events[l].ucFlag)
                  (SlowSlew->NumNewRej)++;
               if (!(events[l].ucFlag & SLOWSLEW_FLAG)) 
	         {
                 (SlowSlew->NumRej)++;
                 events[l].ucFlag |= 0x10;
		 }
               } 
        in_group = 0;       // indicate that we are no longer in a slow section
        }
    if (k >= (events_in))
        return;
      }
    }

//------------------------------------------------------------------------------
/*
 * looks for fast telescope slew and rejects events
 * where fast telescope slew is detected
 */
void FastSlewFilter(FastSlew_t		* FastSlew,
          	    unsigned long    	  events_in,
          	    StdEventRec     	* events)
//------------------------------------------------------------------------------
  {
  long i,j;
  long n,o;
  long debug_i;
  double thisDps;
  fprintf(stderr,"FastSlew: dpsthresh = %lf\n",FastSlew->Dps);
  for (i=0;i<events_in;)
    {
    n = CountThisSpectra(events,(unsigned) i,events_in);
    if (i+n < events_in)
      {
#ifdef DEBUG_FASTSLEW
  fprintf(stderr,"\nslewrate is %lf\n", co_GetSlewRate(events[i].dbRa,
                                    events[i].dbDec,
                                    events[i].dbJulDate,
                                    events[i+n].dbRa,
                                    events[i+n].dbDec,
                                    events[i+n].dbJulDate));

  fprintf(stderr,"in FastSlew : secdiff is %lf i is %d n is %d\n", 
        SECDIFF(events[i].dbJulDate, events[i+n].dbJulDate), i, n);
  fprintf(stderr,"====> %s  %s  %lf  %lf  %lf\n", 
        events[i].stUTime, events[i+n].stUTime, events[i].dbJulDate, 
        events[i+n].dbJulDate, co_SkyAngle(events[i].dbRa,
                                           events[i].dbDec,
                                           events[i+n].dbRa,
                                           events[i+n].dbDec));
#endif

      /* check degrees per second between spectra groups  */
      if ((thisDps = co_GetSlewRate(events[i].dbRa,
                                    events[i].dbDec,
                                    events[i].dbJulDate,
                                    events[i+n].dbRa,
                                    events[i+n].dbDec,
                                    events[i+n].dbJulDate)) > FastSlew->Dps)
        {
        /* if dps > threshold, then blitz out events in next group */
        o = CountThisSpectra(events,(unsigned)(i+n),events_in); /* get group size */

#ifdef DEBUG_FASTSLEW
  fprintf(stderr,
        "====> group starting at (i+n) %ld is rejectd. this dps is %lf  n is %ld  o is %ld\n",
        i+n, thisDps, n, o);
#endif

        for (j=i+n, debug_i=0; j < i+n+o;j++, debug_i++)
            {
            if (!events[j].ucFlag)
                (FastSlew->NumNewRej)++;
            (FastSlew->NumRej)++;
            events[j].ucFlag |= FASTSLEW_FLAG;
            }
#ifdef DEBUG_FASTSLEW
        fprintf(stderr,"====> REJECTED %ld\n", debug_i);
#endif
        }

#ifdef DEBUG_FASTSLEW
       else
            fprintf(stderr,"====> group starting at (i+n) %ld is OK. this dps is %lf \n", 
        i+n, thisDps);
#endif

       i += n;
      }
    else return;
 
     }
  }


//=============================================================================
//============================== RFI Filters ==================================
//=============================================================================


//------------------------------------------------------------------------------
/*
 * Function to exclude events according to rfi2 condition three (see rfi5.man).
 * Note: in "fast" mode this function always goes to the next unflagged event
 * when getting the next base event. In "statistics" mode this function
 * procedes one event at a time skipping no events regardless of the results
 * of other flagging.  Note, this function sets bit-3 (the LSB is bit-1)
 * of events[x].ucFlag.
 */

void PassBandFilter(PassBand_t	  * PassBand,
		    char            mode,
                    unsigned long  * index,
                    StdEventRec   * events,
                    unsigned long    events_in,
                    unsigned        tel_diameter)
//------------------------------------------------------------------------------
  { /* begin function PassBandFilter */

  long          i,j;              /* Index variables                           */ 
  unsigned long count;            /* Keeps track of number of events outside   */
                                 /*  beam res for comparison to 'limit'       */ 
  double ra_res,dec_res;          /* ra and dec resolution for beamwidth */
  double freq;

  /* used in calculating number of dead frequency bins */
  double lastdeadfreq;
  double deadfreqsum;
     
   deadfreqsum=0.0;
   lastdeadfreq=0.0;

   // take frequency from mid frequency in file and make it MHz
   freq = events[index[(long)(events_in/2)]].dbFrequency / 1e6;
   // freq = 1470.0;
   co_GetBeamRes(&dec_res,&ra_res, (long)freq,tel_diameter,PassBand->NumBeams);
   dec_res = 0.1 * PassBand->NumBeams;

#ifdef DEBUG_PASSBAND
   fprintf(stderr,"freq mid = %lf\n", events[index[(long)(events_in/2)]].dbFrequency);
   fprintf(stderr,"PassBand->FreqRes    = %f\n",PassBand->FreqRes);
   fprintf(stderr,"PassBand->Limit    = %u\n",PassBand->Limit);
   fprintf(stderr,"PassBand->NumBeams    = %f\n",PassBand->NumBeams);
   fprintf(stderr,"freq    = %f\n",freq);
   fprintf(stderr,"ra res  = %f\n",ra_res);
   fprintf(stderr,"dec res = %f\n",dec_res);
#endif
     
   for (i = 0; i < events_in; i++)      /* For each event do                  */ 
    {

    /* If mode is fast and event already flagged, skip event. */
 
    if (mode && events[index[i]].ucFlag)
      continue;
 
    count = 0;                  /* Otherwise reset limit-counter.  */
 
    /* Scan events forward and backward through PassBand->FreqRes.  If more than */
    /*  'limit' events are found outside beam res, then flag event.     */
     
    /* Forward scan */
 
    for (j = i + 1;
         j < events_in &&
         events[index[j]].dbFrequency - events[index[i]].dbFrequency <= PassBand->FreqRes;
         j++)
      {
 
      /* call function to check if beam resolution is violated */
      if (co_SkyAngle(events[index[j]].dbRa, events[index[j]].dbDec,
                      events[index[i]].dbRa, events[index[i]].dbDec) >
          dec_res)
        {
        count++;                 /* if beam-res violated increment count  */
        if (count > PassBand->Limit)  /* if count exceeds limit then           */
          {

          if (!(events[index[i]].ucFlag)) /* count new events rejected */
            (PassBand->NumNewRej)++;
          if (!(events[index[i]].ucFlag & PASSBAND_FLAG))
            {
            (PassBand->NumRej)++;
            events[index[i]].ucFlag |= PASSBAND_FLAG;
            }

          if (!(events[index[j]].ucFlag)) 
            (PassBand->NumNewRej)++;
          if (!(events[index[j]].ucFlag & PASSBAND_FLAG))
            {
            (PassBand->NumRej)++;
            events[index[j]].ucFlag |= PASSBAND_FLAG;
            }
 
          if (events[index[i]].dbFrequency - lastdeadfreq < PassBand->FreqRes)
             PassBand->DeadFreqBand += (events[index[i]].dbFrequency - lastdeadfreq);
          else
             PassBand->DeadFreqBand += (PassBand->FreqRes * 2);
          lastdeadfreq = events[index[i]].dbFrequency;
          break;                          /*  quit loop            */
          }
        }
      } /* end forward scan */
 
 
    /* Scan backward if event has not yet been found to need flagging.   */
 
    if (!(events[index[i]].ucFlag & PASSBAND_FLAG)) /* If bit-3 is not set, then do */
      for (j = i - 1;
           j >= 0 &&
           events[index[i]].dbFrequency - events[index[j]].dbFrequency <= PassBand->FreqRes;
           j--)
        {
 
        /* call function to check if beam resolution is violated */
 
        if (co_SkyAngle(events[index[j]].dbRa, events[index[j]].dbDec,
                        events[index[i]].dbRa, events[index[i]].dbDec) >
            dec_res)
          {
          count++;                 /* if beam-res violated increment count  */
          if (count > PassBand->Limit)  /* if count exceeds limit then           */
            {

            if (!(events[index[i]].ucFlag)) /* count new events rejected */
              (PassBand->NumNewRej)++;
            if (!(events[index[i]].ucFlag & PASSBAND_FLAG))
              {
              (PassBand->NumRej)++;
              events[index[i]].ucFlag |= PASSBAND_FLAG;
              }
 
            if (!(events[index[j]].ucFlag))
              (PassBand->NumNewRej)++;
            if (!(events[index[j]].ucFlag & PASSBAND_FLAG))
              {
              (PassBand->NumRej)++;
              events[index[j]].ucFlag |= PASSBAND_FLAG;
              }

            if (events[index[i]].dbFrequency - lastdeadfreq < PassBand->FreqRes)
                PassBand->DeadFreqBand += (events[index[i]].dbFrequency - lastdeadfreq);
            else
               PassBand->DeadFreqBand += (PassBand->FreqRes * 2);
            lastdeadfreq = events[index[i]].dbFrequency;
            break;                          /*  quit loop            */  
            }
          }
        } /* end backward scan */
 
    } /* end scanning events */
  } /* end function PassBandFilter */


//------------------------------------------------------------------------------
/*
 * Function to exclude events according to SpreadSpectrum condition (see rfi5.man).
 * Function uses bit 4 in the flag field to indicate events to be
 * excluded.  Function also returns the number of events rejected by
 * this algorithm that were not previously rejected by another algorithm.
 * Note that we check for signal entropy within a single spectra.
 * Is the std deviation of the expected number of events per spectra the 
 * sqrt of the expected number of events per spectra? 
 */

void SpreadSpectrumFilter(SpreadSpectrum_t     * SpreadSpectrum,
                          unsigned               events_in,
                          StdEventRec          * events)
//------------------------------------------------------------------------------
  {  
  unsigned i,j;  /* general loop control */
  unsigned n;    /* number of events having same time, one group at a time */
  double   ent;  /* entropy, as returned from SignalEntropy() */

#ifdef DEBUG_SSPECTRUM
  fprintf(stderr, "====================== debugging sspectrum ===================\n");
#endif

  for (i=0;i < events_in;i += n)

    /* check number of events at same time, if less then limit, do nothing
     * else compute entropy.  If entropy is greater then entrop_limit, then
     * flag group of events */

    if (!((n = CountThisSpectra(events,i,events_in)) <= SpreadSpectrum->EventLimit))
      {  

#ifdef DEBUG_SSPECTRUM
      fprintf(stderr,"n = %u when i = %ld and time  = %lf\n", n, i, events[i].dbJulDate);
#endif

      if((ent = SignalEntropy(events,i,n)) > SpreadSpectrum->EntropyLimit)
         {

#ifdef DEBUG_SSPECTRUM
   	 fprintf(stderr,"entropy = %10.5f\n",ent);
#endif

         /* flag events if both conditions are met */
 
         for(j=0;j<n;j++)
           {
           if (!events[i+j].ucFlag)
             (SpreadSpectrum->NumNewRej)++;        /* count those events flagged by this
                                                * algorithm only */
            (SpreadSpectrum->NumRej)++;        
            events[i+j].ucFlag |= SPREADSPECTRUM_FLAG;
            }
	 SpreadSpectrum->SpectraRej++;

          } /* end if entropy limit */

       } /* end if SpreadSpectrum limit */

   } /* end function SpreadSpectrumFilter */


// =============================================================================
// ============================ Helper Routines ================================
// =============================================================================


  char *usage =
"Usage:\ns3rfi5 tel_diam_in_feet threshold [-r4br rfi4_bin_res]\n\
[-r3br rfi3_bin_res] [-f freq_res] [-b num_beams]\n\
[-r4l rfi4_limit] [-r3l rfi3_limit] [-PassBandl PassBand_limit]\n\
[-1 do_SpreadSpectrum] [-2 do_PassBand] [-3 do_rfi3] [-4 do_rfi4]\n\
[-6 do_SlowSlew] [-7 do_FastSlew] [-r6ds SlowSlewDeltaSecs] [-r6bas SlowSlewBaSecs]\n\
[-r6ang SlowSlewDeltaAngle] [-FastSlewDps max_slew_rate]\n\
[-SpreadSpectruml SpreadSpectrum_limit] [-e entropy_limit] [-m mode]\n\
[-SpreadSpectrum SpreadSpectrum_file_name] [-p2s pass-2 sensitivity]\n\
[-rfi rfi_filename] [-p number of passes (1 or 2)]\n\
[-p1s pass-1 sensitivity] [-PassBand freq_rfi_filename]\n\
[-i intermediate_filename] [-rfi3 rfi3_filename]\n\
[-rfi4 rfi4_filename] [-SlowSlew SlowSlew_filename]\n\
[-FastSlew FastSlew_filename] [-summary sum_filename]\n\
infile_name outfile_name\n";

//------------------------------------------------------------------------
long CountObsSeconds(StdEventRec * events, unsigned num_events)
//------------------------------------------------------------------------
// Function subtracts out all intervals of time more than
// 5 seconds between events.

      {
      long i;
      double obtime;
      double secs;
      obtime = (events[num_events-1].dbJulDate - events[0].dbJulDate) * 86400.0;
      for (i=0;i<num_events-1;i++)
        {
        if ((secs = ((events[i+1].dbJulDate - events[i].dbJulDate) * 86400)) > 5.0)
          obtime -= secs;
        }
      return((long)obtime);
      }


//------------------------------------------------------------------------
long GetExpNumEvents(double NumFftBins, double NumSpectra, double thresh)
//------------------------------------------------------------------------
/* Now calculate expected number of events from observing time and thresh.
   For example, with threshold = 17, NUM_FFT_BINS = 2^22, we should see
   1 event every (1 * e^17)/2^22 integrations.  That is, every 5.76
   integrations.  5.6 * 1.7  = 9.8 sec, so we should see say, 30.6 events in
   5 minutes.
   ie, we should see 1 event every "e^thresh/num_fft_bins" integrations.
*/ 
  {
  return(unsigned)(NumFftBins * NumSpectra / exp(thresh));
  }


//------------------------------------------------------------------------
 /*
  * Function returns the entropy of a signal.  Function is passed a pointer
  * to the events array, an index into the array, and the number of events
  * that have the same detection time.  Events are sorted by bin number and entropy is
  * computed.
  */
  
double SignalEntropy(StdEventRec * events,     /* pointer to array */
                      unsigned    index,      /* index into array */
                      unsigned    n)          /* number of events read in */
//------------------------------------------------------------------------
    {
    unsigned i,j;
    double   entropy;
    double   temp;       /* used in sort swap */
    double   sort_list[MAX_HITS]; /* events are put into this list for
                                       *   sorting by bin number */
 
    entropy = 0;
 
    if (n > MAX_HITS)
      {
      fprintf(stderr,"SpreadSpectrum error: number of events recorded in one fft\n");
      fprintf(stderr,"%u exceeds the number expected.  You may need to\n", n);
      fprintf(stderr,"change value of constant MAX_HITS and recompile.\n");
      fprintf(stderr,"(MAX_HITS is in DARFIR.h)\n");
      exit(1);
      }
 
    // Put bin numbers in array for sorting.
    for (i=0;i<n;i++,index++)
      sort_list[i] = CONTIGBIN(events[index].lBinNumber, events[index].ucBid);
 
    // sort by bin number
    qsort((char *)sort_list, n, sizeof(double), FloatCompare); 
     
/*
    for (i=0;i<n;i++,index++)
{
fprintf(stderr, "sort_list_n = %lf\n", sort_list[i]);
}
*/


    for (i=1,j=0;i<n;i++,j++) /* square values */
{
//fprintf(stderr, "%lf  %lf  %lf\n", sort_list[j],  sort_list[j + 1], sort_list[j] -  sort_list[j + 1]);
      entropy += ((sort_list[j] - sort_list[j + 1]) *
                  (sort_list[j] - sort_list[j + 1]));
//fprintf(stderr, "sub_entropy = %lf\n", entropy);
//exit(1);
}
    return(entropy/n);
    }


//------------------------------------------------------------------------
 /*
  * Function is passed a pointer to the events array and an index
  * value into the array. Function returns the number of events having
  * the same detection time (same fft, starting from the event pointed to by ind
ex).
  */

long CountThisSpectra(StdEventRec * events,
                     unsigned    index,
                     unsigned    events_in)
//------------------------------------------------------------------------
    {
    unsigned count = 1;
    while (index < (events_in - 1)
            && (events[index].dbJulDate == events[index+1].dbJulDate))
       {
      index++;
      count++;
       }
    return(count);
    }


//------------------------------------------------------------------------
/* function counts the number of spectra lost due to rfix
   you pass to it a mask for the rejectiona algorghm*/
void CountRejSpectra(unsigned       * rcount,
                        StdEventRec      * events,
                        unsigned         num_events,
                        unsigned char    rfimask)
//------------------------------------------------------------------------
     {
     long i;
     double last_dbJulDate;
     last_dbJulDate=0;
     for (i=0;i< num_events;i++)
       if (events[i].ucFlag & rfimask)
            {
            if (events[i].dbJulDate != last_dbJulDate)
               {
               (*rcount)++;
               last_dbJulDate = events[i].dbJulDate;
               }
            } 
      }

//------------------------------------------------------------------------
/* Function computes reduction parameters.  This function attempts to
   set parameters so that the limit is 0 and the probability of seeing
   more than 0 different events in a bin is less than 'p'.  However, it
   may be that there are too many events and that even with a .5 Hz
   resolution (1 Hz window) it just can't reject at the given value 'p'.
   In this latter case the limit is raised.  That is, the limit is raised
   such that the probability of seeing more than 'limit' events in a
   .5 Hz window is < p. 

   This function computes the 2 floating parameters used in the pass band filter.
   These parameters are:
	freq resolution in Hz
	event limit within this freq resolution
   The 1 fixed parameter, not computed here, is:
	beam (spacial) resolution
   The pass band filter then :
	foreach event
		if there are more than "event limit" events within "freq
		resolution", but violating "beam resolution" then
			flag the base event and all events within "freq
			resolution" as RFI.

   So how are the 2 free parameters arrived at?  We start out with event limit
   at 0 and freq res at 0.5 Hz.  We want there to be no more than "p" probability
   that we will find more than event limit events with freq res.  If this is
   already the case, then we slowly widen freq res until it is as wide as possible
   with the event limit being 0.  Why?  The wider the freq res, the less work that
   the pass band filter must do.  But what if don't even start out meeting our 
   probability requirement?  We can't shrink our freq res, as the actual resolution
   of the s4 instrument is 0.6 Hz.  So, we slowly increase event limit until we
   meet our probability requirement.

   How do we calculate the probability that we will see more than event limit
   events within freq res of any given event?  This is given by the poisson
   probability distribution.  This is :  
		p(N) = (lambda^N  *  e^-lambda) / N!
   In this case, N is the limit.  With N=0, the equation is reduced to:
		p(N) = e^-lambda 

   What will happen if we have a file with pure noise?  We will reject p%
   of our data anyway.  This is because this is because our parameters are 
   set so that there is p probability that we will see more than limit events
   that violate our beam resolution.  by definition, this will happen p%
   of the time.  
   

*/


void CalcPassBandParms(long num_events, double FreqCoverage, double p, double * bin_res, unsigned long * limit)
//------------------------------------------------------------------------
 
  {
  unsigned i;      /* loop control */
  double prob;     /* used in calculation of probabilities */
  double prob_sum; /* for summing up probabilities */
  double lambda;
  double local_lambda; /* local variable for calculating probability in loop 1 */
 
  lambda = num_events / FreqCoverage;	// mean number of events per Hz
  *bin_res = 0.5;
  *limit = 0;
 /* increase resolution if possible */
 
  prob = exp(-lambda);
  while((1-prob) <= p)
    {
    (*bin_res) += 0.5;
    local_lambda = lambda * (*bin_res) * 2;
    prob = exp(-local_lambda);
#ifdef DEBUG_CALCPARMS
  fprintf(stderr, "==================== CALC PARMS  with limit = 0 ====================\n");
 fprintf(stderr, "p = %lf bin_res = %lf  lambda = %lf  local_lambda = %lf  prob = %lf limit = %ld\n", 
         p, *bin_res, lambda, local_lambda, prob, *limit);
#endif
    }
 
  /* if could not increase resolution then increase limit */
 
  if (*bin_res == 0.5)
    {
    prob_sum = prob;
    for (i=1;(1-prob_sum) > p;i++)
      {
      prob *= lambda / i;
      prob_sum += prob;
      (*limit)++;
#ifdef DEBUG_CALCPARMS
  fprintf(stderr, "==================== CALC PARMS  with limit = %ld  ====================\n", 
          *limit);
  fprintf(stderr, "p = %lf bin_res = %lf  lambda = %lf  local_lambda = %lf  prob = %lf limit = %ld\n", 
          p, *bin_res, lambda, local_lambda, prob, *limit);
#endif
      }
    }
 }   

//------------------------------------------------------------------------
/* Output report on program effectiveness:  This function reports to the
   standard output the number of events excluded, and the bin number of bins
   with one or more events excluded.  Also reported is the number of events
   excluded at each individual bin along with a line from the input file of
   one of the events. */
void Report(StdEventRec * events, unsigned long events_in, unsigned long * ndx)
//------------------------------------------------------------------------

  {

  unsigned long bins_ex;  /* Number of bins with one or more events excluded. */
  unsigned long hits;     /* For each bin having excluded events, this var-   */
                         /*  iable keeps track of the number of events       */  
                         /*  excluded at that bin.                           */
  unsigned long i;        /* Index variable.                                  */

  /* For each bin with events excluded, report to std out a line number, bin */
  /*  number, and the number of events exluded at that bin number.           */  

  /* Ouput heading. */
  fprintf(stdout,"\nline no.  events ex  bin num\n");

  bins_ex = 0;                       /* Initialize counter of bins having    */
                                     /*  events excluded.                    */    

  for (i = 0; i < events_in; i++)    /* For each event do:                   */  
    {
    if (events[ndx[i]].ucFlag)
      {                              /*  If event has been flaged (flag!=0), */
                                     /*    find all adjacent events at same  */
                                     /*    bin number that have also been    */
                                     /*    flagged and count them.           */  
 
      fprintf(stdout,"%6u  ",ndx[i]+1); /* Output line number. */
 
      hits = 1; /* Set counter to keep track of number of flagged events at  */
                /*  each bin number.                                         */
 
      while ((i < events_in - 1) &&
             (events[ndx[i]].lBinNumber == events[ndx[i+1]].lBinNumber))
        if (events[ndx[++i]].ucFlag)
          hits++;
 
      fprintf(stdout,"%6u  %10ld\n", hits, events[ndx[i]].lBinNumber);
      bins_ex++;              /* Count each unique bin with events excluded */
      }
    } /* End line-by-line report to stdout */
  (void) fprintf(stdout,"\n%u 0.6-hertz bins with one or more ",bins_ex);
  (void) fprintf(stdout,"events excluded\n");
  } /* End function 'report' */


//------------------------------------------------------------------------
/* Output full report on program efectivness:  This function outputs data
   on the performance of each rejection algorithm separatly.  NOTE: This
   function cannot be used unless all rejection algorithms were run in
   statistics mode.  The following data is output to the stdout file:
   For each bin having one or more events excluded, a line number of one
   event in the group, the number of events rejected by each algorithm listed
   separately, the number of events rejected by the combined algorithms, and
   the bin number itself. After the above report is written, the number of
   bins having one or more events excluded, is reported for each individual
   algorithm, and the program as a whole. */
 
 
void StatisticsReport(StdEventRec     * events,
                       unsigned long    events_in,
                       unsigned long  * ndx)
//------------------------------------------------------------------------
 
  {
 
  unsigned long bins_ex;  /* Number of bins with one or more events excluded. */
  unsigned long hits;     /* For each bin having excluded events, this var-   */
                         /*  iable keeps track of the number of events       */  
                         /*  excluded at that bin.                           */
 
  unsigned long SpreadSpectrum_ex;  /* Number of events excluded by SpreadSpectrum.               */  
  unsigned long SpreadSpectrum_binx;/* Number of bins with events flagged by SpreadSpectrum.      */  
  unsigned long SpreadSpectrum_hits;/* Keeps track of the number of events excluded at  */
                         /*  a particular bin. (analogous to 'hits' above).  */
 
  unsigned long PassBand_ex;  /* Number of events excluded by PassBand.               */  
  unsigned long PassBand_binx;/* Number of bins with events flagged by PassBand.      */  
  unsigned long PassBand_hits;/* Keeps track of the number of events excluded at  */
                         /*  a particular bin. (analogous to 'hits' above.   */
 
  unsigned long SlowSlew_ex;  /* Number of events excluded by SlowSlew.               */  
  unsigned long SlowSlew_binx;/* Number of bins with events flagged by SlowSlew.      */  
  unsigned long SlowSlew_hits;/* Keeps track of the number of events excluded at  */
                         /*  a particular bin. (analogous to 'hits' above.)  */
  unsigned long FastSlew_ex;  /* Number of events excluded by FastSlew.               */  
  unsigned long FastSlew_binx;/* Number of bins with events flagged by FastSlew.      */  
  unsigned long FastSlew_hits;/* Keeps track of the number of events excluded at  */
                         /*  a particular bin. (analogous to 'hits' above.)  */
 
  unsigned long i;        /* Index variable.                                  */
 
  /* For each bin with events excluded, report to std out a line number, bin */
  /*  number, and the number of events exluded at that bin number.           */  
 
  /* Initialize counters. */
 
  bins_ex   = 0;
  SpreadSpectrum_ex   = 0;
  SpreadSpectrum_binx = 0;
  PassBand_ex   = 0;
  PassBand_binx = 0;
  SlowSlew_ex   = 0;
  SlowSlew_binx = 0;
  FastSlew_ex   = 0;
  FastSlew_binx = 0;
 
  /* Output file heading. */
  fprintf(stdout,
        "\nline no.   SSpec   PBand   SSlew   FSlew all-algs   bin num\n");
          
 
  for (i = 0; i < events_in; i++)    /* For each event do:                   */  
    {
    if (events[ndx[i]].ucFlag)
      {                              /*  If event has been flaged (flag!=0), */
                                     /*    find all adjacent events at same  */
                                     /*    bin number that have also been    */
                                     /*    flagged and count them.           */  
 
/* Set counter to keep track of number of flagged events at each bin number. */
/* That is, events flagged by any algorithm.                                 */
 
      hits = 1;
 
/* Initialize counters for the number of events flagged at each bin  by      */  
/* individual algorithms.                                                    */
      
      SpreadSpectrum_hits = 0;
      PassBand_hits = 0;
      SlowSlew_hits = 0;
      FastSlew_hits = 0;
 
/* Keep count of number of events flaged by each rejection algorithm. */
/* Also, if an event was flagged by one of the rejection algorithms   */
/* start counter of the number of events rejected at this bin by this */
/* algorithm.                                                         */
 
      if (events[ndx[i]].ucFlag & PASSBAND_FLAG)
        {
        PassBand_ex++;
        PassBand_hits = 1;
        } 
          
      if (events[ndx[i]].ucFlag & SPREADSPECTRUM_FLAG)
        {
        SpreadSpectrum_ex++;
        SpreadSpectrum_hits = 1;
        } 
          
      if (events[ndx[i]].ucFlag & SLOWSLEW_FLAG)
        {
        SlowSlew_ex++;
        SlowSlew_hits = 1;
        } 
          
      if (events[ndx[i]].ucFlag & FASTSLEW_FLAG)
        {
        FastSlew_ex++;
        FastSlew_hits = 1;
        } 
          
      fprintf(stdout,"%6u  ",ndx[i]+1); /* Output line number. */
 
 
      while ((i < events_in - 1) &&
             (events[ndx[i]].lBinNumber == events[ndx[i+1]].lBinNumber))
        if (events[ndx[++i]].ucFlag)         /* If event is flagged, then count */
          {                                /*  it as an event flagged at this */
          hits++;                          /*  bin number.                    */ 
 
          /* Also update counters of individual algorithm performance.        */ 
 
          if (events[ndx[i]].ucFlag & PASSBAND_FLAG)
            {
            PassBand_ex++;
            PassBand_hits++;
            }
          if (events[ndx[i]].ucFlag & SPREADSPECTRUM_FLAG)
            {
            SpreadSpectrum_ex++;
            SpreadSpectrum_hits++;
            }
 
          if (events[ndx[i]].ucFlag & SLOWSLEW_FLAG)
            {
            SlowSlew_ex++;
            SlowSlew_hits++;
            }
 
          if (events[ndx[i]].ucFlag & FASTSLEW_FLAG)
            {
            FastSlew_ex++;
            FastSlew_hits++;
            }
          }
 
      fprintf(stdout,"%6u  %6u  %6u  %6u  %6d  %10ld\n",
          SpreadSpectrum_hits,PassBand_hits,SlowSlew_hits,
          FastSlew_hits,hits, events[ndx[i]].lBinNumber);
 
      /* Count number of bins with one or more events excluded. */
 
      bins_ex++;      /* All algorithms. */
      if (SpreadSpectrum_hits)  /* SpreadSpectrum.           */
        SpreadSpectrum_binx++;  /*                 */
      if (PassBand_hits)  /* PassBand.           */
        PassBand_binx++;  /*                 */
      if (SlowSlew_hits)  /* SlowSlew.           */
        SlowSlew_binx++;
      if (FastSlew_hits)  /* FastSlew.           */
        FastSlew_binx++;
          
      }  
    } /* End line-by-line report to stdout */
      
  /* Output status report on the number of bins that had one or more events */
  /* excluded.                                                              */
 
  (void) fprintf(stdout,"\nNumber of 0.6-hertz bins with one ");
  (void) fprintf(stdout,"or more events excluded:\n");
  (void) fprintf(stdout,"All algorithms --> %u\n",bins_ex);
  (void) fprintf(stdout,"SSpec          --> %u\n",SpreadSpectrum_binx);
  (void) fprintf(stdout,"PBand          --> %u\n",PassBand_binx);
  (void) fprintf(stdout,"SlowSlew       --> %u\n",SlowSlew_binx);
  (void) fprintf(stdout,"FastSlew       --> %u\n",FastSlew_binx);
  (void) fprintf(stdout,"\nNumber of events excluded:\n");
  (void) fprintf(stdout,"SSpec          --> %u\n",SpreadSpectrum_ex);
  (void) fprintf(stdout,"PBand          --> %u\n",PassBand_ex);
  (void) fprintf(stdout,"SlowSlew       --> %u\n",SlowSlew_ex);
  (void) fprintf(stdout,"FastSlew       --> %u\n",FastSlew_ex);
 
  } /* End function 'statistics_report' */
 


//------------------------------------------------------------------------
void GetOps(int               		argc,
            char           	** 	argv,
            SlowSlew_t      	*	SlowSlew,
            FastSlew_t      	*	FastSlew,
	    PassBand_t      	*	PassBand,
  	    SpreadSpectrum_t  	*	SpreadSpectrum,
            unsigned        	* 	tel_diameter,
            long            	* 	mode,   
            char           	** 	fout_i_name,
            char           	** 	fout_rfi_name,
            char           	** 	fout_summary_name,
            double          	* 	p1s,    
            double          	* 	p2s,  
            long            	* 	two_pass, 
            long            	* 	auto_set,
            BOOLEAN         	* 	UpdateDB) 
//------------------------------------------------------------------------
  {

  char TestIn[256];   // used to test input strings before assigning addr

  long i;             // to loop through cpmmand line operands 

  if (argc < 1)
     {
     fprintf(stderr,"%s\n",usage);
     exit(1);
     }
    
  if (sscanf(argv[1],"%u",tel_diameter) != 1)
     {
     perror("error reading first arg (telescope diameter in integer feet)");
     fprintf(stderr,"%s\n",usage);
     exit(1);
     }

  for (i = 2; i < argc - 2;i++)
    {
#ifdef DEBUG_GETOPS
  fprintf(stderr,"%d of %d : %s\n", i, argc, argv[i]);
#endif

    if (strcmp(argv[i], "-SlowSlewDs") == 0)  /* delta seconds (SlowSlew)*/
      {
      if (sscanf(argv[++i], "%d", &SlowSlew->DeltaSecs) != 1)
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-SlowSlewBas") == 0)       /* before/after seconds (SlowSlew)*/
      {
      if (sscanf(argv[++i], "%d", &SlowSlew->BaSecs) != 1)
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-SlowSlewLimit") == 0)       /* delta angle (SlowSlew)*/
      {
      if (sscanf(argv[++i], "%lf", &SlowSlew->Limit) != 1)
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-FastSlewDps") == 0)   /* degrees/second (FastSlew)*/
      {
      if (sscanf(argv[++i], "%lf", &FastSlew->Dps) != 1)
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-e") == 0)       /* signal entropy (SpreadSpectrum) */
      {
      if (sscanf(argv[++i], "%lf", &SpreadSpectrum->EntropyLimit) != 1)
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-f") == 0)       /* (PassBand) frequency res.  */
      {
      if (sscanf(argv[++i], "%f", &PassBand->FreqRes) == 1)
        *auto_set = 0;
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-b") == 0)       /* (PassBand) frequency res.  */
      {
      if (sscanf(argv[++i], "%f", &PassBand->NumBeams) != 1)
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-SpreadSpectruml") == 0)    /* (SpreadSpectrum) "limit" */
      {
      if (sscanf(argv[++i], "%lf", &SpreadSpectrum->SigmaLimit) != 1)
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-PassBandl") == 0)   /* (PassBand) "limit" for         */
      {
      if (sscanf(argv[++i], "%u", &PassBand->Limit) == 1)
        *auto_set = 0;
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-do_SpreadSpectrum") == 0)  /* Set run flag SpreadSpectrum*/
      {
      SpreadSpectrum->DoIt = TRUE;
      }
    else if (strcmp(argv[i], "-do_PassBand") == 0)     /* Set run flag for PassBand      */
      {
      PassBand->DoIt = TRUE;
      }
    else if (strcmp(argv[i], "-do_SlowSlew") == 0)     /* Set run flag for SlowSlew      */
      {
      SlowSlew->DoIt = TRUE;
      }
    else if (strcmp(argv[i], "-do_FastSlew") == 0)     /* Set run flag for FastSlew      */
      {
      FastSlew->DoIt = TRUE;
      }
    else if (strcmp(argv[i], "-db") == 0)            /* update summary db      */
      {
      *UpdateDB = TRUE;
      }
    else if (strcmp(argv[i], "-m") == 0)     /* Program run mode           */
      {
      if (sscanf(argv[++i], "%d", mode) != 1)
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      if (*mode != 1 && *mode != 0)            /* Default to fast mode if    */
        {                                    /*  illeagal mode enetered.   */
        fprintf(stderr,"Illegal mode entered --> 0 = statistics, 1 = fast\n");
        perror("default to fast mode\n");
        *mode = 1;
        }
      }
    else if (strcmp(argv[i], "-i") == 0)     /* file for intermediate results */
      {                                      
      if (sscanf(argv[++i], "%s", TestIn) == 1)
        *fout_i_name = argv[i];
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-rfi") == 0)   /* rfi file     */
      {                                     
      if (sscanf(argv[++i], "%s", TestIn) == 1)
        *fout_rfi_name =  argv[i];
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-PassBand") == 0)  
      {                                      
      if (sscanf(argv[++i], "%s", TestIn) == 1)
        PassBand->foutName = argv[i];
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-SpreadSpectrum") == 0)   
      {                                      
      if (sscanf(argv[++i], "%s", TestIn  ) == 1)
        SpreadSpectrum->foutName = argv[i];
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-SlowSlew") == 0)   
      {                                      
      if (sscanf(argv[++i], "%s", TestIn) == 1)
        SlowSlew->foutName = argv[i];
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-FastSlew") == 0)   
      {                                      
      if (sscanf(argv[++i], "%s", TestIn) == 1)
        FastSlew->foutName = argv[i];
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-summary") == 0) 
      {                                      
      if (sscanf(argv[++i], "%s", TestIn) == 1)
        *fout_summary_name =  argv[i];
      else
        {
        perror("Error in command line\n");
        fprintf(stderr,"%s\n",usage);
        exit(1);
        }
      }
    else if (strcmp(argv[i], "-p1s") == 0)
       {
       if (sscanf(argv[++i],"%lf",p1s) != 1)
         {
         perror("Error in command line");
         fprintf(stderr,"%s\n",usage);
         exit(1);
         }
       }
    else if (strcmp(argv[i], "-p2s") == 0)
       {
       if (sscanf(argv[++i],"%lf",p2s) != 1)
         {
         perror("Error in command line");
         fprintf(stderr,"%s\n",usage);
         exit(1);
         }
       }
    else if (strcmp(argv[i], "-p") == 0)
       {
       if (sscanf(argv[++i],"%d",two_pass) != 1)
         {
         perror("Error in command line");
         fprintf(stderr,"%s\n",usage);
         exit(1);
         }
       else
         *two_pass = (*two_pass == 2);
       }
    else
       {
       perror("Unknown flag\n");
       fprintf(stderr,"%s\n",usage);
       exit(1);
       }
   }
  }


//------------------------------------------------------------------------
BOOLEAN OpenFiles(
      FILE ** fin_p,        char * fin_name,
      FILE ** fout_p,       char * fout_name,
      FILE ** fout_i,       char * fout_i_name,         BOOLEAN * output_i,
      FILE ** fout_rfi,     char * fout_rfi_name,       BOOLEAN * output_rfi,
      FILE ** fout_SlowSlew,    char * fout_SlowSlew_name,      BOOLEAN * output_SlowSlew,
      FILE ** fout_FastSlew,    char * fout_FastSlew_name,      BOOLEAN * output_FastSlew,
      FILE ** fout_PassBand,    char * fout_PassBand_name,      BOOLEAN * output_PassBand,
      FILE ** fout_SpreadSpectrum,    char * fout_SpreadSpectrum_name,      BOOLEAN * output_SpreadSpectrum,
      FILE ** fout_summary, char * fout_summary_name,   BOOLEAN * output_summary,
      FILE ** fout_comb,    char * fout_comb_name)
//------------------------------------------------------------------------

  {
  long NumFiles = 10;
  long i;

  char    * Name[NumFiles];
  char    * Type[NumFiles];
  FILE   ** File[NumFiles];
  BOOLEAN * Flag[NumFiles];

  BOOLEAN DummyFlag;
  BOOLEAN RetVal = TRUE;    // Be optimistic;

  // set 'em up...

  Name[0]  = fin_name;
  Name[1]  = fout_name; 
  Name[2]  = fout_i_name;
  Name[3]  = fout_rfi_name; 
  Name[4]  = fout_SpreadSpectrum_name;
  Name[5]  = fout_PassBand_name;
  Name[6]  = fout_SlowSlew_name;
  Name[7]  = fout_FastSlew_name;
  Name[8] = fout_summary_name;
  Name[9] = fout_comb_name;
     
  File[0]  = fin_p;
  File[1]  = fout_p;
  File[2]  = fout_i;
  File[3]  = fout_rfi;
  File[4]  = fout_SpreadSpectrum;
  File[5]  = fout_PassBand;
  File[6]  = fout_SlowSlew;
  File[7]  = fout_FastSlew;
  File[8] = fout_summary;
  File[9] = fout_comb;

  Type[0]  = "r";
  Type[1]  = "w";
  Type[2]  = "w";
  Type[3]  = "w";
  Type[4]  = "w";
  Type[5]  = "w";
  Type[6]  = "w";
  Type[7]  = "w";
  Type[8] = "a";
  Type[9] = "w";

  Flag[0]  = &DummyFlag;
  Flag[1]  = &DummyFlag;
  Flag[2]  = output_i;
  Flag[3]  = output_rfi;
  Flag[4]  = output_SpreadSpectrum;
  Flag[5]  = output_PassBand;
  Flag[6]  = output_SlowSlew;
  Flag[7]  = output_FastSlew;
  Flag[8] = output_summary;
  Flag[9] = &DummyFlag;

  // ... and knock 'em down

  for(i = 0; i < NumFiles; i++)
    if(Name[i] != NULL)
      if((*File[i] = fopen(Name[i], Type[i])) != NULL)
        *(Flag[i]) = TRUE;
      else
        RetVal = FALSE;  // bummer

  return(RetVal);
  }

//------------------------------------------------------------------------
/*
 * Function to write unflagged events to output file.  Also write
 * to file of rfi and to file of frequency correlated rfi if specified.
 */

void OutputEvents(SlowSlew_t           SlowSlew,
                   FastSlew_t           FastSlew,
		   PassBand_t	    	PassBand,
                   SpreadSpectrum_t	SpreadSpectrum,
                   FILE               * fin_p,
                   FILE               * fout_p,
                   FILE               * fout_rfi,
                   FILE               * fout_comb,
                   BOOLEAN              output_rfi,
                   BOOLEAN              output_combination,
                   char                 combination,
                   StdEventRec        * events,
                   unsigned long        events_in,
                   unsigned long      * events_out)
//------------------------------------------------------------------------
  { 

  long  i;              /* Index variable.         */
  long  out_cnt;        /* Counts events output.   */
     
  for (i = 0,out_cnt = 0; i < events_in; i++)    
    {                                           
    if (!events[i].ucFlag)                     
      {                                       
      io_WriteStdEvent(fout_p, &events[i]);			// write good event
      out_cnt++;                            
      }                                    
    else                                  
      {                                  
      if (output_rfi)
	io_WriteStdEvent(fout_rfi, &events[i]);			// write bad event
      if (PassBand.OutputRFI && (events[i].ucFlag & PASSBAND_FLAG))
	io_WriteStdEvent(PassBand.fout, &events[i]);	// write pass band bad event
      if (SpreadSpectrum.OutputRFI && (events[i].ucFlag & SPREADSPECTRUM_FLAG))
	io_WriteStdEvent(SpreadSpectrum.fout, &events[i]);	// write pass band bad event
      if (SlowSlew.OutputRFI && (events[i].ucFlag & SLOWSLEW_FLAG))
	io_WriteStdEvent(SlowSlew.fout, &events[i]);	// write slow slew bad event
      if (FastSlew.OutputRFI && (events[i].ucFlag & FASTSLEW_FLAG))
	io_WriteStdEvent(FastSlew.fout, &events[i]);	// write fast slew bad event
      if (output_combination && ((events[i].ucFlag & combination) == combination))             
	io_WriteStdEvent(fout_comb, &events[i]);		// write combination bad event
 
      }  
    } /* End for loop */
    *events_out = out_cnt;
  } 
     


// ============================ quick sort compare routines ================================

//------------------------------------------------------------------------
static  int FreqCompare(const void *i, const void *j)
//------------------------------------------------------------------------
{
  if (events[*((int *)i)].dbFrequency > events[*((int *)j)].dbFrequency)
     return (1);
  if (events[*((int *)i)].dbFrequency < events[*((int *)j)].dbFrequency)
     return (-1);
   return (0);
}
 
//------------------------------------------------------------------------
static  int BinCompare(const void *i, const void *j)
//------------------------------------------------------------------------
{
  long bini, binj;

  bini = CONTIGBIN(events[*((int *)i)].lBinNumber, events[*((int *)i)].ucBid);
  binj = CONTIGBIN(events[*((int *)j)].lBinNumber, events[*((int *)j)].ucBid);

  if (bini > binj)
     return (1);
  if (bini < binj)
     return (-1);
   return (0); 
} 
 
//------------------------------------------------------------------------ 
static  int FloatCompare(const void *i, const void *j) 
//------------------------------------------------------------------------ 
{ 
  if (*((double *)i) > *((double *)j)) 
     return (1); 
  if (*((double *)i) < *((double *)j)) 
     return (-1); 
   return (0); 
} 


//------------------------------------------------------------------------
void output_cntrl_structs(	SlowSlew_t          *  SlowSlew,
                                FastSlew_t          *  FastSlew,	
				PassBand_t          *  PassBand,
  				SpreadSpectrum_t    *  SpreadSpectrum)
//------------------------------------------------------------------------

{

	if(SlowSlew) {
		fprintf(stderr, "SlowSlew.DoIt = %ld SlowSlew.OutputRFI = %ld SlowSlew.DeltaSecs = %ld SlowSlew.BaSecs = %ld SlowSlew.Limit = %lf SlowSlew.NumRej = %ld SlowSlew.NumNewRej = %ld SlowSlew.SpectraLost = %ld\n\n",
  		SlowSlew->DoIt,
  		SlowSlew->OutputRFI,
  		SlowSlew->DeltaSecs,
  		SlowSlew->BaSecs,
  		SlowSlew->Limit,
  		SlowSlew->NumRej,
  		SlowSlew->NumNewRej,
  		SlowSlew->SpectraLost);
		if(SlowSlew->foutName) {
			fprintf(stderr, "%s\n\n", SlowSlew->foutName);
		}
	}

	if(FastSlew) {
		fprintf(stderr, "FastSlew.DoIt = %ld FastSlew.OutputRFI = %ld FastSlew.Dps = %lf FastSlew.NumRej = %ld FastSlew.NumNewRej = %ld FastSlew.SpectraLost = %ld\n\n",
  		FastSlew->DoIt,
  		FastSlew->OutputRFI,
  		FastSlew->Dps,
  		FastSlew->NumRej,
  		FastSlew->NumNewRej,
  		FastSlew->SpectraLost);
		if(FastSlew->foutName) {
			fprintf(stderr, "%s\n\n", FastSlew->foutName);
		}
	}

	if(PassBand) {
		fprintf(stderr, "PassBand.DoIt = %d PassBand.OutputRFI = %ld PassBand.FreqRes = %lf PassBand.NumBeams = %lf PassBand.Limit = %ld PassBand.NumRej = %ld PassBand.NumNewRej = %ld PassBand.DeadFreqBand = %lf\n\n", 
  		PassBand->DoIt,
  		PassBand->OutputRFI,
  		PassBand->FreqRes,
  		PassBand->NumBeams,
  		PassBand->Limit ,
  		PassBand->NumRej,
  		PassBand->NumNewRej,
  		PassBand->DeadFreqBand);
		if(PassBand->foutName) {
			fprintf(stderr, "%s\n\n", PassBand->foutName);
		}
	}

	if(SpreadSpectrum) {
		fprintf(stderr, "SpreadSpectrum.DoIt = %d SpreadSpectrum.OutputRFI = %d SpreadSpectrum.EntropyLimit = %lf SpreadSpectrum.SigmaLimit = %lf SpreadSpectrum.NumRej = %ld SpreadSpectrum.NumNewRej = %ld\n\n",
  		SpreadSpectrum->DoIt,
  		SpreadSpectrum->OutputRFI,
  		SpreadSpectrum->EntropyLimit,
  		SpreadSpectrum->SigmaLimit,
  		SpreadSpectrum->NumRej,
  		SpreadSpectrum->NumNewRej);
		if(SpreadSpectrum->foutName) {
			fprintf(stderr, "%s\n\n", SpreadSpectrum->foutName);
		}
	}

}


//------------------------------------------------------------------------
long CountFftBoards(long events_in) {
//------------------------------------------------------------------------
  // s4 is a multiboard system, thus the true number of FFT bins depends
  // on the number of active FFT boards in the file.  We assume that this
  // is constant throughout the file.  Here we count the FFT boards.

  long i;
  long NumFftBoards = 0;
  long BoardCnt[NUMBOARDS];

  for(i = 0; i < NUMBOARDS; i++)
    BoardCnt[i] = 0;
  for(i = 0; i < events_in; i++)
    BoardCnt[events[i].ucBid] = 1;
  for(i = 0; i < NUMBOARDS; i++)
    NumFftBoards += BoardCnt[i];

  return(NumFftBoards);
}

