/*
 * Source Code Name  : s4rfistats.cc
 * Programmer        : Jeff Cobb
 *                   : Project SERENDIP, University of California, Berkeley CA
 * Compiler          : GNU gcc
 *
 * Given a date range in "dbase form" (eg 01ap98 05ma99), this program
 * will compile RFI filtering statistics for that range.  The data
 * source is the s4 summary database.
 *
 * If only one date is given, stats for that one date will be output.
 */
#include <stdio.h>
#include <fcntl.h>
#include "SIREN.h"
#include "s4database.h"
#include "s4summary.h"

#define VERSION 1.0

const char * usage = "usage : s4rfistats startdate <enddate>";

main(int argc, char ** argv)
  {
  long i, Num;
  int DB;
  char Date1[DBASE_FILENAME_LEN];
  char Date2[DBASE_FILENAME_LEN];
  char TestDate[DBASE_FILENAME_LEN];

  long 	NumEvents			= 0,
	NumSpectra			= 0,
	NumEventsOut			= 0,
	NumEventsExpected		= 0,
	NumPassBandRej			= 0,
	NumSpreadSpectrumRej		= 0,
	NumFastSlewSkipped		= 0,
	NumFastSlewRej			= 0,
	NumFastSlewSpectraRej		= 0,
	NumSlowSlewSkipped		= 0,
	NumSlowSlewRej			= 0,
	NumSlowSlewSpectraRej		= 0;

  DbaseSumRcd_t ThisRcd;

  switch(argc) {
	case 3 :				// file range
		strcpy(Date1, argv[1]);
        	strcpy(Date2, argv[2]);
		break;
	case 2 :				// 1 file
		strcpy(Date1, argv[1]);
        	strcpy(Date2, argv[1]);
		break;
	case 1 :				// all files
		;				// (not implemented)
	default :
		;
  }

  DB = sum_GetDataBase(O_RDONLY);
  Num = sum_GetNumInDateRange(Date1, Date2, DB);
  strcpy(TestDate, Date1);
  while(sum_GetNextRcd(TestDate, &ThisRcd, FALSE, DB, O_RDONLY) && 
        strcmp(TestDate, Date2) != 0) {
	NumSpectra			+=	ThisRcd.ulNumSpectra;
	NumEvents                       += 	ThisRcd.ulRfiFilterEventsIn;
        NumEventsOut                    +=  	ThisRcd.ulRfiFilterEventsOut;
        NumEventsExpected               += 	ThisRcd.ulRfiFilterExpectedOut;
        NumPassBandRej                  +=	ThisRcd.ulPassBand_NumRej;
        NumSpreadSpectrumRej            +=    	ThisRcd.ulSpreadSpectrum_NumRej;
        NumFastSlewRej                  += 	ThisRcd.ulFastSlew_NumRej;
        NumFastSlewSpectraRej           += 	ThisRcd.ulFastSlew_SpectraLost;
        NumSlowSlewRej                  += 	ThisRcd.ulSlowSlew_NumRej;
        NumSlowSlewSpectraRej           += 	ThisRcd.ulSlowSlew_SpectraLost;
        //fprintf(stdout, "%s\n", ThisRcd.sDbaseName);
    }
  if(strcmp(TestDate, Date2) == 0) {
	NumSpectra                      +=      ThisRcd.ulNumSpectra;
        NumEvents                       +=      ThisRcd.ulRfiFilterEventsIn;
        NumEventsOut                    +=      ThisRcd.ulRfiFilterEventsOut;
        NumEventsExpected               +=      ThisRcd.ulRfiFilterExpectedOut;
        NumPassBandRej                  +=      ThisRcd.ulPassBand_NumRej;
        NumSpreadSpectrumRej            +=      ThisRcd.ulSpreadSpectrum_NumRej;
        NumFastSlewRej                  +=      ThisRcd.ulFastSlew_NumRej;
        NumFastSlewSpectraRej           +=      ThisRcd.ulFastSlew_SpectraLost;
        NumSlowSlewRej                  +=      ThisRcd.ulSlowSlew_NumRej;
        NumSlowSlewSpectraRej           +=      ThisRcd.ulSlowSlew_SpectraLost;  
        //fprintf(stdout, "%s\n", ThisRcd.sDbaseName);   // catch the last one
  }

  sum_GetNextRcd(TestDate, &ThisRcd, TRUE, DB, O_RDONLY);   // last call to clean up

  fprintf(stdout, "Total Events In              : %ld\n", NumEvents);
  fprintf(stdout, "Total Spectra In             : %ld\n", NumSpectra);
  fprintf(stdout, "Total Events Out             : %ld\n", NumEventsOut);
  fprintf(stdout, "Slow Slew Reject Rate        : %lf\n", (double)NumSlowSlewRej/NumEvents);
  fprintf(stdout, "Fast Slew Reject Rate        : %lf\n", (double)NumFastSlewRej/NumEvents);
  fprintf(stdout, "Pass Band Reject Rate        : %lf\n", (double)NumPassBandRej/NumEvents);
  fprintf(stdout, "Spread Spectrum Reject Rate  : %lf\n", (double)NumSpreadSpectrumRej/NumEvents);
  fprintf(stdout, "Combined Reject Rate         : %lf\n", (double)(NumEvents-NumEventsOut)/NumEvents);
  fprintf(stdout, "\nFirst : %s   Last : %s   Num in range : %ld\n", Date1, Date2, Num);

}
