/******************************************************************************
source code name   : s4spctcnt.cc
programmer         : Charles Donnelly
compiler           : cc on Sun Workstation
                   :
purpose            : Program counts the number of spectra in a dbase file


                   : Program does a line-by-line file read.  All contiguous
                   : lines having the same dbUTime are considered to be from the
                   : same spectra.

*******************************************************************************
modifications:
	 6-13-88 - created
	 1-9-91  - modified
	 6-1-95  - scott converted to SERENDIP IV
*******************************************************************************
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "SIREN.h"
#include "s4io.h"



int main(int argc, char *argv[])

  {
  FILE          *fin_p;               /* input file                           */
  unsigned int  num_spectra;          /* number of spectra in file            */
  unsigned long events_in;            /* number of events in input file       */
  double        temp_ut;              /*  used for reading in dbUTime         */ 

	 
  StdEventRec event;                  /* standard event record */


            /* process command line, should be file name only */

  if (argc != 2)
	  {
	  perror("usage: s4spctcnt in_file");
	  exit(1);
	  }


           /* open input file for read */

  if ((fin_p = fopen(argv[argc -1], "r")) == NULL)
	  {
	  perror("Error: can't open input file");
	  exit(1);
	  }
          
         /* count number of events in file  */

  events_in = (io_CountRecords(fin_p,STDEVENT_REC)); 
 
  if (events_in < 0)
          {
          perror("Trouble counting events in file");
          exit(1);
          }
 
  if (events_in == 0)
          {
          perror("Empty File");
          exit(1);
          }

  
   fprintf(stdout,"Now counting spectra\n\n");


         /* get first event and read first local sidereal time in file */

   num_spectra = 1;           /*   begin counting the number of spectra */


        /* load first event and retain spectra for comparison  */

   io_GetNextRecord(fin_p,&event,STDEVENT_REC);

   temp_ut = event.dbUTime;


      /* loop through dbas file.  As dbUTime changes in comparison to
         temp_ut, reassign dbUTime to temp_ut (for comparison to next
         event's dbUTime), and increment spectra counter.
     */

 
   while (io_GetNextRecord(fin_p,&event,STDEVENT_REC))
      {	
       if (temp_ut != event.dbUTime)
          {
          temp_ut = event.dbUTime;
          num_spectra++;
          }
      }
      

   if (!feof(fin_p))
          {
          perror("Bad file read.");
          exit(1);
	  }


  fclose(fin_p);


  fprintf(stdout,"Number of events input = %u\n",events_in);
  fprintf(stdout,"Number of spectra = %u\n",num_spectra);

  } /* end main() */

