 /***************************************************************************
 * Programmer        : Jeff Cobb
 * Source code name  : s4timerang.cc
 * Compiler          : cc on Sun Workstation
 * Purpose           : Reads a file in the dbas format and outputs
 *                     the time and date of the first and last record
 *                     as well as the filename to stdout.  This can be
 *                     run in a looping script on a list of dbas files
 *                     to produce a list of file time ranges.  This is
 *                     an aid to finding "files-of-origin" for intereting
 *                     events.
 *
 *                     see also: s4findofile.cc
 **************************************************************************
 *  modifications:
 *    11/30/93   chuck  fixed locos from float to double
 *    3/28/95    scott  convert to ANSI and SERENDIP IV
 **************************************************************************
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "SIREN.h"
#include "s4io.h"



int main(int argc, char *argv [])

  {

  StdEventRec event1;       /* first record in file */
  StdEventRec event2;       /* last record in file  */

  FILE * fin_p;


		 /* Open input file */

	 if ((fin_p = fopen(argv[1], "r")) == NULL)
		 {
		 fprintf(stderr, "ERROR: Can't open input file - %s.\n", argv[1]);
		 exit(1);
		 }

		  /* Get first valid event */

	 if ((io_GetNextRecord(fin_p,&event1,STDEVENT_REC)) != 1)
		 {
		 fprintf(stderr, "ERROR: unable to read FIRST event in %s\n", argv[1]);
		 exit(1);
		 }

		 /*  Get last valid event */

	if ((io_GetLastRecord(fin_p,&event2,STDEVENT_REC)) != 1)
		 {
		 fprintf(stderr,"ERROR: unable to read LAST event in %s\n", argv[1]);
		 exit(1);
		 }

        fprintf(stdout, "%s %s %s %s %s\n",
			event1.stUTime,event1.stDate, event2.stUTime, event2.stDate,
			 argv[1]);

	return(0);
 }


